/*
   GNUstep ProjectCenter - http://www.gnustep.org

   Copyright (C) 2000-2002 Free Software Foundation

   Author: Philippe C.D. Robert <probert@siggraph.org>

   This file is part of GNUstep.

   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.

   $Id: PCProjectBuilder.h,v 1.21 2003/06/20 14:52:19 stoyan Exp $
*/

#ifndef _PCPROJECTBUILDER_H
#define _PCPROJECTBUILDER_H

#include <AppKit/AppKit.h>

#ifndef GNUSTEP_BASE_VERSION
@protocol ProjectComponent;
#else
#include <ProjectCenter/ProjectComponent.h>
#endif

@class PCProject;
@class PCButton;

@interface PCProjectBuilder : NSObject <ProjectComponent>
{
  NSPanel         *buildPanel;
  NSBox           *componentView;
  PCButton        *buildButton;
  PCButton        *cleanButton;
  PCButton        *installButton;
  PCButton        *optionsButton;
  NSPopUpButton   *popup;
  
  NSPanel         *optionsPanel;
  NSTextField     *buildTargetHostField;
  NSTextField     *buildTargetArgsField;

  NSTextView      *logOutput;
  NSTextView      *errorOutput;

  NSString        *makePath;

  id              buildStatusField;
  id              targetField;

  PCProject       *currentProject;
  NSDictionary    *currentOptions;

  NSString        *statusString;
  NSMutableString *buildTarget;
  NSMutableArray  *buildArgs;
  SEL             postProcess;
  NSTask          *makeTask;

  NSFileHandle    *readHandle;
  NSFileHandle    *errorReadHandle;
}

- (id) initWithProject: (PCProject *)aProject;
- (void) dealloc;

- (NSPanel *) createBuildPanel;
- (NSPanel *) buildPanel;
- (NSView *) componentView;
- (void) setTooltips;

- (void) build: (id)sender;
- (void) popupChanged: (id)sender;

- (void) logStdOut: (NSNotification *)aNotif;
- (void) logErrOut: (NSNotification *)aNotif;

- (void) buildDidTerminate: (NSNotification *)aNotif;
//- (void) buildDidTerminate: (int)status;

- (void) copyPackageTo: (NSString *)path;

@end

@interface PCProjectBuilder (BuildLogging)

- (void)logString:(NSString *)string error:(BOOL)yn;
- (void)logString:(NSString *)string error:(BOOL)yn newLine:(BOOL)newLine;
- (void)logData:(NSData *)data error:(BOOL)yn;

@end

@interface PCProjectBuilder (BuildThread)

- (void) make: (NSDictionary *)data;

@end

#endif
