;   NSZone.scm - Tests of GNUstep libraries from Guile
;   Copyright (C) 1998 Free SoftwareFoundation, Inc.
;
;   Written by:  Richard Frith-Macdonald <richard@brainstorm.co.uk>
;   Date: July 1998
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

;
;	Tests for zone functions
;

(define zones '())

(greg-testcase "NSDefaultZone() returns something" #t
  (lambda ()
    (define aZone (NSDefaultMallocZone))
    (if (equal? aZone voidp-nil)
      #f
      (begin
	(set! zones (cons aZone zones))
	#t
      )
    )
  )
)

(greg-testcase "NSCreateZone() works for a normal zone" #t
  (lambda ()
    (define aZone (NSCreateZone 1024 1024 1))
    (set! zones (cons aZone zones))
    (not (equal? aZone voidp-nil))
  )
)

(greg-testcase "NSCreateZone() works for an unfreeable zone" #t
  (lambda ()
    (define aZone (NSCreateZone 1024 1024 0))
    (set! zones (cons aZone zones))
    (not (equal? aZone voidp-nil))
  )
)

(do
  ((list zones (cdr list)) (counter 1 (+ counter 1)))
  ((eq? list '()))
  (let*
    (
      (aZone (car list))
      (vp #f)
      (ovp #f)
      (ptr #f)
      (zoneName gstep-nil)
    )
    (begin
      (greg-testcase "NSSetZoneName() accepts an NSString" #t
	(lambda ()
	  (NSSetZoneName aZone ($$ "My Zone"))
	  (gc)
	  #t
	)
      )

      (greg-testcase "NSZoneName() returns the string previously set" #t
	(lambda ()
	  (set! zoneName (NSZoneName aZone))
	  (gstep-bool->bool ([] zoneName isEqual: ($$ "My Zone")))
	)
      )

      (greg-testcase "NSZoneCalloc() works for 17 items of size 12" #t
	(lambda ()
	  (set! vp (NSZoneCalloc aZone 17 12))
	  (= (voidp-length vp) (* 17 12))
	)
      )

      (greg-testcase "NSZoneFree() releases memory from NSZoneCalloc()" #t
	(lambda ()
	  (NSZoneFree aZone vp)
	  (gc)
	  #t
	)
      )

      (greg-testcase "NSZoneMalloc() works for item of size 2000" #t
	(lambda ()
	  (set! vp (NSZoneMalloc aZone 2000))
	  (= (voidp-length vp) 2000)
	)
      )

      (greg-testcase "NSZoneFree() releases memory from NSZoneMalloc()" #t
	(lambda ()
	  (NSZoneFree aZone vp)
	  (gc)
	  #t
	)
      )

      (greg-testcase "NSZoneRealloc() works to grow 1000 to 2000" #t
	(lambda ()
	  (set! ovp (NSZoneMalloc aZone 1000))
	  (set! vp (NSZoneRealloc aZone ovp 2000))
	  (= (voidp-length vp) 2000)
	)
      )

      (greg-testcase "NSZoneRealloc() works to shrink 2000 to 1000" #t
	(lambda ()
	  (set! vp (NSZoneRealloc aZone vp 1000))
	  (= (voidp-length vp) 1000)
	)
      )

      (greg-testcase "NSZoneFromPointer() returns zone memory came from" #t
	(lambda ()
	  (set! ptr (NSZoneFromPointer vp))
	  (equal? ptr aZone)
	)
      )

      (greg-testcase "NSZoneFree() releases memory held after realloc" #t
	(lambda ()
	  (NSZoneFree aZone vp)
	  (gc)
	  #t
	)
      )

      (if (equal? aZone (NSDefaultMallocZone))
	#t
	(greg-testcase "NSRecycleZone() seems to operate" #t
	  (lambda ()
	    ; Leave an NSString object in the zone - it should get deallocated
	    ; without crashing.
	    ([] "NSString" allocWithZone: aZone)
	    (NSRecycleZone aZone)
	    (set! aZone #f)
	    (gc)
	    #t
	  )
	)
      )
    )
  )
)

