
;   general.scm - Tests of calling Objective-C methods from Guile
;   Copyright (C) 2001 Free SoftwareFoundation, Inc.
;
;   Written by: Richard frith-Macdonald <rfm@gnu.org>
;   Date: Aug 2001
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

(begin

  (define path gstep-nil)
  (define data gstep-nil)
  (define parser gstep-nil)
  (define doc gstep-nil)

  (greg-testcase "Can parse one char base64 mime1.dat" #t
    (lambda ()
      (set! path ($$ (string-append (greg-directory) "/mime1.dat")))
      (set! data ([] "NSData" dataWithContentsOfFile: path))
      (set! doc ([] "GSMimeParser" documentFromData: data))
      ; (display ([] doc content))
      (and (gstep-bool ([] parser parse: data))
	(gstep-bool ([] ([] ([] ([] doc content) objectAtIndex: 0) content) isEqual: ($$ "a")))
      )
    )
  )
  (greg-testcase "Can parse two char base64 mime2.dat" #t
    (lambda ()
      (set! path ($$ (string-append (greg-directory) "/mime2.dat")))
      (set! data ([] "NSData" dataWithContentsOfFile: path))
      (set! doc ([] "GSMimeParser" documentFromData: data))
      (and
	(gstep-bool ([] ([] doc content) isEqual: ($$ "aa")))
      )
    )
  )
  (greg-testcase "Can parse three char base64 mime3.dat" #t
    (lambda ()
      (set! path ($$ (string-append (greg-directory) "/mime3.dat")))
      (set! data ([] "NSData" dataWithContentsOfFile: path))
      (set! doc ([] "GSMimeParser" documentFromData: data))
      (and
	(gstep-bool ([] ([] doc content) isEqual: ($$ "aaa")))
      )
    )
  )

  (greg-testcase "Can parse multipart text mime4.dat" #t
    (lambda ()
      (set! path ($$ (string-append (greg-directory) "/mime4.dat")))
      (set! data ([] "NSData" dataWithContentsOfFile: path))
      (set! doc ([] "GSMimeParser" documentFromData: data))
      ;(display ([] doc content))
      (and
	(gstep-bool ([] ([] ([] ([] doc content) objectAtIndex: 0) content) isEqual: ($$ "hello\n")))
	(gstep-bool ([] ([] ([] ([] doc content) objectAtIndex: 1) content) isEqual: ($$ "there\n")))
      )
    )
  )

  (greg-testcase "Can extract content name from mime4.dat" #t
    (lambda ()
      (set! path ($$ (string-append (greg-directory) "/mime4.dat")))
      (set! data ([] "NSData" dataWithContentsOfFile: path))
      (set! doc ([] "GSMimeParser" documentFromData: data))
      (and
	(gstep-bool ([] ([] ([] ([] doc content) objectAtIndex: 0) contentName) isEqual: ($$ "a.a")))
      )
    )
  )

  (greg-testcase "Can extract content file name from mime4.dat" #t
    (lambda ()
      (set! path ($$ (string-append (greg-directory) "/mime4.dat")))
      (set! data ([] "NSData" dataWithContentsOfFile: path))
      (set! doc ([] "GSMimeParser" documentFromData: data))
      (and
	(gstep-bool ([] ([] ([] ([] doc content) objectAtIndex: 0) contentFile) isEqual: ($$ "a.a")))
      )
    )
  )

  (greg-testcase "Can extract content type from mime4.dat" #t
    (lambda ()
      (set! path ($$ (string-append (greg-directory) "/mime4.dat")))
      (set! data ([] "NSData" dataWithContentsOfFile: path))
      (set! doc ([] "GSMimeParser" documentFromData: data))
      (and
	(gstep-bool ([] ([] ([] ([] doc content) objectAtIndex: 0) contentType) isEqual: ($$ "text")))
      )
    )
  )

  (greg-testcase "Can extract content sub type from mime4.dat" #t
    (lambda ()
      (set! path ($$ (string-append (greg-directory) "/mime4.dat")))
      (set! data ([] "NSData" dataWithContentsOfFile: path))
      (set! doc ([] "GSMimeParser" documentFromData: data))
      (and
	(gstep-bool ([] ([] ([] ([] doc content) objectAtIndex: 0) contentSubtype) isEqual: ($$ "plain")))
      )
    )
  )

  (greg-testcase "Can parse http document mime5.dat" #t
    (lambda ()
      (set! path ($$ (string-append (greg-directory) "/mime5.dat")))
      (set! data ([] "NSData" dataWithContentsOfFile: path))
      (set! doc ([] "GSMimeParser" documentFromData: data))
      (and
	(gstep-bool ([] ([] doc contentSubtype) isEqual: ($$ "xml")))
      )
    )
  )

  (greg-testcase "Can parse multipart mixed mime6.dat" #t
    (lambda ()
      (set! path ($$ (string-append (greg-directory) "/mime6.dat")))
      (set! data ([] "NSData" dataWithContentsOfFile: path))
      (set! doc ([] "GSMimeParser" documentFromData: data))
      (equal? ([] ([] doc content) count) 3)
    )
  )

  (greg-testcase "mime6.dat binary data part matches mime7.dat" #t
    (lambda ()
      (set! path ($$ (string-append (greg-directory) "/mime7.dat")))
      (set! data ([] "NSData" dataWithContentsOfFile: path))
      (gstep-bool ([] ([] ([] ([] doc content) objectAtIndex: 1) content)
        isEqual: data)
      )
    )
  )

)


