
;   test01.scm - Tests of calling Objective-C methods from Guile
;   Copyright (C) 2003 Free SoftwareFoundation, Inc.
;
;   Written by: Richard frith-Macdonald <rfm@gnu.org>
;   Date: May 2003
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

(begin

  (define enc ([] "GSMimeDocument" encodingFromCharset: ($$ "utf-8")))
  (define data gstep-nil)
  (define parser gstep-nil)
  (define doc gstep-nil)

  (greg-testcase "Can parse one header" #t
    (lambda ()
      (set! parser ([] "GSMimeParser" mimeParser))
      (set! data ([] ($$ "Content-type: application/xxx\r\n")
        dataUsingEncoding: enc))
      (set! doc ([] parser mimeDocument))
      (and
	(gstep-bool ([] parser parse: data))
        (gstep-bool ([] parser isInHeaders))
	(not (eq? gstep-nil doc))
      )
    )
  )

  (greg-testcase "First header not complete until next starts" #t
    (lambda ()
      (eq? ([] doc contentType) gstep-nil)
    )
  )

  (greg-testcase "Adding partial header is ok" #t
    (lambda ()
      (set! data ([] ($$ "Content-id: <")
        dataUsingEncoding: enc))
      (and
	(gstep-bool ([] parser parse: data))
        (gstep-bool ([] parser isInHeaders))
      )
    )
  )

  (greg-testcase "Parsed first header as expected" #t
    (lambda ()
      (and
        (gstep-bool ([] ($$ "application") isEqual: ([] doc contentType)))
        (gstep-bool ([] ($$ "xxx") isEqual: ([] doc contentSubtype)))
      )
    )
  )

  (greg-testcase "Completing partial header is ok" #t
    (lambda ()
      (set! data ([] ($$ "hello>\r\n")
        dataUsingEncoding: enc))
      (and
	(gstep-bool ([] parser parse: data))
        (gstep-bool ([] parser isInHeaders))
      )
    )
  )

  (greg-testcase "Partial header not complete until next starts" #t
    (lambda ()
      (eq? ([] doc contentID) gstep-nil)
    )
  )

  (greg-testcase "Folded header is ok" #t
    (lambda ()
      (set! data ([] ($$ "Folded\r\n")
        dataUsingEncoding: enc))
      (gstep-bool ([] parser parse: data))
      (set! data ([] ($$ " : testing\r\n")
        dataUsingEncoding: enc))
      (and
	(gstep-bool ([] parser parse: data))
        (gstep-bool ([] parser isInHeaders))
      )
    )
  )

  (greg-testcase "Parsed partial header as expected" #t
    (lambda ()
      (gstep-bool ([] ($$ "<hello>") isEqual: ([] doc contentID)))
    )
  )

  (greg-testcase "Folded header not complete until next starts" #t
    (lambda ()
      (eq? ([] doc headerNamed: ($$ "Folded")) gstep-nil)
    )
  )

  (greg-testcase "partial end-of-line is ok" #t
    (lambda ()
      (set! data ([] ($$ "\r")
        dataUsingEncoding: enc))
      (and
	(gstep-bool ([] parser parse: data))
        (gstep-bool ([] parser isInHeaders))
      )
    )
  )

  (greg-testcase "Parsed folded header as expected" #t
    (lambda ()
      (gstep-bool ([] ([] ([] doc headerNamed: ($$ "Folded")) value) isEqual: ($$ "testing")))
    )
  )

  (greg-testcase "completing end-of-line is ok" #t
    (lambda ()
      (set! data ([] ($$ "\n")
        dataUsingEncoding: enc))
      (and
	(gstep-bool ([] parser parse: data))
        (not (gstep-bool ([] parser isInHeaders)))
      )
    )
  )

  (greg-testcase "charset is inferred" #t
    (lambda ()
      (set! doc ([] "GSMimeDocument" documentWithContent: 
	([] ($$ "\"\\UFE66???\"") propertyList) type:
	($$ "text/plain") name: gstep-nil))
      ([] doc rawMimeData)
      (gstep-bool ([] ([] ([] doc headerNamed: ($$ "content-type")) parameterForKey: ($$ "charset")) isEqual: ($$ "utf-8")))
    )
  )
)


