;;
;;   Copyright (C) 2002 Free SoftwareFoundation, Inc.
;;
;;   Written by: Nicola Pero <nicola@brainstorm.co.uk>
;;   Date: March 2002
;;
;;   This file is part of the GNUstep project.
;;
;;   This library is free software; you can redistribute it and/or
;;   modify it under the terms of the GNU Library General Public
;;   License as published by the Free Software Foundation; either
;;   version 2 of the License, or (at your option) any later version.
;;   
;;   This library is distributed in the hope that it will be useful,
;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;   Library General Public License for more details.
;;
;;   You should have received a copy of the GNU Library General Public
;;   License along with this library; if not, write to the Free
;;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

(begin

  (define document gstep-nil)
  (define namespace gstep-nil)
  (define node gstep-nil)

  ;; GSXMLDocument basics
  (test-for-class "GSXMLDocument +alloc returns a GSXMLDocument" 
		  "GSXMLDocument" ([] "GSXMLDocument" alloc))

  (test-for-class "GSXMLDocument +documentWithVersion returns a GSXMLDocument" 
		  "GSXMLDocument" 
		  ([] "GSXMLDocument" documentWithVersion: ($$ "1.0")))

  ;; GSXMLNode basics
  (test-for-class "GSXMLNode +alloc returns a GSXMLNode"
		  "GSXMLNode" ([] "GSXMLNode" alloc))

  (greg-testcase "GSXMLNode +new returns nil" #t
    (lambda () (equal? ([] "GSXMLNode" new) gstep-nil)))

  ;; GSXMLNamespace basics
  (test-for-class "GSXMLNamespace +alloc returns a GSXMLNamespace"
		  "GSXMLNamespace" ([] "GSXMLNamespace" alloc))

  (greg-testcase "GSXMLNamespace +new returns nil" #t
    (lambda () (equal? ([] "GSXMLNamespace" new) gstep-nil)))


  (set! document ([] "GSXMLDocument" documentWithVersion: ($$ "1.0")))

  (greg-testcase "Can create a document node" #t
    (lambda ()
      (set! node ([] document makeNodeWithNamespace: gstep-nil
	name: ($$ "nicola") content: gstep-nil))
      (not (equal? node gstep-nil))
    )
  )

  ;; Now some basic tasks
  (greg-testcase "Can set a document node as root node" #t
    (lambda ()
      ([] document setRoot: node)
      (gstep-bool ([] ([] document root) isEqual: node))
    )
  )

  (greg-testcase 
   "Can set a document node (with children) as root node" #t
   (lambda ()
     (let*
       (
	  (node ([] document makeNodeWithNamespace: gstep-nil
		             name: ($$ "nicola")
		             content: gstep-nil)))
       ([] node makeChildWithNamespace: gstep-nil
	   name: ($$ "paragraph")
	   content: ($$ "Hi this is some text"))
       ([] node makeChildWithNamespace: gstep-nil
	   name: ($$ "paragraph")
	   content: ($$ "Hi this is some more text"))
       ([] node makeChildWithNamespace: gstep-nil
	   name: ($$ "paragraph")
	   content: ($$ "Hi this is even more text"))
       ([] document setRoot: node)
       (gstep-bool ([] ([] document root) isEqual: node))
       )
     )
   )

  (greg-testcase "Can create a node namespace" #t
    (lambda ()
      (set! namespace ([] node makeNamespaceHref: ($$ "http://www.gnustep.org")
		      prefix: ($$ "gnustep")))
      (not (equal? namespace gstep-nil))
    )
  )

  (greg-testcase 
   "Can create a node with a namespace" #t
   (lambda ()
     (let*
	 ( (node ([] document makeNodeWithNamespace: namespace
	    name: ($$ "nicola") content: gstep-nil)))
       (gstep-bool ([] ([] node namespace) isEqual: namespace))
       )
     )
   )

  (greg-testcase 
   "Can create a node with same namespace as another node" #t
   (lambda ()
     (let*
	 ( (node ([] document makeNodeWithNamespace: namespace
	    name: ($$ "another") content: gstep-nil)))
       (gstep-bool ([] ([] node namespace) isEqual: namespace))
       )
     )
   )

  (greg-testcase 
   "A namespace remembers its prefix" #t
   (lambda ()
     (let* ()
       (gstep-bool ([] ([] namespace prefix) isEqual: ($$ "gnustep")))
       )
     )
   )

  )
