
;   create.scm - Creation tests for NSArchiver/NSUnarchiver
;   Copyright (C) 1998 Free SoftwareFoundation, Inc.
;
;   Written by:  Adam Fedor <fedor@gnu.org>
;   Date: Jan 2000
;   (Adapted from tests written by Richard Frith-Macdonald)
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

;
;	Do everything inside a 'begin' procedure so that any variables
;	defined will have scope limited to this file.
;
(begin

  ;
  ;	Define variables that we are going to use to contain values between
  ;	individual tests - they need to be defined here so they are in the
  ;	scope enclosing all the tests.
  ;
  (define data1 gstep-nil)

  ;
  ;	Simple test for the '+new' method - we invoke the method, then we
  ;	check that the result is not a nil object, it is of the correct
  ;	class, and that it is a dictionary with zero keys.
  ;
  (greg-testcase "+new creates an empty archiver" #t
    (lambda ()
      (define obj ([] "NSArchiver" new))
      (and
	(not (equal? obj gstep-nil))
	(gstep-bool ([] obj isKindOfClass: ([] "NSArchiver" class)))
	(not (equal? gstep-nil ([] obj archiverData)))
      )
    )
  )

  (greg-testcase "-initForWritingWithMutableData: seems ok" #t
    (lambda ()
      (define obj ([] "NSArchiver" alloc))
      (set! data1 ([] "NSMutableData" dataWithLength: 0))
      (set! obj ([] obj initForWritingWithMutableData: data1))
      (and
	(not (equal? obj gstep-nil))
	(gstep-bool ([] obj isKindOfClass: ([] "NSArchiver" class)))
	(equal? data1 ([] obj archiverData)))
      )
    )
  )


  ;
  ;	Tests on NSUnarchiver
  ;
  (test-exception "Creating an NSUnarchivever with nil data"
	NSInvalidArgumentException
    (lambda ()
      (define obj ([] "NSUnarchiver" alloc))
      (set! obj ([] obj initForReadingWithData: gstep-nil))
      #f
    )
  )

  ;(test-exception "Creating an NSUnarchiver with invalid data"
  ;	NSRangeException
  ;  (lambda ()
  ;    (define obj ([] "NSUnarchiver" alloc))
  ;    (set! data1 ([] "NSData" data))
  ;    (set! obj ([] obj initForReadingWithData: data1))
  ;    #f
  ;  )
  ;)






