
;   test00.scm - General tests for NSAttributedString
;   Copyright (C) 1998 Free SoftwareFoundation, Inc.
;
;   Written by:  Adam Fedor <fedor@gnu.org>
;   Date: Oct 2000
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

;
;	Do everything inside a 'begin' procedure so that any variables
;	defined will have scope limited to this file.
;
(begin

  ;
  ;	Define variables that we are going to use to contain values between
  ;	individual tests - they need to be defined here so they are in the
  ;	scope enclosing all the tests.
  ;
  (define val1 gstep-nil)
  (define key1 gstep-nil)
  (define val3 gstep-nil)
  (define str1 gstep-nil)
  (define dict1 gstep-nil)

  (define r (NSMakeRange 0 6))
  (define r1 0)
  (define r2 0)
  (define astr1 gstep-nil)
  (define astr2 gstep-nil)

  (define rngptr1 voidp-nil)
  

  ;
  ;	Now initialise those variables inside a testcase so that, in the
  ;	event of a major error in intialisation, we get an unresolved test.
  ;
  (greg-testcase "We can build basic strings and objects for tests" #t
    (lambda ()
      ;
      ;	Three strings to use as attributes
      ;
      (set! val1 ($$ "Helvetica 12-point"))
      (set! key1 ($$ "NSFontAttributeName"))
      (set! str1 ($$ "Attributed string test."))

      (set! rngptr1 (list->voidp (list r1 r2) "i"))

      (set! dict1 ([] "NSDictionary" dictionaryWithObject: val1 forKey: key1))
      #t
    )
  )

  ;
  ; Create an NSAttributedString
  ;
  (greg-testcase "-initWithString:attributes: works" #t
    (lambda ()
      (set! astr1 ([] ([] "NSAttributedString" alloc) initWithString: str1 
		   attributes: dict1))
      (and
	(not (equal? astr1 gstep-nil))
	(gstep-bool ([] astr1 isKindOfClass: ([] "NSAttributedString" class)))
	(gstep-bool ([] ([] astr1 string) isEqual: str1))
      )
    )
  )

  ;
  ; Minimal tests
  ;
  (greg-testcase "-attributesAtIndex:effectiveRange: works" #t
    (lambda ()
      (define obj ([] astr1 attributesAtIndex: 0 effectiveRange: rngptr1))
      (and
 	(not (equal? obj gstep-nil))
	(gstep-bool ([] obj isKindOfClass: ([] "NSDictionary" class)))
	(equal? 1 ([] obj count))
	(not (equal? 0 (NSRange-length r)))
      )
    )
  )

  (greg-testcase "-attribute:atIndex:effectiveRange: works" #t
    (lambda ()
      (define obj ([] astr1 attribute: key1 atIndex: 0 effectiveRange: rngptr1))
      (and
 	(not (equal? obj gstep-nil))
	(gstep-bool ([] obj isEqual: val1))
	(not (equal? 0 (NSRange-length r)))
      )
    )
  )

  (greg-testcase "-attributedSubstringFromRange works" #t
    (lambda ()
      (define obj ([] astr1 attributedSubstringFromRange: r))
      (and
 	(not (equal? obj gstep-nil))
	(gstep-bool ([] obj isKindOfClass: ([] "NSAttributedString" class)))
      )
    )
  )

  (greg-testcase "extract and compare using -isEqualToAttributedString works" #t
    (lambda ()
      (set! r (NSMakeRange 0 ([] astr1 length)))
      (set! astr2 ([] astr1 attributedSubstringFromRange: r))
      (and
 	(not (equal? astr2 gstep-nil))
	(gstep-bool ([] astr2 isEqualToAttributedString: astr1))
      )
    )
  )


)
