;   create.scm - Creation tests for NSBunclde
;   Copyright (C) 1998, 2000, 2002 Free SoftwareFoundation, Inc.
;
;   Written by:  Adam Fedor <fedor@gnu.org>
;   Date: Apr 2000
;   Rewritten: Nicola Pero <nicola@brainstorm.co.uk>, February 2002
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

;;
;;	Do everything inside a 'begin' procedure so that any variables
;;	defined will have scope limited to this file.
;;
(begin

  ;;
  ;;	Define variables that we are going to use to contain values between
  ;;	individual tests - they need to be defined here so they are in the
  ;;	scope enclosing all the tests.
  ;;
  (define path gstep-nil)
  (define gnustep-bundle gstep-nil)
  (define bundle gstep-nil)

  ;;
  ;;	Now initialize those variables inside a testcase so that, in the
  ;;	event of a major error in intialisation, we get an unresolved test.
  ;;
  (greg-testcase 
   "We can build basic objects for NSBundle creation tests" #t
   (lambda ()
     (set! path 
	   ([] 
	    ([] ([] "NSFileManager" defaultManager) currentDirectoryPath)
	    stringByAppendingPathComponent: ($$ "/Resources")))
     #t
     )
   )

  ;;;
  ;;; Testing creating a bundle using +mainBundle.
  ;;;
  
  ;; Unfortunately, the tool which is running these tests doesn't have 
  ;; a +mainBundle, so +mainBundle correctly returns nil.

  (test-for-nil "+mainBundle returns nil if the tool has no bundle"
		([] "NSBundle" mainBundle))

  ;; TODO: We should test more of +mainBundle.  We need either to add
  ;; a mainBundle to the tool running those tests, or perhaps create a
  ;; custom tool in a subdir, and check that +mainBundle works inside
  ;; that tool.

  ;;;
  ;;; Testing creating a bundle from path.
  ;;;

  ;; We want to keep the result of ([] "NSBundle" bundleWithPath: path)
  ;; in a variable
  (greg-testcase 
   "+bundleWithPath does not crash" #t
   (lambda () (set! bundle ([] "NSBundle" bundleWithPath: path)) #t)
   )
  
  (test-for-class "+bundleWithPath returns a bundle" "NSBundle" bundle)
  (test-for-string "a bundle has a path" ([] bundle bundlePath))
  (test-for-equal-objects "bundlePath returns the correct path"
			  ([] bundle bundlePath)
			  path)
  (test-for-class "a bundle has an infoDictionary" "NSDictionary"
		  ([] bundle infoDictionary))

  (test-for-nil
   "+bundleWithPath returns nil for a non-existing path"
   ([] "NSBundle" bundleWithPath: 
    ([] path stringByAppendingPathComponent: ($$ "/Nicola"))))


  ;; Do this test after creating a bundle and retaining it (bundle is
  ;; retained)
  (test-for-array "+allBundles returns an array" ([] "NSBundle" allBundles))

  ;;;
  ;;; Testing creating the +gnustepBundle.
  ;;;

  (greg-testcase 
   "+gnustepBundle does not crash" #t
   (lambda () (set! gnustep-bundle ([] "NSBundle" gnustepBundle)) #t)
   )

  (test-for-class "+gnustepBundle returns a bundle" "NSBundle" gnustep-bundle)
  (test-for-string "+gnustepBundle has a path" ([] gnustep-bundle bundlePath))

  ;; The gnustepBundle doesn't have an infoDictionary, but the method
  ;; should return an empty dictionary (*not* nil).
  (test-for-class "+gnustepBundle has a infoDictionary" "NSDictionary"
		   ([] gnustep-bundle infoDictionary))
)

