;   general.scm - Check general bundle methods
;   Copyright (C) 1998, 2000, 2002 Free SoftwareFoundation, Inc.
;
;   Written by:  Adam Fedor <fedor@gnu.org>
;   Date: May 2000
;   Rewritten by: Nicola Pero, February 2002
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

;
;	Do everything inside a 'begin' procedure so that any variables
;	defined will have scope limited to this file.
;
(begin

  ;
  ;	Define variables that we are going to use to contain values between
  ;	individual tests - they need to be defined here so they are in the
  ;	scope enclosing all the tests.
  ;
  (define main-bundle gstep-nil)
  (define gnustep-bundle gstep-nil)

  ;;
  ;; Create the mainBundle and test major functions
  ;;

  ;; Unfortunately, the mainBundle is nil for the tool running these tests,
  ;; so we can't test much :-)  we'd like to test the following features
  ;; of the mainBundle - 

  ;; "+mainBundle makes a bundle for us" 
  ;; "-principalClass returns a nil class for tests main" 
  ;; "-classNamed returns the correct class" 
  ;; "-resourcePath returns a path" 
  ;; "we can set and get bundle version" 
  ;; "-infoDictionary returns a dictionary" 
  ;; "-load behaves properly on the main bundle" 


  ;;
  ;; Test NSBundle gnustep methods
  ;;

  (test-for-string "+_gnustep_target_cpu returns a string" 
		   ([] "NSBundle" _gnustep_target_cpu))

  (test-for-string "+_gnustep_target_os returns a string" 
		   ([] "NSBundle" _gnustep_target_os))

  (test-for-string "+_gnustep_target_dir returns a string" 
		   ([] "NSBundle" _gnustep_target_dir))

  (test-for-string "+_library_combo returns a string" 
		   ([] "NSBundle" _library_combo))

  ;;
  ;; Create the GNUstep bundle and test major functions
  ;;
  (greg-testcase "+gnustepBundle does not crash" #t
		 (lambda () 
		   (set! gnustep-bundle ([] "NSBundle" gnustepBundle)) #t)
		 )

  (test-for-class "+gnustepBundle makes a bundle for us" "NSBundle" 
		  gnustep-bundle)
  
  (test-for-nil "-principalClass returns nil for the +gnustepBundle" 
		([] gnustep-bundle principalClass))

  ;; FIXME - is this behaviour correct ? - I don't think so
  ;; ... gnustepBundle didn't load *any* code
  (test-for-equal-objects "-classNamed returns the correct class"
			  ([] gnustep-bundle classNamed: ($$ "NSArray"))
			  ([] "NSArray" class))
  
  (test-for-string "-resourcePath returns a string"
		   ([] gnustep-bundle resourcePath))

  (greg-testcase "we can set and get gnustep bundle version" #t
    (lambda ()
      ([] gnustep-bundle setBundleVersion: 42)
      (eq? 42 ([] gnustep-bundle bundleVersion))
    )
  )

  (test-for-class "-infoDictionary returns a dictionary" "NSDictionary"
		  ([] gnustep-bundle infoDictionary))

  (greg-testcase "-load behaves properly on the gnustep bundle" #t
    (lambda () (gstep-bool ([] gnustep-bundle load))))

)
