;;   load.scm - Loading tests for NSBunclde
;;   Copyright (C) 2002 Free SoftwareFoundation, Inc.
;;
;;   Written by:  Nicola Pero <nicola@brainstorm.co.uk>
;;   Date: February 2002
;;
;;   This file is part of the GNUstep project.
;;
;;   This library is free software; you can redistribute it and/or
;;   modify it under the terms of the GNU Library General Public
;;   License as published by the Free Software Foundation; either
;;   version 2 of the License, or (at your option) any later version.
;;   
;;   This library is distributed in the hope that it will be useful,
;;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;   Library General Public License for more details.
;;
;;   You should have received a copy of the GNU Library General Public
;;   License along with this library; if not, write to the Free
;;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

(begin

  (define path gstep-nil)
  (define bundle gstep-nil)

  ;; Create the bundle we want to test
  (greg-testcase 
   "+bundleWithPath does not crash" #t
   (lambda () 
     (set! path ([]
		 ([] 
		  ([] ([] "NSFileManager" defaultManager) currentDirectoryPath)
		  stringByAppendingPathComponent: ($$ "Resources")
		  )
		 stringByAppendingPathComponent: ($$ "NSBundleTest.bundle")
		 )
	   )
	   
     (set! bundle ([] "NSBundle" bundleWithPath: path))
     #t
     )
   )
  
  (test-for-class "+bundleWithPath returns a bundle" "NSBundle" bundle)
  (test-for-string "the bundle has a path" ([] bundle bundlePath))
  (test-for-equal-objects "bundlePath returns the correct path"
			  ([] bundle bundlePath) path)
  (test-for-string "a bundle has a resource path" ([] bundle resourcePath))
  (test-for-class "a bundle has an infoDictionary" "NSDictionary"
		  ([] bundle infoDictionary))
  
  (greg-testcase "-load returns YES" #t
		 (lambda () (gstep-bool ([] bundle load))))
  
  (test-for-non-nil "-load actually loaded the class" 
		    (gstep-lookup-class "NSBundleTestPrincipalClass"))

  (test-for-non-nil "we can instantiate a loaded class" 
		    ([] "NSBundleTestPrincipalClass" new))

  (test-for-non-nil "-principalClass is not nil" ([] bundle principalClass))
  
  (test-for-equal-objects "-principalClass works"
			  ([] ([] bundle principalClass) description)
			  ($$ "NSBundleTestPrincipalClass"))
  
  (test-for-non-nil "we can instantiate -principalClass"
		    ([] ([] bundle principalClass) new))
  
  (test-for-equal-objects "-classNamed: works"
			  ([] bundle principalClass)
			  ([] bundle classNamed: 
			   ($$ "NSBundleTestPrincipalClass")))
  
  (test-for-equal-objects "+bundleForClass: works"
			  ([] "NSBundle" bundleForClass: 
			   ([] bundle principalClass))
			  bundle)
  
  (greg-testcase "we can set and get bundle version" #t
		 (lambda ()
		   ([] bundle setBundleVersion: 42)
		   (eq? 42 ([] bundle bundleVersion))
		   )
		 )

  (greg-testcase "+allBundles contains a bundle after we loaded it" #t
		 (lambda () 
		   (gstep-bool ([] ([] "NSBundle" allBundles) 
				containsObject: bundle))))
  )

