;   resources.scm - Check resource path methods for NSBundle
;   Copyright (C) 1998, 2000, 2002 Free SoftwareFoundation, Inc.
;
;   Written by:  Adam Fedor <fedor@gnu.org>
;   Date: May 2000
;
;   Modified a lot: Nicola Pero, February 2002
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

;
;	Do everything inside a 'begin' procedure so that any variables
;	defined will have scope limited to this file.
;
(begin

  ;
  ;	Define variables that we are going to use to contain values between
  ;	individual tests - they need to be defined here so they are in the
  ;	scope enclosing all the tests.
  ;
  (define bundle gstep-nil)
  (define gnustep-bundle gstep-nil)

  ;
  ;	Now initialize those variables inside a testcase so that, in the
  ;	event of a major error in intialisation, we get an unresolved test.
  ;
  (greg-testcase 
   "We can build basic objects for NSBundle resource tests" #t
   (lambda ()
     (set! bundle ([] "NSBundle" bundleWithPath: 
		   ([] ([] "NSFileManager" defaultManager) 
		    currentDirectoryPath)))
     (set! gnustep-bundle ([] "NSBundle" bundleForLibrary: ($$ "gnustep-base")))
      #t
    )
  )

  ;;
  ;; Creating the basic NSBundle bundle we want to use for testing
  ;;
  (test-for-class "bundleWithPath gave us a bundle" "NSBundle" bundle)

  ;;
  ;; Class resource path methods for main bundle
  ;;
  (test-for-array "-pathsForResourcesOfType:inDirectory: gives an array"
		  ([] bundle pathsForResourcesOfType: ($$ "scm")
		   inDirectory: gstep-nil))

  (test-for-nil "-pathForResource:ofType:inDirectory: works with nil args" 
		([] bundle pathForResource: ($$ "hiwelf0-2") ofType: gstep-nil 
		 inDirectory: gstep-nil)
		)

  (test-for-nil "-pathForResource:ofType: works with nil args"
		([] bundle pathForResource: ($$ "hiwelf0-2") ofType: gstep-nil)
		)

  (test-for-equal-objects "-resourcePath returns the correct path"
			  ([] ([] bundle bundlePath) 
			   stringByAppendingPathComponent: ($$ "Resources"))
			  ([] bundle resourcePath))

  ;;
  ;; Class resource path methods for gnustep bundle
  ;;
  (test-for-string "+pathForResource:ofType:inDirectory: works"
		   ([] "NSBundle" pathForResource: ($$ "whitespaceCharSet")
		    ofType: ($$ "dat")
		    inDirectory: ([] ([] gnustep-bundle bundlePath)
				  stringByAppendingPathComponent: 
				  ($$ "NSCharacterSets"))
		    )
		   )

  (test-for-string "+pathForResource:ofType:inDirectory:withVersion: works"
		   ([] "NSBundle" pathForResource: ($$ "whitespaceCharSet")
		    ofType: ($$ "dat")
		    inDirectory: ([] ([] gnustep-bundle bundlePath)
				  stringByAppendingPathComponent: 
				  ($$ "NSCharacterSets"))
		    withVersion: 0
    		   )
   )

  (test-for-array "-pathsForResourcesOfType:inDirectory: gives an array"
		  ([] gnustep-bundle pathsForResourcesOfType: ($$ "dat")
		   inDirectory: ($$ "NSCharacterSets"))
		  )

  (test-for-string "-pathForResource:ofType:inDirectory: finds a file"
		   ([] gnustep-bundle pathForResource: ($$ "whitespaceCharSet")
		    ofType: ($$ "dat")
		    inDirectory: ($$ "NSCharacterSets"))
		   )

  (test-for-nil "-pathForResource:ofType:inDirectory: doesn't find non-existing files"
		([] gnustep-bundle pathForResource: ($$ "whitespaceCharSet")
		 ofType: ($$ "8nicola8")
		 inDirectory: ($$ "NSCharacterSets"))
		)

  (test-for-nil "-pathForResource:ofType:inDirectory: doesn't find files in non-existing dirs"
		([] gnustep-bundle pathForResource: ($$ "whitespaceCharSet")
		 ofType: gstep-nil
		 inDirectory: ($$ "NSCharacterSets_dummy"))
		)

  (test-for-string "-pathForResource:ofType:inDirectory: with nil type finds a file"
		   ([] gnustep-bundle pathForResource: ($$ "whitespaceCharSet")
		    ofType: gstep-nil
		    inDirectory: ($$ "NSCharacterSets"))
		   )

  (test-for-nil "-pathForResource:ofType:inDirectory: with nil type doesn't find non-existing files"
		   ([] gnustep-bundle pathForResource: ($$ "whhwefnicolafe")
		    ofType: gstep-nil
		    inDirectory: ($$ "NSCharacterSets"))
		   )

  (test-for-string "-pathForResource:ofType: finds a file"
		   ([] gnustep-bundle pathForResource: ($$ "NSTimeZones")
		    ofType: gstep-nil)
		   )

  )


