
;   test00.scm - Miscellaneous tests for NSCalandarDate
;   Copyright (C) 1998 Free SoftwareFoundation, Inc.
;
;   Written by:  Adam Fedor <fedor@gnu.org>
;   Date: Oct 2000
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

;
;	Do everything inside a 'begin' procedure so that any variables
;	defined will have scope limited to this file.
;
(begin

  ;
  ;	Define variables that we are going to use to contain values between
  ;	individual tests - they need to be defined here so they are in the
  ;	scope enclosing all the tests.
  ;
  (define val1 gstep-nil)
  (define val2 gstep-nil)
  (define date1 gstep-nil)
  (define date2 gstep-nil)

  ;
  ;	Now initialize those variables inside a testcase so that, in the
  ;	event of a major error in intialisation, we get an unresolved test.
  ;
  (greg-testcase "We can build basic objects for creation tests" #t
    (lambda ()
      (set! val1 ($$ "1999-12-31 23:59:59"))
      (set! val2 ($$ "%Y-%m-%d %H:%M:%S"))
      #t
    )
  )

  ;
  ; Simple class tests.
  ;
  (greg-testcase "+calendarDate works" #t
    (lambda ()
      (set! date1 ([] "NSCalendarDate" calendarDate))
      (and
	(not (equal? date1 gstep-nil))
	(gstep-bool ([] date1 isKindOfClass: ([] "NSCalendarDate" class)))
      )
    )
  )

  (greg-testcase "+dateWithString:calendarFormat: works" #t
    (lambda ()
      (set! date1 ([] "NSCalendarDate" dateWithString: val1 calendarFormat: val2))
      (not (equal? date1 gstep-nil))
    )
  )

  ; Note: This test might log a warning if the locale is different than English
  ; probably setting LC_ALL to 'en' in your environment will fix things.
  (greg-testcase "+dateWithString:calendarFormat: with nil format works" #t
    (lambda ()
      (set! date1 ([] "NSCalendarDate" dateWithString: ($$ "Fri Oct 27 08:41:14 MDT 2000")  calendarFormat: gstep-nil))
      (not (equal? date1 gstep-nil))
    )
  )

  ; d0   Date to compare
  ; y    expected year
  ; m    expected month
  ; d    expected day
  ; h    expected hour
  ; i    expected minute
  ; s    expected second
  (define (test-datecheck d0 y m d h i s)
    (begin
      
      (greg-testcase 
        (string-append "  Date check with " (gstep-nsstring ([] d0 description))) #t
          (lambda ()
	    (and
	      (equal? y ([] d0 yearOfCommonEra))
	      (equal? m ([] d0 monthOfYear))
	      (equal? d ([] d0 dayOfMonth))
	      (equal? h ([] d0 hourOfDay))
	      (equal? i ([] d0 minuteOfHour))
	      (equal? s ([] d0 secondOfMinute))
	    )
	  )
      )
    )
  )

  ;
  ; Now do some checking
  ;
  ; Y2K Checks
  (set! date1 ([] "NSCalendarDate" dateWithString: ($$ "1999-12-31 23:59:59") calendarFormat: val2))
  (test-datecheck date1 1999 12 31 23 59 59)

  (set! date2 ([] date1 dateByAddingYears: 0 months: 0 days: 0 hours: 0 minutes: 0 seconds: 1))
  (test-datecheck date2 2000 01 01 00 00 00)

  (set! date2 ([] date2 dateByAddingYears: 0 months: 0 days: 0 hours: 0 minutes: 0 seconds: 1))
  (test-datecheck date2 2000 01 01 00 00 01)

  (set! date2 ([] date2 dateByAddingYears: 0 months: 0 days: 0 hours: 1 minutes: 0 seconds: 0))
  (test-datecheck date2 2000 01 01 01 00 01)

  (set! date2 ([] date2 dateByAddingYears: 0 months: 0 days: 0 hours: -2 minutes: 0 seconds: 0))
  (test-datecheck date2 1999 12 31 23 00 01)

  ; Y2K is a leap year Checks
  (set! date2 ([] "NSCalendarDate" dateWithString: ($$ "2000-2-28 23:59:59") calendarFormat: val2))
  (test-datecheck date2 2000 02 28 23 59 59)

  (set! date2 ([] date2 dateByAddingYears: 0 months: 0 days: 0 hours: 0 minutes: 0 seconds: 1))
  (test-datecheck date2 2000 02 29 00 00 00)

  (set! date2 ([] date2 dateByAddingYears: 0 months: 0 days: 0 hours: 0 minutes: 0 seconds: 1))
  (test-datecheck date2 2000 02 29 00 00 01)

  (set! date2 ([] date2 dateByAddingYears: 0 months: 0 days: 0 hours: 1 minutes: 0 seconds: 0))
  (test-datecheck date2 2000 02 29 01 00 01)

  (set! date2 ([] date2 dateByAddingYears: 0 months: 0 days: 0 hours: -2 minutes: 0 seconds: 0))
  (test-datecheck date2 2000 02 28 23 00 01)

  (set! date2 ([] date2 dateByAddingYears: 0 months: 0 days: 0 hours: 5 minutes: 0 seconds: 0))
  (test-datecheck date2 2000 02 29 04 00 01)

  (set! date2 ([] date2 dateByAddingYears: 1 months: 0 days: 0 hours: 0 minutes: 0 seconds: 0))
  (test-datecheck date2 2001 03 01 04 00 01)

  (set! date2 ([] date2 dateByAddingYears: 0 months: 0 days: -1 hours: 0 minutes: 0 seconds: 0))
  (test-datecheck date2 2001 02 28 04 00 01)

  (set! date2 ([] date2 dateByAddingYears: 1 months: 0 days: 1 hours: 0 minutes: 0 seconds: 0))
  (test-datecheck date2 2002 03 01 04 00 01)

  ; 2004 is a leap year Checks
  (set! date2 ([] "NSCalendarDate" dateWithString: ($$ "2004-2-28 23:59:59") calendarFormat: val2))
  (test-datecheck date2 2004 02 28 23 59 59)

  (set! date2 ([] date2 dateByAddingYears: 0 months: 0 days: 0 hours: 0 minutes: 0 seconds: 1))
  (test-datecheck date2 2004 02 29 00 00 00)

  (set! date2 ([] date2 dateByAddingYears: 0 months: 0 days: 0 hours: 0 minutes: 0 seconds: 1))
  (test-datecheck date2 2004 02 29 00 00 01)

  (set! date2 ([] date2 dateByAddingYears: 0 months: 0 days: 0 hours: 1 minutes: 0 seconds: 0))
  (test-datecheck date2 2004 02 29 01 00 01)

  (set! date2 ([] date2 dateByAddingYears: 0 months: 0 days: 0 hours: -2 minutes: 0 seconds: 0))
  (test-datecheck date2 2004 02 28 23 00 01)

  ; 2100 is not a leap year Checks
  (set! date2 ([] "NSCalendarDate" dateWithString: ($$ "2100-2-28 23:59:59") calendarFormat: val2))
  (test-datecheck date2 2100 02 28 23 59 59 )

  (set! date2 ([] date2 dateByAddingYears: 0 months: 0 days: 0 hours: 0 minutes: 0 seconds: 1))
  (test-datecheck date2 2100 03 01 00 00 00)

  (set! date2 ([] date2 dateByAddingYears: 0 months: 0 days: 0 hours: 0 minutes: 0 seconds: 1))
  (test-datecheck date2 2100 03 01 00 00 01)

  (set! date2 ([] date2 dateByAddingYears: 0 months: 0 days: 0 hours: 1 minutes: 0 seconds: 0))
  (test-datecheck date2 2100 03 01 01 00 01)

  (set! date2 ([] date2 dateByAddingYears: 0 months: 0 days: 0 hours: -2 minutes: 0 seconds: 0))
  (test-datecheck date2  2100 02 28 23 00 01)


  ; Start daylight savings Checks
  ([] "NSTimeZone" setDefaultTimeZone:
    ([] "NSTimeZone" timeZoneWithName: ($$ "GB")))

  (set! date2 ([] "NSCalendarDate" dateWithString: ($$ "2002-3-31 00:30:00") calendarFormat: val2))
  (test-datecheck date2 2002 03 31 00 30 00)

  (set! date2 ([] date2 dateByAddingYears: 0 months: 0 days: 0 hours: 1 minutes: 0 seconds: 0))
  (test-datecheck date2 2002 03 31 02 30 00)

  (set! date2 ([] date2 dateByAddingYears: 0 months: 0 days: 0 hours: -1 minutes: 0 seconds: 0))
  (test-datecheck date2 2002 03 31 00 30 00)

  (set! date2 ([] date2 dateByAddingYears: 0 months: 0 days: 0 hours: 2 minutes: 0 seconds: 0))
  (test-datecheck date2 2002 03 31 02 30 00)

  (set! date2 ([] date2 dateByAddingYears: 0 months: 0 days: 0 hours: -1 minutes: 0 seconds: 0))
  (test-datecheck date2 2002 03 31 00 30 00)

  (set! date2 ([] date2 dateByAddingYears: 0 months: 0 days: 0 hours: -1 minutes: 0 seconds: 0))
  (test-datecheck date2 2002 03 30 23 30 00)

  ; End daylight savings Checks
  (set! date2 ([] "NSCalendarDate" dateWithString: ($$ "2002-10-27 00:30:00") calendarFormat: val2))
  (test-datecheck date2 2002 10 27 00 30 00)

  (set! date2 ([] date2 dateByAddingYears: 0 months: 0 days: 0 hours: 1 minutes: 0 seconds: 0))
  (test-datecheck date2 2002 10 27 01 30 00)

  (set! date2 ([] date2 dateByAddingYears: 0 months: 0 days: 0 hours: -1 minutes: 0 seconds: 0))
  (test-datecheck date2 2002 10 27 00 30 00)

  (set! date2 ([] date2 dateByAddingYears: 0 months: 0 days: 0 hours: 2 minutes: 0 seconds: 0))
  (test-datecheck date2 2002 10 27 02 30 00)

)
