;   test01.scm - Tests for parsing cal dates when no timezone is specified
;   Copyright (C) 2000 Free SoftwareFoundation, Inc.
;
;   Written by: Nicola Pero <n.pero@mi.flashnet.it>
;   Date: Dec 2000
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

;
;	Do everything inside a 'begin' procedure so that any variables
;	defined will have scope limited to this file.
;
(begin

  (define calendar-date-from-string 
    (lambda (date-string)
      ([] "NSCalendarDate" dateWithString: ($$ date-string)
       calendarFormat: ($$ "%b %d %y %H:%M:%S"))))

  (greg-testcase 
   "+dateWithString:calendarFormat: works if no time zone is specified" #t
   (lambda ()
     (let ((time-from-string 
	    (lambda (date-string)
	      ([] (calendar-date-from-string date-string)
	       timeIntervalSinceReferenceDate))))
       (let ((time1 (time-from-string "Nov 20 02 01:54:22"))
	     (time2 (time-from-string "Nov 20 02 02:54:22"))
	     (time3 (time-from-string "Nov 20 02 03:54:22"))
	     (time4 (time-from-string "Nov 20 02 04:54:22"))
	     (time5 (time-from-string "Nov 20 02 05:54:22"))
	     (time6 (time-from-string "Nov 20 02 06:54:22"))
	     (time7 (time-from-string "Nov 20 02 07:54:22"))
	     (time8 (time-from-string "Nov 20 02 08:54:22"))
	     (time9 (time-from-string "Nov 20 02 09:54:22")))
	 (and (< time1 time2)
	      (< time2 time3)
	      (< time3 time4)
	      (< time4 time5)
	      (< time5 time6)
	      (< time6 time7)
	      (< time7 time8)
	      (< time8 time9))))))
  (greg-testcase 
   "-addTimeInterval: works on a NSCalendarDate parsed with no timezone" #t
   (lambda ()
     (let ((date (calendar-date-from-string "Nov 29 01:25:38")))
       (= (+ ([] date timeIntervalSinceReferenceDate) 1)
	  ([] ([] date addTimeInterval: 1) timeIntervalSinceReferenceDate)))))
  )




