;   test02.scm - Tests for printing calendar dates using calendar formats
;   Copyright (C) 2001 Free SoftwareFoundation, Inc.
;
;   Written by: Nicola Pero <nicola@brainstorm.co.uk>
;   Date: May 2001
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

(begin

  (define my-locale ([] "NSMutableDictionary" new))

  (define tmp-array ([] "NSMutableArray" new))
  ([] tmp-array addObject: ($$ "Gen"))
  ([] tmp-array addObject: ($$ "Feb"))
  ([] tmp-array addObject: ($$ "Mar"))
  ([] tmp-array addObject: ($$ "Apr"))
  ([] tmp-array addObject: ($$ "Mag"))
  ([] tmp-array addObject: ($$ "Giu"))
  ([] tmp-array addObject: ($$ "Lug"))
  ([] tmp-array addObject: ($$ "Ago"))
  ([] tmp-array addObject: ($$ "Set"))  
  ([] tmp-array addObject: ($$ "Ott"))
  ([] tmp-array addObject: ($$ "Nov"))
  ([] tmp-array addObject: ($$ "Dic"))
  ([] my-locale setObject: tmp-array  forKey: ($$ "NSShortMonthNameArray"))

  (set! tmp-array ([] "NSMutableArray" new))
  ([] tmp-array addObject: ($$ "Gennaio"))
  ([] tmp-array addObject: ($$ "Febbraio"))
  ([] tmp-array addObject: ($$ "Marzo"))
  ([] tmp-array addObject: ($$ "Aprile"))
  ([] tmp-array addObject: ($$ "Maggio"))
  ([] tmp-array addObject: ($$ "Giugno"))
  ([] tmp-array addObject: ($$ "Luglio"))
  ([] tmp-array addObject: ($$ "Agosto"))
  ([] tmp-array addObject: ($$ "Settembre"))  
  ([] tmp-array addObject: ($$ "Ottobre"))
  ([] tmp-array addObject: ($$ "Novembre"))
  ([] tmp-array addObject: ($$ "Dicembre"))
  ([] my-locale setObject: tmp-array  forKey: ($$ "NSMonthNameArray"))

  (set! tmp-array ([] "NSMutableArray" new))
  ([] tmp-array addObject: ($$ "Dom"))
  ([] tmp-array addObject: ($$ "Lun"))
  ([] tmp-array addObject: ($$ "Mar"))
  ([] tmp-array addObject: ($$ "Mer"))
  ([] tmp-array addObject: ($$ "Gio"))
  ([] tmp-array addObject: ($$ "Ven"))
  ([] tmp-array addObject: ($$ "Sab"))
  ([] my-locale setObject: tmp-array  forKey: ($$ "NSShortWeekDayNameArray"))

  (set! tmp-array ([] "NSMutableArray" new))
  ([] tmp-array addObject: ($$ "Domenica"))
  ([] tmp-array addObject: ($$ "Lunedi`"))
  ([] tmp-array addObject: ($$ "Martedi`"))
  ([] tmp-array addObject: ($$ "Mercoledi`"))
  ([] tmp-array addObject: ($$ "Giovedi`"))
  ([] tmp-array addObject: ($$ "Venerdi`"))
  ([] tmp-array addObject: ($$ "Sabato"))
  ([] my-locale setObject: tmp-array  forKey: ($$ "NSWeekDayNameArray"))

  (set! tmp-array ([] "NSMutableArray" new))
  ([] tmp-array addObject: ($$ "AM"))
  ([] tmp-array addObject: ($$ "PM"))
  ([] my-locale setObject: tmp-array  forKey: ($$ "NSAMPMDesignation"))
  
  ;; TODO: NSTimeDateFormatString
  
  (define my-birthday ([] "NSCalendarDate" dateWithYear: 1974 
		       month: 11 
		       day: 20 
		       hour: 13 
		       minute: 0 
		       second: 0 
		       timeZone: ([] "NSTimeZone" timeZoneWithName: 
				  ($$ "MET"))))

  (define check-format 
    (lambda (format result)
      (lambda ()
	(gstep-bool ([] ([] my-birthday 
			 descriptionWithCalendarFormat: ($$ format)  
			 locale: my-locale) 
		     isEqualToString: ($$ result))))))
  
  (greg-testcase "%% format works in description" #t 
		 (check-format "%%" "%"))

  (greg-testcase "%a format works in description" #t 
		 (check-format "%a" "Mer"))

  (greg-testcase "%A format works in description" #t 
		 (check-format "%A" "Mercoledi`"))

  (greg-testcase "%b format works in description" #t 
		 (check-format "%b" "Nov"))

  (greg-testcase "%B format works in description" #t 
		 (check-format "%B" "Novembre"))

  (greg-testcase "%d format works in description" #t 
		 (check-format "%d" "20"))

  (greg-testcase "%e format works in description" #t 
		 (check-format "%e" "20"))

  (greg-testcase "%F format works in description" #t 
		 (check-format "%F" "000"))

  (greg-testcase "%H format works in description" #t 
		 (check-format "%H" "13"))

  (greg-testcase "%I format works in description" #t 
		 (check-format "%I" "01"))

  (greg-testcase "%j format works in description" #t 
		 (check-format "%j" "324"))

  (greg-testcase "%m format works in description" #t 
		 (check-format "%m" "11"))

  (greg-testcase "%M format works in description" #t 
		 (check-format "%M" "00"))

  (greg-testcase "%p format works in description" #t 
		 (check-format "%p" "PM"))

  (greg-testcase "%S format works in description" #t 
		 (check-format "%S" "00"))

  (greg-testcase "%w format works in description" #t 
		 (check-format "%w" "3"))

  (greg-testcase "%y format works in description" #t 
		 (check-format "%y" "74"))

  (greg-testcase "%Y format works in description" #t 
		 (check-format "%Y" "1974"))

  (greg-testcase "%Z format works in description" #t 
		 (check-format "%Z" "MET"))

  (greg-testcase "%z format works in description" #t 
		 (check-format "%z" "+0100"))

  (greg-testcase "%d %m %Y format works in description" #t 
		 (check-format "%d %m %Y" "20 11 1974"))

  (greg-testcase "%d %B %y format works in description" #t 
		 (check-format "%d %B %y" "20 Novembre 74"))

  (greg-testcase "%H:%M:%S format works in description" #t 
		 (check-format "%H:%M:%S" "13:00:00"))

  (greg-testcase "%H%%%M%%%S format works in description" #t 
		 (check-format "%H%%%M%%%S" "13%00%00"))

  (greg-testcase "%H:%M (%Z) format works in description" #t 
		 (check-format "%H:%M (%Z)" "13:00 (MET)"))
)

