
;   general.scm - General NSCharacterSet tests.
;   Copyright (C) 2000 Free SoftwareFoundation, Inc.
;
;   Written: Adam Fedor <fedor@gnu.org>
;   Date: Oct 2000
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

(begin

  (define theSet gstep-nil)

  (greg-testcase "Check some characters from alphanumericCharacterSet" #t
    (lambda ()
      (set! theSet ([] "NSCharacterSet" alphanumericCharacterSet))
      (and
        (gstep-bool ([] theSet characterIsMember: (char->integer #\A)))
        (gstep-bool ([] theSet characterIsMember: (char->integer #\Z)))
        (gstep-bool ([] theSet characterIsMember: (char->integer #\a)))
        (gstep-bool ([] theSet characterIsMember: (char->integer #\z)))
        (gstep-bool ([] theSet characterIsMember: (char->integer #\0)))
        (gstep-bool ([] theSet characterIsMember: (char->integer #\9)))
        (not (gstep-bool ([] theSet characterIsMember: (char->integer #\#))))
        (not (gstep-bool ([] theSet characterIsMember: (char->integer #\space))))
        (not (gstep-bool ([] theSet characterIsMember: (char->integer #\newline))))
      )
    )
  )

  (greg-testcase "Check some characters from lowercaseLetterCharacterSet" #t
    (lambda ()
      (set! theSet ([] "NSCharacterSet" lowercaseLetterCharacterSet))
      (and
        (not (gstep-bool ([] theSet characterIsMember: (char->integer #\A))))
        (not (gstep-bool ([] theSet characterIsMember: (char->integer #\Z))))
        (gstep-bool ([] theSet characterIsMember: (char->integer #\a)))
        (gstep-bool ([] theSet characterIsMember: (char->integer #\z)))
        (not (gstep-bool ([] theSet characterIsMember: (char->integer #\0))))
        (not (gstep-bool ([] theSet characterIsMember: (char->integer #\9))))
        (not (gstep-bool ([] theSet characterIsMember: (char->integer #\#))))
        (not (gstep-bool ([] theSet characterIsMember: (char->integer #\space))))
        (not (gstep-bool ([] theSet characterIsMember: (char->integer #\newline))))
      )
    )
  )

  (greg-testcase "Check some characters from whitespaceAndNewlineCharacterSet " #t
    (lambda ()
     (set! theSet ([] "NSCharacterSet" whitespaceAndNewlineCharacterSet))
     (and
        (not (gstep-bool ([] theSet characterIsMember: (char->integer #\A))))
        (not (gstep-bool ([] theSet characterIsMember: (char->integer #\Z))))
        (not (gstep-bool ([] theSet characterIsMember: (char->integer #\a))))
        (not (gstep-bool ([] theSet characterIsMember: (char->integer #\z))))
        (not (gstep-bool ([] theSet characterIsMember: (char->integer #\0))))
        (not (gstep-bool ([] theSet characterIsMember: (char->integer #\9))))
        (not (gstep-bool ([] theSet characterIsMember: (char->integer #\#))))
        (gstep-bool ([] theSet characterIsMember: (char->integer #\space)))
        (gstep-bool ([] theSet characterIsMember: (char->integer #\newline)))
        (gstep-bool ([] theSet characterIsMember: (char->integer #\tab)))
      )
    )
  )

  (greg-testcase "-bitmapRepresentation works" #t
    (lambda ()
      (define data1 ([] theSet bitmapRepresentation))
      (and
        (not (equal? data1 gstep-nil))
        (gstep-bool ([] data1 isKindOfClass: ([] "NSData" class)))
      )
    )
  )

  (greg-testcase "-invertedSet works" #t
    (lambda ()
      (define iSet ([] theSet invertedSet))
      (and
        (not (equal? iSet gstep-nil))
        (gstep-bool ([] iSet isKindOfClass: ([] "NSCharacterSet" class)))
        (gstep-bool ([] iSet characterIsMember: (char->integer #\A)))
        (gstep-bool ([] iSet characterIsMember: (char->integer #\Z)))
        (gstep-bool ([] iSet characterIsMember: (char->integer #\a)))
        (gstep-bool ([] iSet characterIsMember: (char->integer #\z)))
        (gstep-bool ([] iSet characterIsMember: (char->integer #\0)))
        (gstep-bool ([] iSet characterIsMember: (char->integer #\9)))
        (gstep-bool ([] iSet characterIsMember: (char->integer #\#)))
        (not (gstep-bool ([] iSet characterIsMember: (char->integer #\space))))
        (not (gstep-bool ([] iSet characterIsMember: (char->integer #\newline))))
      )
    )
  )

  ;
  ; Custom sets
  ;
  (greg-testcase "Create custom set with characterSetWithCharactersInString:" #t
    (lambda ()
      (set! theSet ([] "NSCharacterSet" characterSetWithCharactersInString: ($$ "Not a set")))
      (and 
        (not (equal? theSet gstep-nil))
        (gstep-bool ([] theSet isKindOfClass: ([] "NSCharacterSet" class)))
      )
    )
  )

  (greg-testcase "Check custom set" #t
    (lambda ()
      (and
        (gstep-bool ([] theSet characterIsMember: (char->integer #\space)))
        (gstep-bool ([] theSet characterIsMember: (char->integer #\N)))
        (gstep-bool ([] theSet characterIsMember: (char->integer #\o)))
        (not (gstep-bool ([] theSet characterIsMember: (char->integer #\A))))
        (not (gstep-bool ([] theSet characterIsMember: (char->integer #\#))))
      )
    )
  )

)


