
;   general.scm - General tests for NSData
;   Copyright (C) 2000 Free SoftwareFoundation, Inc.
;
;   Written by:  Adam Fedor <fedor@gnu.org>
;   Date: Oct 2000
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

;
;	Do everything inside a 'begin' procedure so that any variables
;	defined will have scope limited to this file.
;
(begin

  ;
  ;	Define variables that we are going to use to contain values between
  ;	individual tests - they need to be defined here so they are in the
  ;	scope enclosing all the tests.
  ;
  (define data1 gstep-nil)
  (define data2 gstep-nil)
  (define byte1 voidp-nil)

  (define str1 "Test string for data classes")
  (define ptr1 (string->voidp str1))
  (define str2 "Test string for data classes not copied")
  (define ptr2 (string->voidp str1))
  (voidp-set-malloc! ptr2 #f)

  (define mutable ([] "NSMutableData" dataWithLength: 100))
  (define hold1 ([] mutable mutableBytes))

  ;
  ; Creating and Initializing
  ;
  (greg-testcase "+dataWithBytes:length: works" #t
    (lambda ()
      (set! data1 ([] "NSData" dataWithBytes: ptr1 length: (voidp-length ptr1)))
      (and
	(not (equal? data1 gstep-nil))
	(gstep-bool ([] data1 isKindOfClass: ([] "NSData" class)))
	(equal? (voidp-length ptr1) ([] data1 length))
      )
    )
  )


  (greg-testcase "+dataWithBytesNoCopy:length: works" #t
    (lambda ()
      (set! data2 ([] "NSData" dataWithBytesNoCopy: ptr2 length: (voidp-length ptr2) ))
      (and
	(not (equal? data2 gstep-nil))
	(gstep-bool ([] data2 isKindOfClass: ([] "NSData" class)))
	(equal? (voidp-length ptr2) ([] data2 length))
      )
    )
  )

  (greg-testcase "+dataWithBytes:length works with 0 length" #t
    (lambda ()
      (set! data1 ([] "NSData" dataWithBytes: (voidp-nil) length: 0))
       (and
	(not (equal? data1 gstep-nil))
	(gstep-bool ([] data1 isKindOfClass: ([] "NSData" class)))
      )
    )
  )

  ;
  ; Range tests
  (greg-testcase "Get data range" #t
    (lambda ()
      (define rng (NSMakeRange 2 6))
      ([] data2 getBytes: hold1 range: rng)
      #t
    )
  )

  (test-exception "getBytes:range: with bad location"
        NSRangeException
    (lambda ()
      (define rng (NSMakeRange (voidp-length ptr2) 1))
      (([] data2 getBytes: hold1 range: rng))
      #f
    )
  )

  (test-exception "getBytes:range: with bad length"
        NSRangeException
    (lambda ()
      (define rng (NSMakeRange 1 (voidp-length ptr2)))
      ([] data2 getBytes: hold1 range: rng)
      #f
    )
  )

  (test-exception "subdataWithRange: with bad location"
        NSRangeException
    (lambda ()
      (define rng (NSMakeRange 1 (voidp-length ptr2)))
      (set! byte1 ([] data2 subdataWithRange: rng))
      #f
    )
  )

  (test-exception "subdataWithRange: with bad length"
        NSRangeException
    (lambda ()
      (define rng (NSMakeRange (voidp-length ptr2) 1))
      (set! byte1 ([] data2 subdataWithRange: rng))
      #f
    )
  )

)