
;   create.scm - Creation tests for NSDate
;   Copyright (C) 1998 Free SoftwareFoundation, Inc.
;
;   Written by:  Adam Fedor <fedor@gnu.org>
;   Date: Oct 2000
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

;
;	Do everything inside a 'begin' procedure so that any variables
;	defined will have scope limited to this file.
;
(begin

  ;
  ;	Define variables that we are going to use to contain values between
  ;	individual tests - they need to be defined here so they are in the
  ;	scope enclosing all the tests.
  ;
  (define val1 gstep-nil)
  (define date1 gstep-nil)
  (define date2 gstep-nil)

  ;
  ;	Now initialize those variables inside a testcase so that, in the
  ;	event of a major error in intialisation, we get an unresolved test.
  ;
  (greg-testcase "We can build basic objects for creation tests" #t
    (lambda ()
      (set! val1 ([] "NSString" stringWithCString: "2000-10-19"))
      #t
    )
  )

  ;
  ; Simple class tests.
  ;
  (greg-testcase "+date works" #t
    (lambda ()
      (set! date1 ([] "NSDate" date))
      (and
	(not (equal? date1 gstep-nil))
	(gstep-bool ([] date1 isKindOfClass: ([] "NSDate" class)))
      )
    )
  )

  (greg-testcase "+dateWithString works" #t
    (lambda ()
      (set! date1 ([] "NSDate" dateWithString: ($$ "2000-10-19")))
      (not (equal? date1 gstep-nil))
    )
  )

  (greg-testcase "+dateWithTimeIntervalSinceReferenceDate: works" #t
    (lambda ()
      (set! date2 ([] "NSCalendarDate" dateWithTimeIntervalSinceReferenceDate:
	([] date1 timeIntervalSinceReferenceDate)))
      (not (equal? date2 gstep-nil))
    )
  )

  (greg-testcase "+dateWithString makes correct day" #t
    (lambda ()
      (eq? 19 ([] date2 dayOfMonth))
    )
  )

  (greg-testcase "+dateWithString makes correct month" #t
    (lambda ()
      (eq? 10 ([] date2 monthOfYear))
    )
  )

  (greg-testcase "+dateWithString makes correct year" #t
    (lambda ()
      (eq? 2000 ([] date2 yearOfCommonEra))
    )
  )

  (greg-testcase "+dateWithTimeIntervalSinceNow: works" #t
    (lambda ()
      (set! date1 ([] "NSDate" dateWithTimeIntervalSinceNow: 0))
      (not (equal? date1 gstep-nil))
    )
  )

  (greg-testcase "+dateWithTimeIntervalSince1970: works" #t
    (lambda ()
      (set! date1 ([] "NSDate" dateWithTimeIntervalSince1970: 0))
      (not (equal? date1 gstep-nil))
    )
  )

  (greg-testcase "+dateWithTimeIntervalSinceReferenceDate: works" #t
    (lambda ()
      (set! date1 ([] "NSDate" dateWithTimeIntervalSinceReferenceDate: 0))
      (not (equal? date1 gstep-nil))
    )
  )

  (greg-testcase "+distantFuture works" #t
    (lambda ()
      (set! date1 ([] "NSDate" distantFuture))
      (not (equal? date1 gstep-nil))
    )
  )

  (greg-testcase "+distantPast works" #t
    (lambda ()
      (set! date1 ([] "NSDate" distantPast))
      (not (equal? date1 gstep-nil))
    )
  )


)
