
;   general.scm - Creation tests for NSDate
;   Copyright (C) 1998 Free SoftwareFoundation, Inc.
;
;   Written by:  Adam Fedor <fedor@gnu.org>
;   Date: Oct 2000
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

;
;	Do everything inside a 'begin' procedure so that any variables
;	defined will have scope limited to this file.
;
(begin

  ;
  ;	Define variables that we are going to use to contain values between
  ;	individual tests - they need to be defined here so they are in the
  ;	scope enclosing all the tests.
  ;
  (define cdate gstep-nil)
  (define date1 gstep-nil)
  (define date2 gstep-nil)

  (define comps 0)
  ;
  ;	Now initialize those variables inside a testcase so that, in the
  ;	event of a major error in intialisation, we get an unresolved test.
  ;
  (greg-testcase "We can build basic objects for creation tests" #t
    (lambda ()
      (set! cdate ([] "NSDate" date))
      #t
    )
  )

  ;
  ; Comparison
  ;
  (greg-testcase "+distantFuture is in the future" #t
    (lambda ()
      (set! comps ([] cdate compare: ([] "NSDate" distantFuture)))
      (equal? comps NSOrderedAscending)
    )
  )

  (greg-testcase "+distantPast is in the past" #t
    (lambda ()
      (set! comps ([] cdate compare: ([] "NSDate" distantPast)))
      (equal? comps NSOrderedDescending)
    )
  )

  (greg-testcase "-earlierDate works" #t
    (lambda ()
      (set! date1 ([] "NSDate" dateWithTimeIntervalSinceNow: -600))
      (set! date2 ([] cdate earlierDate: date1))
      (equal? date1 date2)
    )  
  )

  (greg-testcase "-laterDate works" #t
    (lambda ()
      (set! date1 ([] "NSDate" dateWithTimeIntervalSinceNow: -600))
      (set! date2 ([] cdate laterDate: date1))
      (equal? cdate date2)
    )
  )

  (greg-testcase "-isEqualToDate works" #t
    (lambda ()
      (set! date1 ([] "NSDate" dateWithTimeIntervalSinceNow: -600))
      (set! date2 ([] date1 addTimeInterval: 0))
      (gstep-bool ([] date1 isEqualToDate: date2))
    )
  )

)






