
;   create.scm - Tests of calling Objective-C methods from Guile
;   Copyright (C) 1998 Free SoftwareFoundation, Inc.
;
;   Written by:  Richard Frith-Macdonald <richard@brainstorm.co.uk>
;   Date: October 1998
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

;
;	Do everything inside a 'begin' procedure so that any variables
;	defined will have scope limited to this file.
;
(begin

  ;
  ;	Define variables that we are going to use to contain values between
  ;	individual tests - they need to be defined here so they are in the
  ;	scope enclosing all the tests.
  ;
  (define key1 gstep-nil)
  (define key2 gstep-nil)
  (define key3 gstep-nil)
  (define keys1 gstep-nil)
  (define keys2 gstep-nil)
  (define keys3 gstep-nil)
  (define val1 gstep-nil)
  (define val2 gstep-nil)
  (define val3 gstep-nil)
  (define vals1 gstep-nil)
  (define vals2 gstep-nil)
  (define vals3 gstep-nil)
  (define ptrkey1 voidp-nil)
  (define ptrkeys voidp-nil)
  (define ptrval1 voidp-nil)
  (define ptrvals voidp-nil)

  ;
  ;	Now initialise those variables inside a testcase so that, in the
  ;	event of a major error in intialisation, we get an unresolved test.
  ;
  (greg-testcase "We can build basic strings and arrays for tests" #t
    (lambda ()
      ;
      ;	Three strings to use as dictionary keys.
      ;
      (set! key1 ([] "NSString" stringWithCString: "Key1"))
      (set! key2 ([] "NSString" stringWithCString: "Key2"))
      (set! key3 ([] "NSString" stringWithCString: "Key3"))

      ;
      ;	An array of two keys for initialising a dictionary.
      ;
      (set! keys1 ([] ([] "NSArray" arrayWithObject: key1)
			arrayByAddingObject: key2))

      ;
      ;	An array of three keys for initialising a dictionary.
      ;
      (set! keys2 ([] keys1 arrayByAddingObject: key3))

      ;
      ;	An array of three keys for initialising with duplicate keys.
      ;
      (set! keys3 ([] keys1 arrayByAddingObject: key2))

      ;
      ; Create a pointer to a key.
      ;
      (set! ptrkey1 (list->voidp (list key1) "@"))

      ;
      ;	A pointer to a 'C' array of keys.
      ;
      (set! ptrkeys (list->voidp (list key1 key2 key3) "@"))

      ;
      ;	Three strings to use as dictionary values.
      ;
      (set! val1 ([] "NSString" stringWithCString: "Hello"))
      (set! val2 ([] "NSString" stringWithCString: "Goodbye"))
      (set! val3 ([] "NSString" stringWithCString: "Testing"))

      ;
      ;	An array of values to be used to initialise a dictionary.
      ;
      (set! vals1 ([] ([] "NSArray" arrayWithObject: val1)
			arrayByAddingObject: val2))

      ;
      ;	An array of values to be used to initialise with duplicate values.
      ;
      (set! vals2 ([] vals1 arrayByAddingObject: val2))

      ;
      ;	An array of values to be used to initialise a dictionary.
      ;
      (set! vals3 ([] vals1 arrayByAddingObject: val3))

      ;
      ;	A pointer to a value.
      ;
      (set! ptrval1 (list->voidp (list val1) "@"))

      ;
      ;	A pointer to a 'C' array of values.
      ;
      (set! ptrvals (list->voidp (list val1 val2 val3) "@"))
      #t
    )
  )

  ;
  ;	Simple test for the '+new' method - we invoke the method, then we
  ;	check that the result is not a nil object, it is of the correct
  ;	class, and that it is a dictionary with zero keys.
  ;
  (greg-testcase "+new creates an empty dictionary" #t
    (lambda ()
      (define obj ([] "NSDictionary" new))
      (and
	(not (equal? obj gstep-nil))
	(gstep-bool ([] obj isKindOfClass: ([] "NSDictionary" class)))
	(eq? 0 ([] obj count))
      )
    )
  )

  (greg-testcase "+dictionary creates an empty dictionary" #t
    (lambda ()
      (define obj ([] "NSDictionary" dictionary))
      (and
	(not (equal? obj gstep-nil))
	(gstep-bool ([] obj isKindOfClass: ([] "NSDictionary" class)))
	(eq? 0 ([] obj count))
      )
    )
  )

  ;
  ;	Now we test that exceptions are generated as they should be - 
  ;	use a key of nil when trying to create a dictionary.
  ;
  (test-exception "+dictionaryWithObject:forKey: with nil key"
	NSInvalidArgumentException
    (lambda ()
      (define obj ([] "NSDictionary" dictionaryWithObject: val1
						   forKey: gstep-nil))
      #f
    )
  )

  (test-exception "+dictionaryWithObject:forKey: with nil value"
	NSInvalidArgumentException
    (lambda ()
      (define obj ([] "NSDictionary" dictionaryWithObject: gstep-nil
						   forKey: key1))
      #f
    )
  )

  (greg-testcase "+dictionaryWithObject:forKey: builds minimal dictionary" #t
    (lambda ()
      (define obj ([] "NSDictionary" dictionaryWithObject: val1 forKey: key1))
      (and
	(not (equal? obj gstep-nil))
	(gstep-bool ([] obj isKindOfClass: ([] "NSDictionary" class)))
	(eq? 1 ([] obj count))
      )
    )
  )

  (greg-testcase "+dictionaryWithObjects:forKeys:count: builds a dictionary" #t
    (lambda ()
      (define obj ([] "NSDictionary" dictionaryWithObjects: ptrvals
						   forKeys: ptrkeys
						     count: 3))
      (and
	(not (equal? obj gstep-nil))
	(gstep-bool ([] obj isKindOfClass: ([] "NSDictionary" class)))
	(eq? 3 ([] obj count))
      )
    )
  )

  (define old gstep-nil)
  (greg-testcase "+dictionaryWithObjects:forKeys: builds a dictionary" #t
    (lambda ()
      (define obj ([] "NSDictionary" dictionaryWithObjects: vals1
						   forKeys: keys1))
      (set! old obj)
      (and
	(not (equal? obj gstep-nil))
	(gstep-bool ([] obj isKindOfClass: ([] "NSDictionary" class)))
	(eq? 2 ([] obj count))
      )
    )
  )

  (test-exception "+dictionaryWithObjects:forKeys: with arrays of different sizes" NSInvalidArgumentException
    (lambda ()
      (define obj ([] "NSDictionary" dictionaryWithObjects: vals1
						   forKeys: keys2))
      #f
    )
  )

  (greg-testcase "we can have multiple identical objects in a dictionary" #t
    (lambda ()
      (define obj ([] "NSDictionary" dictionaryWithObjects: vals2
						   forKeys: keys2))
      (and
	(not (equal? obj gstep-nil))
	(gstep-bool ([] obj isKindOfClass: ([] "NSDictionary" class)))
	(eq? 3 ([] obj count))
      )
    )
  )

  (greg-testcase "we can't have multiple identical keys in a dictionary" #t
    (lambda ()
      (define obj ([] "NSDictionary" dictionaryWithObjects: vals3
						   forKeys: keys3))
      (and
	(not (equal? obj gstep-nil))
	(gstep-bool ([] obj isKindOfClass: ([] "NSDictionary" class)))
	(eq? 2 ([] obj count))
      )
    )
  )

  (greg-testcase "+dictionaryWithDictionary: copies dictionary" #t
    (lambda ()
      (define obj ([] "NSDictionary" dictionaryWithDictionary: old))
      (and
	(not (equal? obj gstep-nil))
	(gstep-bool ([] old isEqual: obj))
      )
    )
  )

)
