
;   general.scm - Tests of NSDictionary methods from Guile
;   Copyright (C) 1998 Free SoftwareFoundation, Inc.
;
;   Written by:  Richard Frith-Macdonald <richard@brainstorm.co.uk>
;   Date: October 1998
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

(begin

  (define key1 gstep-nil)
  (define key2 gstep-nil)
  (define key3 gstep-nil)
  (define keys1 gstep-nil)
  (define keys2 gstep-nil)
  (define keys3 gstep-nil)
  (define keys4 gstep-nil)
  (define val1 gstep-nil)
  (define val2 gstep-nil)
  (define val3 gstep-nil)
  (define vals1 gstep-nil)
  (define vals2 gstep-nil)
  (define vals3 gstep-nil)
  (define vals4 gstep-nil)

  (greg-testcase "We can build basic strings and arrays for tests" #t
    (lambda ()
      (set! key1 ([] "NSString" stringWithCString: "Key1"))
      (set! key2 ([] "NSString" stringWithCString: "Key2"))
      (set! key3 ([] "NSString" stringWithCString: "Key3"))
      (set! keys1 ([] ([] "NSArray" arrayWithObject: key1)
			arrayByAddingObject: key2))
      (set! keys2 ([] keys1 arrayByAddingObject: key3))
      (set! keys3 ([] keys1 arrayByAddingObject: key2))
      (set! val1 ([] "NSString" stringWithCString: "Hello"))
      (set! val2 ([] "NSString" stringWithCString: "Goodbye"))
      (set! val3 ([] "NSString" stringWithCString: "Testing"))
      (set! vals1 ([] ([] "NSArray" arrayWithObject: val1)
			arrayByAddingObject: val2))
      (set! vals2 ([] vals1 arrayByAddingObject: val2))
      (set! vals3 ([] vals1 arrayByAddingObject: val3))

      (set! vals4 ([] vals3 arrayByAddingObject: ([] "NSDate" date)))
      (set! vals4 ([] vals4 arrayByAddingObject: ([] "NSNumber" numberWithInt: 2)))
      (set! vals4 ([] vals4 arrayByAddingObject: ([] "NSData" data)))
      (set! keys4 ([] keys3 arrayByAddingObject: ($$ "date")))
      (set! keys4 ([] keys4 arrayByAddingObject: ($$ "number")))
      (set! keys4 ([] keys4 arrayByAddingObject: ($$ "data")))
      #t
    )
  )

  ;
  ;	Perform tests on an empty dictionary.
  ;
  (define dict gstep-nil)
  (greg-testcase "-count returns zero for an empty dictionary" #t
    (lambda ()
      (define obj ([] "NSDictionary" new))
      (set! dict obj)
      (and
	(not (equal? obj gstep-nil))
	(gstep-bool ([] obj isKindOfClass: ([] "NSDictionary" class)))
	(eq? 0 ([] obj count))
      )
    )
  )

  (greg-testcase "-hash returns zero for an empty dictionary" #t
    (lambda ()
      (eq? 0 ([] dict hash))
    )
  )

  (greg-testcase "-allKeys gives an empty array for an empty dictionary" #t
    (lambda ()
      (define obj ([] dict allKeys))
      (and
	(not (equal? obj gstep-nil))
	(gstep-bool ([] obj isKindOfClass: ([] "NSArray" class)))
	(eq? 0 ([] obj count))
      )
    )
  )

  (greg-testcase "-allKeysForObject: gives nil for an empty dictionary" #t
    (lambda ()
      (define obj ([] dict allKeysForObject: gstep-nil))
      (equal? obj gstep-nil)
    )
  )

  (greg-testcase "-allValues gives an empty array for an empty dictionary" #t
    (lambda ()
      (define obj ([] dict allValues))
      (and
	(not (equal? obj gstep-nil))
	(gstep-bool ([] obj isKindOfClass: ([] "NSArray" class)))
	(eq? 0 ([] obj count))
      )
    )
  )

  (greg-testcase "-objectForKey: gives nil for an empty dictionary" #t
    (lambda ()
      (let*
	(
	  (o1 ([] dict objectForKey: gstep-nil))
	  (o2 ([] dict objectForKey: key1))
	)
	(and
	  (equal? o1 gstep-nil)
	  (equal? o2 gstep-nil)
	)
      )
    )
  )

  (greg-testcase "-keyEnumerator: is ok for empty dictionary" #t
    (lambda ()
      (let*
	(
	  (e ([] dict keyEnumerator))
	  (k1 ([] e nextObject))
	  (k2 ([] e nextObject))
	)
	(and
	  (not (equal? e gstep-nil))
	  (equal? k1 gstep-nil)
	  (equal? k2 gstep-nil)
	)
      )
    )
  )

  (greg-testcase "-objectEnumerator: is ok for empty dictionary" #t
    (lambda ()
      (let*
	(
	  (e ([] dict objectEnumerator))
	  (v1 ([] e nextObject))
	  (v2 ([] e nextObject))
	)
	(and
	  (not (equal? e gstep-nil))
	  (equal? v1 gstep-nil)
	  (equal? v2 gstep-nil)
	)
      )
    )
  )

  (greg-testcase "-objectsForKeys:notFoundMarker: is ok for empty dictionary" #t
    (lambda ()
      (let*
	(
	  (notFound ([] "NSString" stringWithCString: "notFound"))
	  (a ([] dict objectsForKeys: keys1 notFoundMarker: notFound))
	)
	(and
	  (not (equal? a gstep-nil))
	  (gstep-bool ([] a isKindOfClass: ([] "NSArray" class)))
	  (eq? 2 ([] a count))
	  (equal? ([] a objectAtIndex: 0) notFound)
	  (equal? ([] a objectAtIndex: 1) notFound)
	)
      )
    )
  )

  (greg-testcase "-description gives us a text property-list" #t
    (lambda ()
      (let*
	(
	  (obj ([] dict description))
	)
	(begin
	  (set! obj ([] obj propertyList))
	  (and
	    (not (equal? obj gstep-nil))
	    (gstep-bool ([] obj isKindOfClass: ([] "NSDictionary" class)))
	    (eq? 0 ([] obj count))
	  )
	)
      )
    )
  )



  ;
  ;	Perform tests on a dictionary with contents.
  ;

  (greg-testcase "-count returns two for an dictionary with two keys" #t
    (lambda ()
      (let*
	(
	  (obj ([] "NSDictionary" dictionaryWithObjects: vals1
						forKeys: keys1))
	)
	(begin
	  (set! dict obj)
	  (and
	    (not (equal? obj gstep-nil))
	    (gstep-bool ([] obj isKindOfClass: ([] "NSDictionary" class)))
	    (eq? 2 ([] obj count))
	  )
	)
      )
    )
  )

  (greg-testcase "-hash returns two for a dictionary with two keys" #t
    (lambda ()
      (eq? 2 ([] dict hash))
    )
  )

  (greg-testcase "-allKeys gives the keys we put in the dictionary" #t
    (lambda ()
      (define obj ([] dict allKeys))
      (and
	(not (equal? obj gstep-nil))
	(gstep-bool ([] obj isKindOfClass: ([] "NSArray" class)))
	(eq? 2 ([] obj count))
	(gstep-bool ([] obj containsObject: key1))
	(gstep-bool ([] obj containsObject: key2))
      )
    )
  )

  (greg-testcase "-allKeysForObject: gives the key we expect" #t
    (lambda ()
      (define o1 ([] dict allKeysForObject: val1))
      (define o2 ([] dict allKeysForObject: val2))
      (and
	(not (equal? o1 gstep-nil))
	(gstep-bool ([] o1 isKindOfClass: ([] "NSArray" class)))
	(eq? 1 ([] o1 count))
	(gstep-bool ([] o1 containsObject: key1))
	(not (equal? o2 gstep-nil))
	(gstep-bool ([] o2 isKindOfClass: ([] "NSArray" class)))
	(eq? 1 ([] o2 count))
	(gstep-bool ([] o2 containsObject: key2))
      )
    )
  )

  (greg-testcase "-allValues gives the values we put in the dictionary" #t
    (lambda ()
      (define obj ([] dict allValues))
      (and
	(not (equal? obj gstep-nil))
	(gstep-bool ([] obj isKindOfClass: ([] "NSArray" class)))
	(eq? 2 ([] obj count))
	(gstep-bool ([] obj containsObject: val1))
	(gstep-bool ([] obj containsObject: val2))
      )
    )
  )

  (greg-testcase "-objectForKey: gives nil for a nil key" #t
    (lambda ()
      (define obj ([] dict objectForKey: gstep-nil))
      (equal? obj gstep-nil)
    )
  )

  (greg-testcase "-objectForKey: gives nil for a key not in the dictionary" #t
    (lambda ()
      (define obj ([] dict objectForKey: key3))
      (equal? obj gstep-nil)
    )
  )

  (greg-testcase "-objectForKey: gives the objects we added for the keys" #t
    (lambda ()
      (define o1 ([] dict objectForKey: key1))
      (define o2 ([] dict objectForKey: key2))
      (and
        (equal? o1 val1)
        (equal? o2 val2)
      )
    )
  )

  (greg-testcase "-keyEnumerator: enumerates the dictionary" #t
    (lambda ()
      (let*
	(
	  (e ([] dict keyEnumerator))
	  (k1 ([] e nextObject))
	  (k2 ([] e nextObject))
	  (k3 ([] e nextObject))
	)
	(and
	  (not (equal? k1 gstep-nil))
	  (not (equal? k2 gstep-nil))
	  (equal? k3 gstep-nil)
	  (not (equal? k1 k2))
	  (gstep-bool ([] keys1 containsObject: k1))
	  (gstep-bool ([] keys1 containsObject: k2))
	)
      )
    )
  )

  (greg-testcase "-objectEnumerator: enumerates the dictionary" #t
    (lambda ()
      (let*
	(
	  (e ([] dict objectEnumerator))
	  (v1 ([] e nextObject))
	  (v2 ([] e nextObject))
	  (v3 ([] e nextObject))
	)
	(and
	  (not (equal? v1 gstep-nil))
	  (not (equal? v2 gstep-nil))
	  (equal? v3 gstep-nil)
	  (not (equal? v1 v2))
	  (gstep-bool ([] vals1 containsObject: v1))
	  (gstep-bool ([] vals1 containsObject: v2))
	)
      )
    )
  )

  (greg-testcase "-objectsForKeys:notFoundMarker: is ok for dictionary" #t
    (lambda ()
      (let*
	(
	  (notFound ([] "NSString" stringWithCString: "notFound"))
	  (a ([] dict objectsForKeys: keys2 notFoundMarker: notFound))
	)
	(and
	  (not (equal? a gstep-nil))
	  (gstep-bool ([] a isKindOfClass: ([] "NSArray" class)))
	  (eq? 3 ([] a count))
	  (equal? ([] a objectAtIndex: 0) val1)
	  (equal? ([] a objectAtIndex: 1) val2)
	  (equal? ([] a objectAtIndex: 2) notFound)
	)
      )
    )
  )

  (greg-testcase "-keysSortedByValueUsingSelector: seems ok" #t
    (lambda ()
      (define a ([] dict keysSortedByValueUsingSelector: "compare:"))
      (and
	(not (equal? a gstep-nil))
	(gstep-bool ([] a isKindOfClass: ([] "NSArray" class)))
	(eq? 2 ([] a count))
	(equal? ([] a objectAtIndex: 0) key2)
	(equal? ([] a objectAtIndex: 1) key1)
      )
    )
  )

  (greg-testcase "-description gives us a text property-list" #t
    (lambda ()
      (define obj ([] dict description))
      (set! obj ([] obj propertyList))
      (and
	(not (equal? obj gstep-nil))
	(gstep-bool ([] obj isKindOfClass: ([] "NSDictionary" class)))
	(gstep-bool ([] obj isEqual: dict))
      )
    )
  )

  (greg-testcase "we can create a dictionary with several keys" #t
    (lambda ()
      (set! dict 
	([] "NSDictionary" dictionaryWithObjects: vals4 forKeys: keys4))
      (not (equal? gstep-nil dict))
    )
  )

  (greg-testcase "data/number/data are ok in serialized property-list" #t
    (lambda ()
      (define obj ([] "NSSerializer" serializePropertyList: dict))
      (set! obj ([] "NSDeserializer" deserializePropertyListFromData: obj mutableContainers: #t))
      (and
	(not (equal? obj gstep-nil))
	(gstep-bool ([] obj isKindOfClass: ([] "NSDictionary" class)))
	(gstep-bool ([] obj isEqual: dict))
      )
    )
  )

)
