
;   general.scm - Tests of calling Objective-C methods from Guile
;   Copyright (C) 2000 Free SoftwareFoundation, Inc.
;
;   Written by: Adam Fedor <fedor@gnu.org>
;   Date: Dec 2000
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

(begin

  (define mgr gstep-nil)
  (define dir gstep-nil)
  (define str gstep-nil)
  (define str1 gstep-nil)

  (greg-testcase "NSFileManager understands [+defaultManager]" #t
    (lambda ()
      (set! mgr ([] "NSFileManager" defaultManager))
      (not (equal? mgr gstep-nil))
    )
  )

  ;
  ; Dir operations
  ;
  (greg-testcase "NSFileManager can create a directory" #t
    (lambda ()
      (set! dir ($$ "NSFileManagerTestDir"))
      (gstep-bool ([] mgr createDirectoryAtPath: dir
		                     attributes: gstep-nil))
    )
  )

  (greg-testcase "NSFileManager can change directories" #t
    (lambda ()
      (gstep-bool ([] mgr changeCurrentDirectoryPath: dir))
    )
  )

  (greg-testcase "NSFileManager can get current dir" #t
    (lambda ()
      (define dir1 ([] mgr currentDirectoryPath))
      (and
        (not (equal? dir1 gstep-nil))
	(gstep-bool ([] ([] dir1 lastPathComponent) isEqual: dir))
      )
    )
  )

  ;
  ; File Operations
  ;
  (greg-testcase "NSFileManager creates a file" #t
    (lambda ()
      (set! str ($$ "A String\n"))
      (gstep-bool ([] mgr createFileAtPath: ($$"NSFMFile")
		     contents: ([] str dataUsingEncoding: 1)
		     attributes: gstep-nil))
    )
  )

  (greg-testcase "NSFileManager file contents match" #t
    (lambda ()
      (define dat1 ([] mgr contentsAtPath: ($$"NSFMFile")))
      (set! str1 ([] ([] "NSString" alloc) initWithData: dat1 encoding: 1))
      (gstep-bool ([] str1 isEqual: str))
    )
  )

  (greg-testcase "NSFileManager copies a file" #t
    (lambda ()
      (gstep-bool ([] mgr copyPath: ($$"NSFMFile")
		            toPath: ($$"NSFMFile1")
			   handler: gstep-nil))
    )
  )

  (greg-testcase "NSFileManager copied file contents match" #t
    (lambda ()
      (define dat1 ([] mgr contentsAtPath: ($$"NSFMFile1")))
      (set! str1 ([] ([] "NSString" alloc) initWithData: dat1 encoding: 1))
      (gstep-bool ([] str1 isEqual: str))
    )
  )

  (greg-testcase "NSFileManager moves a file" #t
    (lambda ()
      (gstep-bool ([] mgr movePath: ($$"NSFMFile")
		            toPath: ($$"NSFMFile2")
			   handler: gstep-nil))
    )
  )

  (greg-testcase "NSFileManager moved file contents match" #t
    (lambda ()
      (define dat1 ([] mgr contentsAtPath: ($$"NSFMFile2")))
      (set! str1 ([] ([] "NSString" alloc) initWithData: dat1 encoding: 1))
      (gstep-bool ([] str1 isEqual: str))
    )
  )

  ;(greg-testcase "NSFileManager links a file" #f
  ;  (lambda ()
  ;    (gstep-bool ([] mgr linkPath: ($$"NSFMFile2")
	;	            toPath: ($$"NSFMFile3")
	;		   handler: gstep-nil))
  ;  )
  ;)

  (greg-testcase "NSFileManager creates a symbolic link" #t
    (lambda ()
      (gstep-bool ([] mgr createSymbolicLinkAtPath: ($$"NSFMFile3")
				       pathContent: ($$"NSFMFile2")))
    )
  )

  (greg-testcase "NSFileManager removes a symbolic link" #t
    (lambda ()
      (gstep-bool ([] mgr removeFileAtPath: ($$"NSFMFile3")
				   handler: gstep-nil))
    )
  )

  (greg-testcase "NSFileManager removed link does not exist" #t
    (lambda ()
      (not (gstep-bool ([] mgr fileExistsAtPath: ($$"NSFMFile3"))))
    )
  )

  (greg-testcase "NSFileManager removed link target still exists" #t
    (lambda ()
      (gstep-bool ([] mgr fileExistsAtPath: ($$"NSFMFile1")))
    )
  )

  (greg-testcase "NSFileManager removes a file" #t
    (lambda ()
      (gstep-bool ([] mgr removeFileAtPath: ($$"NSFMFile1")
	  	                   handler: gstep-nil))
    )
  )

  ; 
  ; File Access
  ;
  (greg-testcase "NSFileManager fileExistsAtPath works" #t
    (lambda ()
      (gstep-bool ([] mgr fileExistsAtPath: ($$"NSFMFile2")))
    )
  )

  (greg-testcase "NSFileManager removed file does not exist" #t
    (lambda ()
      (not (gstep-bool ([] mgr fileExistsAtPath: ($$"NSFMFile1"))))
    )
  )

  (greg-testcase "NSFileManager isReadableFileAtPath: works" #t
    (lambda ()
      (gstep-bool ([] mgr isReadableFileAtPath: ($$"NSFMFile2")))
    )
  )

  (greg-testcase "NSFileManager isWritableFileAtPath: works" #t
    (lambda ()
      (gstep-bool ([] mgr isWritableFileAtPath: ($$"NSFMFile2")))
    )
  )

  (greg-testcase "NSFileManager isDeletableFileAtPath: works" #t
    (lambda ()
      (gstep-bool ([] mgr isDeletableFileAtPath: ($$"NSFMFile2")))
    )
  )

   (greg-testcase "NSFileManager isExecutableFileAtPath: works" #t
    (lambda ()
      (not (gstep-bool ([] mgr isExecutableFileAtPath: ($$"NSFMFile2"))))
    )
  )


  (test-exception "[mgr removeFileAtPath: @\".\" handler: nil]"
        NSInvalidArgumentException
    (lambda ()
      ([] mgr removeFileAtPath: ($$ ".") handler: gstep-nil)
      #f
    )
  )

  (greg-testcase "NSFileManager can create a subdirectory" #t
    (lambda ()
      (gstep-bool ([] mgr createDirectoryAtPath: ($$ "subdir")
		                     attributes: gstep-nil))
    )
  )

  (greg-testcase "NSFileManager can move into subdir" #t
    (lambda ()
      (gstep-bool ([] mgr changeCurrentDirectoryPath: ($$ "subdir")))
    )
  )

  (test-exception "[mgr removeFileAtPath: @\".\" handler: nil]"
        NSInvalidArgumentException
    (lambda ()
      ([] mgr removeFileAtPath: ($$ ".") handler: gstep-nil)
      #f
    )
  )

  (test-exception "[mgr removeFileAtPath: @\"..\" handler: nil]"
        NSInvalidArgumentException
    (lambda ()
      ([] mgr removeFileAtPath: ($$ "..") handler: gstep-nil)
      #f
    )
  )

  ;
  ; Last Test - Remove dir
  ;
  (greg-testcase "NSFileManager can move up directories" #t
    (lambda ()
      (gstep-bool ([] mgr changeCurrentDirectoryPath: ($$ "../..")))
    )
  )

  (greg-testcase "NSFileManager removes a directory" #t
    (lambda ()
      (gstep-bool ([] mgr removeFileAtPath: dir
	  	                   handler: gstep-nil))
    )
  )


)



