
;   create.scm - Tests of calling Objective-C methods from Guile
;   Copyright (C) 2000 Free SoftwareFoundation, Inc.
;
;   Written by: Adam Fedor <fedor@gnu.org>
;   Date: Dec 2000
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

(begin

  (define bnd gstep-nil)
  (define tar gstep-nil)
  (define pxy gstep-nil)
  (define inv gstep-nil)
  (define ret gstep-nil)
  (define retp voidp-nil)

  ;;
  ;; Load the bundle with our required classes. Check if it's already been
  ;; loaded from a previous test.
  ;;
  (test-load-bundle "NicolaTest" "Resources" "BaseLibrary")


  (greg-testcase "We can build some objects for tests" #t
    (lambda ()
      (set! tar ([] "InvokeTarget" new))
      (set! pxy ([] ([] "InvokeProxy" alloc) initWithTarget: tar))
      #t
    )
  )

  ;
  ; Function to invoke a method
  ;
  (define (test-invoke selx)
    (begin
      
      (greg-testcase (string-append "  Invoke " selx)  #t
	(lambda ()
	  (define sig gstep-nil)
	  (set! sig  ([] tar methodSignatureForSelector: selx))
	  (set! inv  ([] "NSInvocation" invocationWithMethodSignature: sig))
	  ([] inv setSelector: selx)
	  ([] inv invokeWithTarget: tar)
	  #t
        )
      )
    )
  )

  ;
  ; Function to invoke a method with an argument
  ;
  (define (test-invoke-arg selx argp)
    (begin
      
      (greg-testcase (string-append "  Invoke " selx)  #t
	(lambda ()
	  (define sig gstep-nil)
	  (set! sig  ([] tar methodSignatureForSelector: selx))
	  (set! inv  ([] "NSInvocation" invocationWithMethodSignature: sig))
	  ([] inv setSelector: selx)
	  ([] inv setArgument: argp atIndex: 2)
	  ([] inv invokeWithTarget: tar)
	  #t
        )
      )
    )
  )

  (test-invoke "retChar")
  (greg-testcase "Can return chars" #t
    (lambda ()
      (set! retp (list->voidp (list 0) "C"))
      ([] inv getReturnValue: retp)
      (and
        (equal? (list 99) (voidp->list retp "C" 1))
        (equal? 99 ([] pxy retChar))
      )
    )
  )

  (test-invoke "retShort")
  (greg-testcase "Can return shorts" #t
    (lambda ()
      (set! retp (list->voidp (list 0) "S"))
      ([] inv getReturnValue: retp)
      (and
        (equal? (list 12345) (voidp->list retp "S" 1))
        (equal? 12345 ([] pxy retShort))
      )
    )
  )

  (test-invoke "retInt")
  (greg-testcase "Can return ints" #t
    (lambda ()
      (set! retp (list->voidp (list 0) "i"))
      ([] inv getReturnValue: retp)
      (and
        (equal? (list 123456) (voidp->list retp "i" 1))
        (equal? 123456 ([] pxy retInt))
      )
    )
  )

  (test-invoke "retLong")
  (greg-testcase "Can return longs" #t
    (lambda ()
      (set! retp (list->voidp (list 0) "l"))
      ([] inv getReturnValue: retp)
      (and
        (equal? (list 123456) (voidp->list retp "l" 1))
        (equal? 123456 ([] pxy retLong))
      )
    )
  )

  (test-invoke "retFloat")
  (greg-testcase "Can return floats" #t
    (lambda ()
      (set! retp (list->voidp (list 0) "f"))
      ([] inv getReturnValue: retp)
      (set! ret (list-ref (voidp->list retp "f" 1) 0))
      (and
        (>= .001 (abs (- 123.456 ret)))
        (>= .001 (abs (- 123.456 ([] pxy retFloat))))
      )
    )
  )

  (test-invoke "retDouble")
  (greg-testcase "Can return doubles" #t
    (lambda ()
      (set! retp (list->voidp (list 0) "d"))
      ([] inv getReturnValue: retp)
      (set! ret (list-ref (voidp->list retp "d" 1) 0))
      (and
        (>= .001 (abs (- 123.456 ret)))
        (>= .001 (abs (- 123.456 ([] pxy retFloat))))
      )
    )
  )

  (test-invoke "retObject")
  (greg-testcase "Can return objects" #t
    (lambda ()
      (set! retp (list->voidp (list gstep-nil) "@"))
      ([] inv getReturnValue: retp)
      (set! ret (list-ref (voidp->list retp "@" 1) 0))
      (and
        (equal? tar ret)
        (equal? tar ([] pxy retObject))
      )
    )
  )

  (test-invoke-arg "loopChar:" (list->voidp (list 1) "C"))
  (greg-testcase "Can send/return chars" #t
    (lambda ()
      (set! retp (list->voidp (list 0) "C"))
      ([] inv getReturnValue: retp)
      (set! ret (list-ref (voidp->list retp "C" 1) 0))
      (and
        (equal? 2 ret)
        (equal? 2 ([] pxy loopChar: 1))
      )
    )
  )

  (test-invoke-arg "loopShort:" (list->voidp (list 1) "S"))
  (greg-testcase "Can send/return shorts" #t
    (lambda ()
      (set! retp (list->voidp (list 0) "S"))
      ([] inv getReturnValue: retp)
      (set! ret (list-ref (voidp->list retp "S" 1) 0))
      (and
        (equal? 2 ret)
        (equal? 2 ([] pxy loopShort: 1))
      )
    )
  )

  (test-invoke-arg "loopInt:" (list->voidp (list 1) "i"))
  (greg-testcase "Can send/return ints" #t
    (lambda ()
      (set! retp (list->voidp (list 0) "i"))
      ([] inv getReturnValue: retp)
      (set! ret (list-ref (voidp->list retp "i" 1) 0))
      (and
        (equal? 2 ret)
        (equal? 2 ([] pxy loopInt: 1))
      )
    )
  )

  (test-invoke-arg "loopLong:" (list->voidp (list 1) "l"))
  (greg-testcase "Can send/return longs" #t
    (lambda ()
      (set! retp (list->voidp (list 0) "l"))
      ([] inv getReturnValue: retp)
      (set! ret (list-ref (voidp->list retp "l" 1) 0))
      (and
        (equal? 2 ret)
        (equal? 2 ([] pxy loopLong: 1))
      )
    )
  )

  (test-invoke-arg "loopFloat:" (list->voidp (list 1) "f"))
  (greg-testcase "Can send/return floats" #t
    (lambda ()
      (set! retp (list->voidp (list 0) "f"))
      ([] inv getReturnValue: retp)
      (set! ret (list-ref (voidp->list retp "f" 1) 0))
      (and
        (>= .001 (abs (- 2 ret)))
        (>= .001 (abs (- 2 ([] pxy loopFloat: 1))))
      )
    )
  )

  (test-invoke-arg "loopDouble:" (list->voidp (list 1) "d"))
  (greg-testcase "Can send/return doubles" #t
    (lambda ()
      (set! retp (list->voidp (list 0) "d"))
      ([] inv getReturnValue: retp)
      (set! ret (list-ref (voidp->list retp "d" 1) 0))
      (and
        (>= .001 (abs (- 2 ret)))
        (>= .001 (abs (- 2 ([] pxy loopDouble: 1))))
      )
    )
  )

  (test-invoke-arg "loopObject:" (list->voidp (list pxy) "@"))
  (greg-testcase "Can send/return objects" #t
    (lambda ()
      (set! retp (list->voidp (list gstep-nil) "@"))
      ([] inv getReturnValue: retp)
      (set! ret (list-ref (voidp->list retp "@" 1) 0))
      (and
        (equal? pxy ret)
        (equal? pxy ([] pxy loopObject: pxy))
      )
    )
  )

  (test-invoke-arg "loopString:" (list->voidp (list "Hello There") "*"))
  (greg-testcase "Can send/return strings" #t
    (lambda ()
      (set! retp (list->voidp (list "") "*"))
      ([] inv getReturnValue: retp)
      (set! ret (list-ref (voidp->list retp "*" 1) 0))
      (and
        (equal? "Iello There" ret)
        (equal? "Iello There" ([] pxy loopString: "Hello There"))
      )
    )
  )

  (greg-testcase "Can send/return small structs" #t
    (lambda ()
      (equal? (list 7 10) ([] pxy loopSmall: (list 7 9)))
    )
  )

  (greg-testcase "Can send/return large structs" #t
    (lambda ()
      (equal? (list 9 "Hello" 11.0) ([] pxy loopLarge: (list 8 "Hello" 11.0)))
    )
  )


)




