
; test01.scm - Tests of NSInvocation with object arguments and return value
; Copyright (C) 2003 Free Software Foundation, Inc.
;
; Written by: Wim Oudshoorn <woudshoo@xs4all.nl>
; Date: January 2003
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


; Test if the return value is retained.  It is in the Apple OpenStep 
; edition for Windows (YellowBox)

(greg-testcase
 "Retain return value" #t
 (lambda () 
   (let* ((tar ([] "InvokeTarget" new))
	  (sig ([] tar methodSignatureForSelector: "retObject"))
	  (inv ([] "NSInvocation" invocationWithMethodSignature: sig))
	  (rtc ([] ([] tar retObject) retainCount)))
     ([] inv setSelector: "retObject")
     ([] inv invokeWithTarget: tar)
     (equal? (+ 1 rtc) ([] ([] tar retObject) retainCount)))))

;
; Test if the new arguments are retained when the flag retain
; Arguments is set.
;
(greg-testcase
 "Will retain arguments after retainArguments" #t
 (lambda () 
   (let* ((tar ([] "InvokeTarget" new))
	  (sig ([] tar methodSignatureForSelector: "loopObject:"))
	  (inv ([] "NSInvocation" invocationWithMethodSignature: sig))
	  (tarp (list->voidp (list tar) "@"))
	  (rtc ([] tar retainCount)))
     ([] inv setSelector: "loopObject:")
     ([] inv retainArguments)
     ([] inv setArgument: tarp atIndex: 2)
     (equal? (+ 1 rtc) ([] tar retainCount)))))

;
; Check that the default is not retaining the arguments.
;
(greg-testcase
 "Default not retain arguments" #t
 (lambda () 
   (let* ((tar ([] "InvokeTarget" new))
	  (sig ([] tar methodSignatureForSelector: "loopObject:"))
	  (inv ([] "NSInvocation" invocationWithMethodSignature: sig))
	  (tarp (list->voidp (list tar) "@"))
	  (rtc ([] tar retainCount)))
     ([] inv setSelector: "loopObject:")
     ([] inv setArgument: tarp atIndex: 2)
     (equal? rtc ([] tar retainCount)))))


;
; Check that nil target returns indeed nil object
;
(greg-testcase
 "Check if nil target works" #t
 (lambda ()
   (let* ((tar ([] "InvokeTarget" new))
	  (sig ([] tar methodSignatureForSelector: "retObject"))
	  (inv ([] "NSInvocation" invocationWithMethodSignature: sig))
	  (retp (list->voidp (list gstep-nil) "@")))
     ([] inv setSelector: "retObject")
     ([] inv invokeWithTarget: gstep-nil)
     (catch #t
	    (lambda ()
	      ([] inv getReturnValue: retp)
	      (equal? (list-ref (voidp->list retp "@" 1) 0) gstep-nil))
	    (lambda (key . args)
	      #f)))))

; Exceptions
;
; Check the following:
;  1 - Exceptions thrown in the invoked method are passed through.
;  2 - Fetching a return value afterwards also results in a exception.
;
(greg-testcase
 "Exception in invocation 1" #t
 (lambda ()
   (let* ((tar ([] "InvokeTarget" new))
	  (sig ([] tar methodSignatureForSelector: "returnIdButThrowException"))
	  (inv ([] "NSInvocation" invocationWithMethodSignature: sig))
	  (retp (list->voidp (list gstep-nil) "@")))
     ([] inv setSelector: "returnIdButThrowException")
     (catch #t
	    (lambda ()
	      ([] inv invokeWithTarget: tar)
	      (greg-dlog "Should have gotten an exception when invoke is called, but got none")
	      #f)  
	    (lambda (key . args)
	      (catch #t
		     (lambda ()
		       ([] inv  getReturnValue: retp)
		       (greg-dlog "Should have gotten an exception when returnvalue is retrieved, but got none")
		       #f)
		     (lambda (key . args)
		       #t)))))))

;
; Exceptions
;
; Basically the same as testcase "Exception in invocation 1" 
; But now we first do a successfull call and than try with the same
; invocation to do what the previous testcase does.

(greg-testcase
 "Exception in invocation 2" #t
 (lambda ()
   (let* ((tar ([] "InvokeTarget" new))
	  (sig ([] tar methodSignatureForSelector: "returnIdButThrowException"))
	  (inv ([] "NSInvocation" invocationWithMethodSignature: sig))
	  (retp (list->voidp (list gstep-nil) "@")))
     ([] inv setSelector: "retObject")
     ([] inv invokeWithTarget: tar)
     ([] inv  getReturnValue: retp)
     ([] inv setSelector: "returnIdButThrowException")
     (catch #t
	    (lambda ()
	      ([] inv invokeWithTarget: tar)
	      (greg-dlog "Should have gotten an exception when invoke is called, but got none")
	      #f)  
	    (lambda (key . args)
	      (catch #t
		     (lambda ()
		       ([] inv  getReturnValue: retp)
		       (greg-dlog "Should have gotten an exception when returnvalue is retrieved, but got none")
		       #f)
		     (lambda (key . args)
		       #t)))))))


	  