
;   create.scm - Creation tests for NSMutableArray
;   Copyright (C) 2000 Free SoftwareFoundation, Inc.
;
;   Written by:  Adam Fedor <fedor@gnu.org>
;   Date: Nov 2000
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

;
;	Do everything inside a 'begin' procedure so that any variables
;	defined will have scope limited to this file.
;
(begin

  ;
  ;	Define variables that we are going to use to contain values between
  ;	individual tests - they need to be defined here so they are in the
  ;	scope enclosing all the tests.
  ;
  (define val1 gstep-nil)
  (define val2 gstep-nil)
  (define val3 gstep-nil)
  (define vals1 gstep-nil)
  (define vals2 gstep-nil)
  (define vals3 gstep-nil)
  (define ptrkey1 voidp-nil)
  (define ptrkeys voidp-nil)
  (define ptrval1 voidp-nil)
  (define ptrvals voidp-nil)

  ;
  ;	Now initialise those variables inside a testcase so that, in the
  ;	event of a major error in intialisation, we get an unresolved test.
  ;
  (greg-testcase "We can build basic strings  and arrays for tests" #t
    (lambda ()
      ;
      ;	Three strings to use as array items
      ;
      (set! val1 ([] "NSString" stringWithCString: "Hello"))
      (set! val2 ([] "NSString" stringWithCString: "Goodbye"))
      (set! val3 ([] "NSString" stringWithCString: "Testing"))

      ;
      ;	A pointer to a value.
      ;
      (set! ptrval1 (list->voidp (list val1) "@"))

      ;
      ;	A pointer to a 'C' array of values.
      ;
      (set! ptrvals (list->voidp (list val1 val2 val3) "@"))
      #t
    )
  )

  ;
  ;	Simple test for the '+new' method - we invoke the method, then we
  ;	check that the result is not a nil object, it is of the correct
  ;	class, and that it is a dictionary with zero keys.
  ;
  (greg-testcase "+arrayWithCapacity creates an empty array" #t
    (lambda ()
      (define obj ([] "NSMutableArray" arrayWithCapacity: 10))
      (and
	(not (equal? obj gstep-nil))
	(gstep-bool ([] obj isKindOfClass: ([] "NSMutableArray" class)))
	(eq? 0 ([] obj count))
      )
    )
  )

  (greg-testcase "+array creates an empty array" #t
    (lambda ()
      (define obj ([] "NSArray" array))
      (and
	(not (equal? obj gstep-nil))
	(gstep-bool ([] obj isKindOfClass: ([] "NSArray" class)))
	(eq? 0 ([] obj count))
      )
    )
  )

  ;
  ;	Now we test that exceptions are generated as they should be - 
  ;	use a nil object when trying to create a array.
  ;
  (test-exception "+arrayWithObject: with nil object"
	NSInvalidArgumentException
    (lambda ()
      (define obj ([] "NSArray" arrayWithObject: gstep-nil))
      #f
    )
  )

  (greg-testcase "+arrayWithObject: builds minimal array" #t
    (lambda ()
      (define obj ([] "NSArray" arrayWithObject: val1))
      (and
	(not (equal? obj gstep-nil))
	(gstep-bool ([] obj isKindOfClass: ([] "NSArray" class)))
	(eq? 1 ([] obj count))
      )
    )
  )

  (define old gstep-nil)
  (greg-testcase "+arrayWithObjects: builds an array" #t
    (lambda ()
      (define obj ([] "NSArray" arrayWithObjects: ptrvals
						     count: 3))
      (set! old obj)
      (and
	(not (equal? obj gstep-nil))
	(gstep-bool ([] obj isKindOfClass: ([] "NSArray" class)))
	(eq? 3 ([] obj count))
      )
    )
  )

  (greg-testcase "+arrayWithArray: copies array" #t
    (lambda ()
      (define obj ([] "NSArray" arrayWithArray: old))
      (and
	(not (equal? obj gstep-nil))
	(gstep-bool ([] old isEqual: obj))
      )
    )
  )

)
