;   general.scm - General tests for NSMutableArray
;   Copyright (C) 2000 Free SoftwareFoundation, Inc.
;
;   Written by:  Adam Fedor <fedor@gnu.org>
;   Date: Nov 2000
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

(begin

  (define val1 gstep-nil)
  (define val2 gstep-nil)
  (define val3 gstep-nil)
  (define vals1 gstep-nil)
  (define vals2 gstep-nil)
  (define vals3 gstep-nil)

  (greg-testcase "We can build basic strings and arrays for tests" #t
    (lambda ()
      (set! val1 ([] "NSString" stringWithCString: "Hello"))
      (set! val2 ([] "NSString" stringWithCString: "A Goodbye"))
      (set! val3 ([] "NSString" stringWithCString: "Testing All Strings"))
      (set! vals1 ([] ([] "NSArray" arrayWithObject: val1)
			arrayByAddingObject: val2))
      (set! vals2 ([] vals1 arrayByAddingObject: val2))
      (set! vals3 ([] vals1 arrayByAddingObject: val3))
      #t
    )
  )

  ;
  ;	Perform tests on an empty array.
  ;
  (define arr gstep-nil)
  (greg-testcase "-count returns zero for an empty array" #t
    (lambda ()
      (define obj ([] "NSArray" new))
      (set! arr obj)
      (and
	(not (equal? obj gstep-nil))
	(gstep-bool ([] obj isKindOfClass: ([] "NSArray" class)))
	(eq? 0 ([] obj count))
      )
    )
  )

  (greg-testcase "-hash returns zero for an empty array" #t
    (lambda ()
      (eq? 0 ([] arr hash))
    )
  )

  (greg-testcase "-containsObject works" #t
    (lambda ()
      (and
	(not (equal? vals3 gstep-nil))
	(gstep-bool ([] vals3 containsObject: val2))
      )
    )
  )

  (define valo gstep-nil)
  (greg-testcase "-indexOfObject: finds object" #t
    (lambda ()
      (set! valo ([] "NSString" stringWithCString: "A Goodbye"))
      (and
	(not (equal? vals3 gstep-nil))
	(eq? 1 ([] vals3 indexOfObject: valo))
      )
    )
  )

  (greg-testcase "-indexOfObjectIdenticalTo: finds identical object" #t
    (lambda ()
      (and
	(not (equal? vals3 gstep-nil))
	(eq? 1 ([] vals3 indexOfObjectIdenticalTo: val2))
      )
    )
  )

  (greg-testcase "-objectEnumerator: is ok for empty array" #t
    (lambda ()
      (let*
	(
	  (e ([] arr objectEnumerator))
	  (v1 ([] e nextObject))
	  (v2 ([] e nextObject))
	)
	(and
	  (not (equal? e gstep-nil))
	  (equal? v1 gstep-nil)
	  (equal? v2 gstep-nil)
	)
      )
    )
  )

  (greg-testcase "-description gives us a text property-list" #t
    (lambda ()
      (let*
	(
	  (obj ([] arr description))
	)
	(begin
	  (set! obj ([] obj propertyList))
	  (and
	    (not (equal? obj gstep-nil))
	    (gstep-bool ([] obj isKindOfClass: ([] "NSArray" class)))
	    (eq? 0 ([] obj count))
	  )
	)
      )
    )
  )



  ;
  ;	Perform tests on a array with contents.
  ;

  (greg-testcase "-count returns two for an array with two objects" #t
    (lambda ()
      (let*
	(
	  (obj ([] "NSArray" arrayWithArray: vals1))
	)
	(begin
	  (set! arr obj)
	  (and
	    (not (equal? obj gstep-nil))
	    (gstep-bool ([] obj isKindOfClass: ([] "NSArray" class)))
	    (eq? 2 ([] obj count))
	  )
	)
      )
    )
  )

  (greg-testcase "-hash returns two for an array with two objects" #t
    (lambda ()
      (eq? 2 ([] arr hash))
    )
  )

  (greg-testcase "-indexOfObject: gives NSNotFound for a nil object" #t
    (lambda ()
      (equal? NSNotFound ([] arr indexOfObject: gstep-nil))
    )
  )

  (greg-testcase "-indexOfObject: gives NSNotFound for a object not in the array" #t
    (lambda ()
      (equal? NSNotFound ([] arr indexOfObject: val3))
    )
  )

  (greg-testcase "Array is equal to itself using -isEqualToArray:" #t
    (lambda ()
      (gstep-bool ([] vals1 isEqualToArray: vals1))
    )
  )

  (greg-testcase "Similar arrays are not equal using -isEqualToArray:" #t
    (lambda ()
      (not (gstep-bool ([] vals1 isEqualToArray: vals2)))
    )
  )

  (define a gstep-nil)
  (greg-testcase "-subarrayWithRange: seems ok" #t
    (lambda ()
      (define r (NSMakeRange 0 2))
      (set! a ([] arr subarrayWithRange: r))
      (and
	(not (equal? a gstep-nil))
	(gstep-bool ([] a isKindOfClass: ([] "NSArray" class)))
	(eq? 2 ([] a count))
	(equal? ([] a objectAtIndex: 0) val1)
	(equal? ([] a objectAtIndex: 1) val2)
      )
    )
  )

  (test-exception "-subarrayWithRange: with invalid range"
	NSRangeException
    (lambda ()
      (define r (NSMakeRange 1 2))
      (set! a ([] arr subarrayWithRange: r))
      #f
    )
  )

  (define c gstep-nil)
  (define s gstep-nil)
  (greg-testcase "-componentsJoinedByString: seems ok" #t
    (lambda ()
      (set! c ([] "NSString" stringWithCString: "/"))
      (set! s ([] "NSString" stringWithCString: "Hello/A Goodbye"))
      (set! a ([] arr componentsJoinedByString: c))
      (and
	(not (equal? a gstep-nil))
	(gstep-bool ([] a isKindOfClass: ([] "NSString" class)))
	(gstep-bool ([] a isEqual: s))
      )
    )
  )

  (greg-testcase "-objectEnumerator: enumerates the array" #t
    (lambda ()
      (let*
	(
	  (e ([] arr objectEnumerator))
	  (v1 ([] e nextObject))
	  (v2 ([] e nextObject))
	  (v3 ([] e nextObject))
	)
	(and
	  (not (equal? v1 gstep-nil))
	  (not (equal? v2 gstep-nil))
	  (equal? v3 gstep-nil)
	  (not (equal? v1 v2))
	  (gstep-bool ([] vals1 containsObject: v1))
	  (gstep-bool ([] vals1 containsObject: v2))
	)
      )
    )
  )

  (greg-testcase "-sortedArrayUsingSelector: seems ok" #t
    (lambda ()
      (define a ([] arr sortedArrayUsingSelector: "compare:"))
      (and
	(not (equal? a gstep-nil))
	(gstep-bool ([] a isKindOfClass: ([] "NSArray" class)))
	(eq? 2 ([] a count))
	(equal? ([] a objectAtIndex: 0) val2)
	(equal? ([] a objectAtIndex: 1) val1)
      )
    )
  )

  (greg-testcase "-description gives us a text property-list" #t
    (lambda ()
      (define obj ([] arr description))
      (set! obj ([] obj propertyList))
      (and
	(not (equal? obj gstep-nil))
	(gstep-bool ([] obj isKindOfClass: ([] "NSArray" class)))
	(gstep-bool ([] obj isEqual: arr))
      )
    )
  )

)


