;   test00.scm - Test setting simple attributes for NSMutableAttributedString
;   Copyright (C) 2000 Free SoftwareFoundation, Inc.
;
;   Written by:  Nicola Pero <n.pero@mi.flashnet.it>
;   Date: Dec 2000
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

;;
;;	Do everything inside a 'begin' procedure so that any variables
;;	defined will have scope limited to this file.
;;
(begin

  ;; Check that the attributes at `location' of the NSAttributedString
  ;; `string' are `attributes'
  (define check-attributes-in-location 
    (lambda (string location attributes)
      (gstep-bool ([] ([] string attributesAtIndex: location 
		       effectiveRange: (voidp-nil))
		   isEqual: attributes))))

  ;; Check that the attributes in the range `range' of the NSAttributedString
  ;; `string' are all `attributes'
  (define check-attributes-in-range 
    (lambda (string range attributes)
      (let ((location (NSRange-location range))
	    (length (NSRange-length range)))
	(cond 
	 ((> length 0) 
	  (and 
	   (check-attributes-in-location string (+ location (- length 1))
					 attributes)
	   (check-attributes-in-range string 
				      (NSMakeRange location (- length 1)) 
				      attributes)))
	 (else #t)))))


  ;; Our base test string
  (define base-string ($$ "0123456789"))

  ;; The fake attributes we use
  (define red ([] "NSDictionary" 
	       dictionaryWithObject: ($$ "Red") 
	       forKey: ($$ "Color")))

  (define gray ([] "NSDictionary" 
		dictionaryWithObject: ($$ "Gray") 
		forKey: ($$ "Color")))

  (define blue ([] "NSDictionary" 
		dictionaryWithObject: ($$ "Blue") 
		forKey: ($$ "Color")))
  
  ;; This function returns a new attributed string to test - all red
  (define attr-obj (lambda () 
		     ([] ([] "NSMutableAttributedString" alloc) 
		      initWithString: base-string attributes: red)))

  ;; Basic Test 
  (greg-testcase "-initWithString:attributes: works" #t
   (lambda () 
     (let ((attr-string (attr-obj)))
       (and
	(gstep-bool ([] ([] attr-string string) isEqual: base-string))
	(check-attributes-in-range attr-string (NSMakeRange 0 10) red)
	))))

  ;;  
  ;; Now the long list of tests, setting attributes in attributed
  ;; strings and then checking the result
  ;;

  (greg-testcase 
    "-setAttributes:range: works for the whole string" #t
    (lambda ()
     (let ((attr-string (attr-obj)))
       ([] attr-string setAttributes: blue range: (NSMakeRange 0 10))
       (check-attributes-in-range attr-string (NSMakeRange 0 10) blue)
       )))

  (greg-testcase 
   "-setAttributes:range: works for the first half of the string" #t
   (lambda ()
     (let ((attr-string (attr-obj)))
       ([] attr-string setAttributes: blue range: (NSMakeRange 0 5))
       (and
	(check-attributes-in-range attr-string (NSMakeRange 0 5) blue)
	(check-attributes-in-range attr-string (NSMakeRange 5 5) red)
	))))

  (greg-testcase 
   "-setAttributes:range: works in the middle of the string" #t
   (lambda ()
     (let ((attr-string (attr-obj)))
       ([] attr-string setAttributes: blue range: (NSMakeRange 3 5))
       (and
	(check-attributes-in-range attr-string (NSMakeRange 0 3) red)
	(check-attributes-in-range attr-string (NSMakeRange 3 5) blue)
	(check-attributes-in-range attr-string (NSMakeRange 8 2) red)
	))))

  (greg-testcase 
   "-setAttributes:range: works at the end of the string" #t
   (lambda ()
     (let ((attr-string (attr-obj)))
       ([] attr-string setAttributes: blue range: (NSMakeRange 5 5))
       (and
	(check-attributes-in-range attr-string (NSMakeRange 0 5) red)
	(check-attributes-in-range attr-string (NSMakeRange 5 5) blue)
	))))

  (greg-testcase 
   "-setAttributes:range: works in three parts of the string" #t
   (lambda ()
     (let ((attr-string (attr-obj)))
       ([] attr-string setAttributes: blue range: (NSMakeRange 0 3))
       ([] attr-string setAttributes: red range: (NSMakeRange 3 4))
       ([] attr-string setAttributes: gray range: (NSMakeRange 7 3))
       (and
	(check-attributes-in-range attr-string (NSMakeRange 0 3) blue)
	(check-attributes-in-range attr-string (NSMakeRange 3 4) red)
	(check-attributes-in-range attr-string (NSMakeRange 7 3) gray)
	))))

  (greg-testcase 
   "-setAttributes:range: works with overlapping" #t
   (lambda ()
     (let ((attr-string (attr-obj)))
       ([] attr-string setAttributes: blue range: (NSMakeRange 0 5))
       ([] attr-string setAttributes: red range: (NSMakeRange 3 5))
       ([] attr-string setAttributes: gray range: (NSMakeRange 4 5))
       (and
	(check-attributes-in-range attr-string (NSMakeRange 0 3) blue)
	(check-attributes-in-range attr-string (NSMakeRange 3 1) red)
	(check-attributes-in-range attr-string (NSMakeRange 4 5) gray)
	(check-attributes-in-range attr-string (NSMakeRange 9 1) red)
	))))

  (greg-testcase 
   "-setAttributes:range: works with overlapping (2)" #t
   (lambda ()
     (let ((attr-string (attr-obj)))
       ([] attr-string setAttributes: blue range: (NSMakeRange 1 2))
       ([] attr-string setAttributes: blue range: (NSMakeRange 4 2))
       ([] attr-string setAttributes: blue range: (NSMakeRange 7 2))
       ([] attr-string setAttributes: gray range: (NSMakeRange 2 6))
       (and
	(check-attributes-in-range attr-string (NSMakeRange 0 1) red)
	(check-attributes-in-range attr-string (NSMakeRange 1 1) blue)
	(check-attributes-in-range attr-string (NSMakeRange 2 6) gray)
	(check-attributes-in-range attr-string (NSMakeRange 8 1) blue)
	(check-attributes-in-range attr-string (NSMakeRange 9 1) red)
	))))

  (greg-testcase 
   "-setAttributes:range: works with replacing" #t
   (lambda ()
     (let ((attr-string (attr-obj)))
       ([] attr-string setAttributes: blue range: (NSMakeRange 2 5))
       ([] attr-string setAttributes: gray range: (NSMakeRange 2 5))
       (and
	(check-attributes-in-range attr-string (NSMakeRange 0 2) red)
	(check-attributes-in-range attr-string (NSMakeRange 2 5) gray)
	(check-attributes-in-range attr-string (NSMakeRange 7 3) red)
	))))

  (greg-testcase 
   "-setAttributes:range: works with chinese boxes" #t
   (lambda ()
     (let ((attr-string (attr-obj)))
       ([] attr-string setAttributes: blue range: (NSMakeRange 1 8))
       ([] attr-string setAttributes: red range: (NSMakeRange 2 6))
       ([] attr-string setAttributes: gray range: (NSMakeRange 3 4))
       (and
	(check-attributes-in-range attr-string (NSMakeRange 0 1) red)
	(check-attributes-in-range attr-string (NSMakeRange 1 1) blue)
	(check-attributes-in-range attr-string (NSMakeRange 2 1) red)
	(check-attributes-in-range attr-string (NSMakeRange 3 4) gray)
	(check-attributes-in-range attr-string (NSMakeRange 7 1) red)
	(check-attributes-in-range attr-string (NSMakeRange 8 1) blue)
	(check-attributes-in-range attr-string (NSMakeRange 9 1) red)
	))))

  (greg-testcase 
   "-setAttributes:range: works with extending at the end (diff color)"
   #t
   (lambda ()
     (let ((attr-string (attr-obj)))
       ([] attr-string setAttributes: blue range: (NSMakeRange 1 3))
       ([] attr-string setAttributes: gray range: (NSMakeRange 1 4))
       (and
	(check-attributes-in-range attr-string (NSMakeRange 0 1) red)
	(check-attributes-in-range attr-string (NSMakeRange 1 4) gray)
	(check-attributes-in-range attr-string (NSMakeRange 5 5) red)
	))))

  (greg-testcase 
   "-setAttributes:range: works with extending at the end (same color)"
   #t
   (lambda ()
     (let ((attr-string (attr-obj)))
       ([] attr-string setAttributes: gray range: (NSMakeRange 1 3))
       ([] attr-string setAttributes: gray range: (NSMakeRange 1 4))
       (and
	(check-attributes-in-range attr-string (NSMakeRange 0 1) red)
	(check-attributes-in-range attr-string (NSMakeRange 1 4) gray)
	(check-attributes-in-range attr-string (NSMakeRange 5 5) red)
	))))

  (greg-testcase 
   "-setAttributes:range: works with extending at the beginning (diff color)"
   #t
   (lambda ()
     (let ((attr-string (attr-obj)))
       ([] attr-string setAttributes: blue range: (NSMakeRange 2 3))
       ([] attr-string setAttributes: gray range: (NSMakeRange 1 4))
       (and
	(check-attributes-in-range attr-string (NSMakeRange 0 1) red)
	(check-attributes-in-range attr-string (NSMakeRange 1 4) gray)
	(check-attributes-in-range attr-string (NSMakeRange 5 5) red)
	))))

  (greg-testcase 
   "-setAttributes:range: works with extending at the beginning (same color)"
   #t
   (lambda ()
     (let ((attr-string (attr-obj)))
       ([] attr-string setAttributes: gray range: (NSMakeRange 2 3))
       ([] attr-string setAttributes: gray range: (NSMakeRange 1 4))
       (and
	(check-attributes-in-range attr-string (NSMakeRange 0 1) red)
	(check-attributes-in-range attr-string (NSMakeRange 1 4) gray)
	(check-attributes-in-range attr-string (NSMakeRange 5 5) red)
	))))

  (greg-testcase 
   "-setAttributes:range: works with subset at the end (diff color)"
   #t
   (lambda ()
     (let ((attr-string (attr-obj)))
       ([] attr-string setAttributes: blue range: (NSMakeRange 1 3))
       ([] attr-string setAttributes: gray range: (NSMakeRange 2 2))
       (and
	(check-attributes-in-range attr-string (NSMakeRange 0 1) red)
	(check-attributes-in-range attr-string (NSMakeRange 1 1) blue)
	(check-attributes-in-range attr-string (NSMakeRange 2 2) gray)
	(check-attributes-in-range attr-string (NSMakeRange 4 6) red)
	))))

  (greg-testcase 
   "-setAttributes:range: works with subset at the end (same color)"
   #t
   (lambda ()
     (let ((attr-string (attr-obj)))
       ([] attr-string setAttributes: gray range: (NSMakeRange 1 3))
       ([] attr-string setAttributes: gray range: (NSMakeRange 2 2))
       (and
	(check-attributes-in-range attr-string (NSMakeRange 0 1) red)
	(check-attributes-in-range attr-string (NSMakeRange 1 3) gray)
	(check-attributes-in-range attr-string (NSMakeRange 4 6) red)
	))))

  (greg-testcase 
   "-setAttributes:range: works with subset at the beginning (diff color)"
   #t
   (lambda ()
     (let ((attr-string (attr-obj)))
       ([] attr-string setAttributes: blue range: (NSMakeRange 2 3))
       ([] attr-string setAttributes: gray range: (NSMakeRange 2 2))
       (and
	(check-attributes-in-range attr-string (NSMakeRange 0 2) red)
	(check-attributes-in-range attr-string (NSMakeRange 2 2) gray)
	(check-attributes-in-range attr-string (NSMakeRange 4 1) blue)
	(check-attributes-in-range attr-string (NSMakeRange 5 5) red)
	))))

  (greg-testcase 
   "-setAttributes:range: works with subset at the beginning (same color)"
   #t
   (lambda ()
     (let ((attr-string (attr-obj)))
       ([] attr-string setAttributes: gray range: (NSMakeRange 2 3))
       ([] attr-string setAttributes: gray range: (NSMakeRange 2 2))
       (and
	(check-attributes-in-range attr-string (NSMakeRange 0 2) red)
	(check-attributes-in-range attr-string (NSMakeRange 2 3) gray)
	(check-attributes-in-range attr-string (NSMakeRange 5 5) red)
	))))

  (greg-testcase 
   "-setAttributes:range: works with subsets (diff color)"
   #t
   (lambda ()
     (let ((attr-string (attr-obj)))
       ([] attr-string setAttributes: gray range: (NSMakeRange 2 1))
       ([] attr-string setAttributes: gray range: (NSMakeRange 4 1))
       ([] attr-string setAttributes: blue range: (NSMakeRange 1 5))
       (and
	(check-attributes-in-range attr-string (NSMakeRange 0 1) red)
	(check-attributes-in-range attr-string (NSMakeRange 1 5) blue)
	(check-attributes-in-range attr-string (NSMakeRange 6 4) red)
	))))

  (greg-testcase 
   "-setAttributes:range: works with subsets (same color)"
   #t
   (lambda ()
     (let ((attr-string (attr-obj)))
       ([] attr-string setAttributes: blue range: (NSMakeRange 2 1))
       ([] attr-string setAttributes: blue range: (NSMakeRange 4 1))
       ([] attr-string setAttributes: blue range: (NSMakeRange 1 5))
       (and
	(check-attributes-in-range attr-string (NSMakeRange 0 1) red)
	(check-attributes-in-range attr-string (NSMakeRange 1 5) blue)
	(check-attributes-in-range attr-string (NSMakeRange 6 4) red)
	))))
  
  (greg-testcase 
   "-setAttributes:range: works with setting attributes for the whole string"
   #t
   (lambda ()
     (let ((attr-string (attr-obj)))
       ([] attr-string setAttributes: blue range: (NSMakeRange 2 1))
       ([] attr-string setAttributes: blue range: (NSMakeRange 4 1))
       ([] attr-string setAttributes: blue range: (NSMakeRange 7 2))
       ([] attr-string setAttributes: red range: (NSMakeRange 3 2))
       ([] attr-string setAttributes: gray range: (NSMakeRange 0 10))
       (and
	(check-attributes-in-range attr-string (NSMakeRange 0 10) gray)
	))))

  (greg-testcase 
   "-setAttributes:range: works with nearby attributes"
   #t
   (lambda ()
     (let ((attr-string (attr-obj)))
       ([] attr-string setAttributes: blue range: (NSMakeRange 0 1))
       ([] attr-string setAttributes: blue range: (NSMakeRange 1 1))
       ([] attr-string setAttributes: blue range: (NSMakeRange 2 1))
       (and
	(check-attributes-in-range attr-string (NSMakeRange 0 3) blue)
	(check-attributes-in-range attr-string (NSMakeRange 3 7) red)
	))))

  
)  
  



