; Alexander Malmberg <alexander@malmberg.org>

(begin

  (use-modules (ice-9 format))

  ([] ([] "NSMutableAttributedString" new) _sanity)

  ; from test00.scm
  ;; Check that the attributes at `location' of the NSAttributedString
  ;; `string' are `attributes'
  (define check-attributes-in-location 
    (lambda (string location attributes)
      (gstep-bool ([] ([] string attributesAtIndex: location 
		       effectiveRange: (voidp-nil))
		   isEqual: attributes))))

  ;; Check that the attributes in the range `range' of the NSAttributedString
  ;; `string' are all `attributes'
  (define check-attributes-in-range 
    (lambda (string range attributes)
      (let ((location (NSRange-location range))
	    (length (NSRange-length range)))
	(cond 
	 ((> length 0) 
	  (and 
	   (check-attributes-in-location string (+ location (- length 1))
					 attributes)
	   (check-attributes-in-range string 
				      (NSMakeRange location (- length 1)) 
				      attributes)))
	 (else #t)))))


  (define base-1 ($$ "base-1"))
  (define base-2 ($$ "base-2"))

  (define attr-e ([] "NSDictionary" dictionary))
  (define attr-1 ([] "NSDictionary" dictionaryWithObject: ($$ "a") forKey: ($$ "1")))
  (define attr-2 ([] "NSDictionary" dictionaryWithObject: ($$ "b") forKey: ($$ "2")))

  (greg-testcase "-replaceCharactersInRange: withString: works with zero length string" #t
    (lambda ()
      (let ((as ([] ([] "NSMutableAttributedString" alloc) initWithString: base-1 attributes: gstep-nil)))
        ([] as replaceCharactersInRange: (NSMakeRange 2 2) withString: ($$ ""))
	([] as _sanity)
	(gstep-bool ([] ([] as string) isEqual: ($$ "ba-1")))
        )))

  (greg-testcase "-replaceCharactersInRange:withString: works in middle of string" #t
    (lambda ()
      (let ((as ([] ([] "NSMutableAttributedString" alloc) initWithString: base-1 attributes: gstep-nil)))
        ([] as replaceCharactersInRange: (NSMakeRange 2 2) withString: base-2)
	([] as _sanity)
	(gstep-bool ([] ([] as string) isEqual: ($$ "babase-2-1")))
        )))

  (greg-testcase "-replaceCharactersInRange:withString: at end of string works" #t
    (lambda ()
      (let ((as ([] ([] "NSMutableAttributedString" alloc) initWithString: base-1 attributes: gstep-nil)))
        ([] as replaceCharactersInRange: (NSMakeRange 6 0) withString: base-2)
	([] as _sanity)
	(gstep-bool ([] ([] as string) isEqual: ($$ "base-1base-2")))
        )))

  (greg-testcase "-replaceCharactersInRange:withString: at start of string works" #t
    (lambda ()
      (let ((as ([] ([] "NSMutableAttributedString" alloc) initWithString: base-1 attributes: gstep-nil)))
        ([] as replaceCharactersInRange: (NSMakeRange 0 0) withString: base-2)
	([] as _sanity)
	(gstep-bool ([] ([] as string) isEqual: ($$ "base-2base-1")))
        )))

  (greg-testcase "-replaceCharactersInRange:withString: doesn't keep attributes after all characters have been deleted" #t
    (lambda ()
      (let ((as ([] ([] "NSMutableAttributedString" alloc) initWithString: base-1 attributes: attr-1)))
        ([] as setAttributes: attr-2 range: (NSMakeRange 2 4))
	([] as replaceCharactersInRange: (NSMakeRange 0 6) withString: ($$ ""))
	([] as replaceCharactersInRange: (NSMakeRange 0 0) withString: ($$ "aa"))
	([] as replaceCharactersInRange: (NSMakeRange 2 0) withString: ($$ "bb"))
	([] as _sanity)
	(check-attributes-in-range as (NSMakeRange 0 4) attr-e)
	)))

  (greg-testcase "-replaceCharactersInRange:withString: keeps attributes if entire string is replaced" #t
    (lambda ()
      (let ((as ([] ([] "NSMutableAttributedString" alloc) initWithString: base-1 attributes: attr-1)))
	([] as replaceCharactersInRange: (NSMakeRange 0 6) withString: base-2)
	([] as _sanity)
	(and
	  (gstep-bool ([] ([] as string) isEqual: base-2))
	  (check-attributes-in-range as (NSMakeRange 0 6) attr-1)
	)
	)))

  ; When in doubt, use brute-force.
  ;
  ; This doesn't actually test that the result is strictly correct (though
  ; it checks some parts), but it lets the _sanity checking make sure that
  ; the attributes aren't messed up.
  (do
    ((start 0 (+ start 1)))
    ((eq? start 9))

    (do
      ((length 0 (+ length 1)))
      ((eq? (+ length start) 9))

      (greg-testcase (format #f "attribute/(replaceCharacters... with zero length string) interaction _sanity checks ~S ~S" start length) #t
        (lambda ()

	  (let
	    ((as ([] ([] "NSMutableAttributedString" alloc) initWithString: ($$ "aabbccdd") attributes: attr-2))
	     (remove-all (and (eq? start 0) (eq? length 8)))
	    )

	    ([] as setAttributes: attr-1 range: (NSMakeRange 2 2))
	    ([] as setAttributes: attr-e range: (NSMakeRange 4 2))

	    (let
	     ((a-begin (if remove-all attr-e
	        ([] as attributesAtIndex: (if (eq? start 0) length 0) effectiveRange: (voidp-nil))))
	      (a-end (if remove-all attr-e
	        ([] as attributesAtIndex: (if (eq? (+ start length) 8) (- start 1) 8) effectiveRange: (voidp-nil))))
	     )

	      ([] as replaceCharactersInRange: (NSMakeRange start length) withString: ($$ ""))
	      ([] as _sanity)

	      (and
	        (eq? ([] ([] as string) length) (- 8 length))
	        (check-attributes-in-location as 0 a-begin)
	        (check-attributes-in-location as (- 8 length) a-end)
	        )
	    ))))

      (greg-testcase (format #f "attribute/replaceCharacters... interaction _sanity checks ~S ~S" start length) #t
        (lambda ()

	  (let
	    ((as ([] ([] "NSMutableAttributedString" alloc) initWithString: ($$ "aabbccdd") attributes: attr-2))
	     (replace-all (and (eq? start 0) (eq? length 8)))
	    )

	    ([] as setAttributes: attr-1 range: (NSMakeRange 2 2))
	    ([] as setAttributes: attr-e range: (NSMakeRange 4 2))

	    (let
	     ((a-begin
	        ([] as attributesAtIndex:
		  (cond
		    ((and (eq? length 0) (eq? start 0)) 0)
		    ((and (eq? length 0)) (- start 1))
		    (#t start))
		  effectiveRange: (voidp-nil)))
	     )

	      ([] as replaceCharactersInRange: (NSMakeRange start length) withString: ($$ "foo"))
	      ([] as _sanity)

	      (and
	        (eq? ([] ([] as string) length) (- 11 length))
		(check-attributes-in-range as (NSMakeRange start 3) a-begin)
	        )
	    ))))

      )
    )
)

