
;   general.scm - General tests for NSMutableData
;   Copyright (C) 2000 Free SoftwareFoundation, Inc.
;
;   Written by:  Adam Fedor <fedor@gnu.org>
;   Date: Mar 2001
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

;
;	Do everything inside a 'begin' procedure so that any variables
;	defined will have scope limited to this file.
;
(begin

  ;
  ;	Define variables that we are going to use to contain values between
  ;	individual tests - they need to be defined here so they are in the
  ;	scope enclosing all the tests.
  ;
  (define data1 gstep-nil)
  (define mdata gstep-nil)
  (define byte1 voidp-nil)

  (define str1 "Test string for data classes")
  (define ptr1 (string->voidp str1))
  (define str2 "insert")
  (define ptr2 (string->voidp str2))

  ;
  ; Creating and Initializing
  ;
  (greg-testcase "+dataWithBytes:length: works" #t
    (lambda ()
      (set! data1 ([] "NSData" dataWithBytes: ptr1 length: (voidp-length ptr1)))
      (and
	(not (equal? data1 gstep-nil))
	(gstep-bool ([] data1 isKindOfClass: ([] "NSData" class)))
	(equal? (voidp-length ptr1) ([] data1 length))
      )
    )
  )

  (greg-testcase "+data creates mutable data" #t
    (lambda ()
      (set! mdata ([] "NSMutableData" data))
      (and
	(not (equal? mdata gstep-nil))
	(gstep-bool ([] mdata isKindOfClass: ([] "NSMutableData" class)))
	(equal? 0 ([] mdata length))
      )
    )
  )

  (greg-testcase "-setData: works" #t
    (lambda ()
      ([] mdata setData: data1)
      (and
	(not (equal? mdata gstep-nil))
	(equal? (voidp-length ptr1) ([] mdata length))
      )
    )
  )

  (greg-testcase "-replaceBytesInRange:withBytes suceeds" #t
    (lambda ()
      (define rng (NSMakeRange 22 6))
      ([] mdata replaceBytesInRange: rng withBytes: ptr2)
      #t
    )
  )

  (test-exception "-replaceBytesInRange:withBytes out of range"
        NSRangeException
    (lambda ()
      (define rng (NSMakeRange 29 6))
      ([] mdata replaceBytesInRange: rng withBytes: ptr2)
      #f
    )
  )


)
