
;   test00.scm - Tests of basic number operation
;   Copyright (C) 2000 Free SoftwareFoundation, Inc.
;
;   Written by:  Richard Frith-Macdonald <rfm@gnu.org>
;   Date: March 2000
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


(begin

  (define val1 gstep-nil)
  (define val2 gstep-nil)
  (define val3 gstep-nil)
  (define val4 gstep-nil)
  (define val5 gstep-nil)

  (greg-testcase "We can create a boolean (YES)" #t
    (lambda ()
      (set! val1 ([] "NSNumber" numberWithBool: #t))
      (not (equal? val1 gstep-nil))
    )
  )
  (greg-testcase "We can create a boolean (NO)" #t
    (lambda ()
      (set! val2 ([] "NSNumber" numberWithBool: #f))
      (not (equal? val2 gstep-nil))
    )
  )
  (greg-testcase "A boolean YES is not a NO" #t
    (lambda ()
      (not (gstep-bool ([] val1 isEqual: val2)))
    )
  )
  (greg-testcase "A boolean NO is not a YES" #t
    (lambda ()
      (not (gstep-bool ([] val1 isEqual: val2)))
    )
  )
  (greg-testcase "A boolean YES is a YES" #t
    (lambda ()
      (gstep-bool ([] val1 isEqual: ([] "NSNumber" numberWithBool: #t)))
    )
  )
  (greg-testcase "A boolean NO is a NO" #t
    (lambda ()
      (gstep-bool ([] val2 isEqual: ([] "NSNumber" numberWithBool: #f)))
    )
  )

  (greg-testcase "We can create a char number" #t
    (lambda ()
      (set! val1 ([] "NSNumber" numberWithChar: 99))
      (not (equal? val1 gstep-nil))
    )
  )
  (greg-testcase "We can create a signed char number" #t
    (lambda ()
      (set! val1 ([] "NSNumber" numberWithChar: -1))
      (not (equal? val1 gstep-nil))
    )
  )
  (greg-testcase "We can create an unsigned char number" #t
    (lambda ()
      (set! val2 ([] "NSNumber" numberWithUnsignedChar: 255))
      (not (equal? val2 gstep-nil))
    )
  )
  (greg-testcase "A -1 signed char is not a 255 unsigned char" #t
    (lambda ()
      (not (gstep-bool ([] val1 isEqual: val2)))
    )
  )
  (greg-testcase "A -1 signed char is a 255 signed char" #t
    (lambda ()
      (gstep-bool ([] val1 isEqual: ([] "NSNumber" numberWithChar: 255)))
    )
  )
  (set! val1 ([] "NSNumber" numberWithChar: -100))
  (greg-dlog val1)
  (greg-testcase "A -100 signed char is a -100 signed short" #t
    (lambda ()
      (gstep-bool ([] val1 isEqual: ([] "NSNumber" numberWithShort: -100)))
    )
  )
  (greg-testcase "A -100 signed char is a -100 signed int" #t
    (lambda ()
      (gstep-bool ([] val1 isEqual: ([] "NSNumber" numberWithInt: -100)))
    )
  )
  (greg-testcase "A -100 signed char is a -100 signed long" #t
    (lambda ()
      (gstep-bool ([] val1 isEqual: ([] "NSNumber" numberWithLong: -100)))
    )
  )
  ;
  ; NB long long test skipped because Guile doesn't support long long :-(
  ;
  (greg-testcase "A -100 signed char is a -100 float" #t
    (lambda ()
      (set! val2 ([] "NSNumber" numberWithFloat: -100))
      (greg-dlog val2)
      (gstep-bool ([] val1 isEqual: val2))
    )
  )
  (greg-testcase "A -100 signed char is a -100 double" #t
    (lambda ()
      (set! val2 ([] "NSNumber" numberWithDouble: -100))
      (greg-dlog val2)
      (gstep-bool ([] val1 isEqual: val2))
    )
  )

  (set! val1 ([] "NSNumber" numberWithInt: 127))
  (set! val2 ([] "NSNumber" numberWithInt: 128))
  (greg-testcase "integer numbers - 127 < 128" #t
    (lambda ()
      (equal? NSOrderedDescending ([] val2 compare: val1))
    )
  )
  (greg-testcase "integer numbers - 128 > 127" #t
    (lambda ()
      (equal? NSOrderedAscending ([] val1 compare: val2))
    )
  )

  (set! val1 ([] "NSNumber" numberWithChar: 100))
  (set! val2 ([] "NSNumber" numberWithChar: 200))
  (set! val3 ([] "NSNumber" numberWithInt: 200))
  (greg-testcase "A 200 signed char is not a 200 int" #t
    (lambda ()
      (not (gstep-bool ([] val2 isEqual: val3)))
    )
  )
  (greg-testcase "signed char numbers - 200 < 100" #t
    (lambda ()
      (equal? NSOrderedAscending ([] val2 compare: val1))
    )
  )
  (greg-testcase "signed char numbers - 100 > 200" #t
    (lambda ()
      (equal? NSOrderedDescending ([] val1 compare: val2))
    )
  )
)

