;   test01.scm - Tests of isClass
;   Copyright (C) 2001 Free SoftwareFoundation, Inc.
;
;   Written by: Nicola Pero <nicola@brainstorm.co.uk>
;   Date: July 2001
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

(greg-testcase 
 "-isClass returns YES on a Class" #t
 (lambda ()
   (and 
    (gstep-bool ([] "NSObject" isClass))
    (gstep-bool ([] "NSString" isClass))
    (gstep-bool ([] "NSArray" isClass))
    )))

(greg-testcase 
 "-isClass returns NO on an instance" #t
 (lambda ()
   (and 
    (not (gstep-bool ([] ([] "NSObject" new) isClass)))
    (not (gstep-bool ([] ($$ "this is a string") isClass)))
    (not (gstep-bool ([] ([] "NSArray" new) isClass)))
    )))

(greg-testcase 
 "-isClass returns NO on an instance (special evil test for broken libobjc)" #t
 (lambda ()
   (let ((evil-object gstep-nil))
     (begin
       (gstep-new-class 
	"MyEvilClass" "NSObject"
	;; instance variables (mirroring a Class struct)
	(list (cons "class_" "@")
	      (cons "name_" "*")
	      (cons "version_" "l")
	      (cons "info_" "L"))
	'() '())
       
       (set! evil-object ([] "MyEvilClass" new))
       (gstep-set-ivar evil-object "info_" 1)
       
       ;; and now the test - evil-object should not be a class !
       (not (gstep-bool ([] evil-object isClass)))
       ))))

(greg-testcase 
 "-isKindOfClass: works" #t
 (lambda ()
   (and 
    (gstep-bool 
     ([] ([] "NSObject" new) isKindOfClass: ([] "NSObject" class)))
    (gstep-bool 
     ([] ([] "NSString" new) isKindOfClass: ([] "NSString" class)))
    (not 
     (gstep-bool 
      ([] ([] "NSObject" new) isKindOfClass: ([] "NSString" class))))
    (gstep-bool 
     ([] ([] "NSString" new) isKindOfClass: ([] "NSObject" class)))
    (not 
     (gstep-bool 
      ([] ([] "NSString" new) isKindOfClass: ([] "NSArray" class))))
    (gstep-bool 
     ([] ([] "NSMutableString" new) isKindOfClass: ([] "NSString" class)))
    )))

(greg-testcase 
 "+isKindOfClass: works" #t
 (lambda ()
   (and 
    (gstep-bool ([] "NSObject" isKindOfClass: ([] "NSObject" class)))
    (not (gstep-bool ([] "NSString" isKindOfClass: ([] "NSString" class))))
    (not (gstep-bool ([] "NSObject" isKindOfClass: ([] "NSString" class))))
    (gstep-bool ([] "NSString" isKindOfClass: ([] "NSObject" class)))
    )))
    


