
;   basic.scm - Tests of calling Objective-C methods from Guile
;   Copyright (C) 2001 Free SoftwareFoundation, Inc.
;
;   Written by:  Richard Frith-Macdonald <richard@brainstorm.co.uk>
;   Date: May 2001
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


;
; Basic allocation tests
;
(begin
  (define classname "NSProxy")
  (define theClass gstep-nil)
  (define obj0 gstep-nil)
  (define obj1 gstep-nil)
  (define prefix (string-append "Class '" classname "' "))
  (define all-ok #t)

  (if
    (greg-testcase (string-append prefix "exists") #t
      (lambda ()
	(set! theClass (gstep-lookup-class classname))
	(not (equal? theClass gstep-nil))
      )
    )
    (begin
      (if 
	(greg-testcase (string-append prefix "has working alloc") #t
	  (lambda ()
	    (set! obj0 ([] theClass alloc))
	    (not (equal? obj0 gstep-nil))
	  )
	)
	(set! all-ok
	  (and
	    (test-exception "NSProxy -isKindOfClass raises exception"
	      ($$ "NSInvalidArgumentException")
	      (lambda ()
		(gstep-bool->bool ([] obj0 isKindOfClass: theClass))
	      )
	    )
	    (test-exception "NSProxy -isMemberOfClass raises exception"
	      ($$ "NSInvalidArgumentException")
	      (lambda ()
		(gstep-bool->bool ([] obj0 isMemberOfClass: theClass))
	      )
	    )
	  )
	)
	(set! all-ok #f)
      )

      (if
	(greg-testcase (string-append prefix "has working allocWithZone:") #t
	  (lambda ()
	    (set! obj1 ([] theClass allocWithZone: testZone))
	    (not (equal? obj1 gstep-nil))
	  )
	)
	(set! all-ok
	  (and
	    (greg-testcase (string-append prefix "uses zone for alloc") #t
	      (lambda ()
		(define theZone (NSZoneFromPointer obj1))
		(greg-dlog "Object zone " theZone "\n")
		(greg-dlog "Test zone " testZone "\n")
		(equal? theZone testZone)
	      )
	    )
	    (greg-testcase (string-append prefix "-zone works") #t
	      (lambda ()
		(equal? ([] obj1 zone) testZone)
	      )
	    )
	  )
	)
	(set! all-ok #f)
      )
    )
    (set! all-ok #f)
  )
  all-ok
)


