
;   test00.scm - Tests of calling Objective-C methods from Guile
;   Copyright (C) 2001 Free SoftwareFoundation, Inc.
;
;   Written by:  Richard Frith-Macdonald <richard@brainstorm.co.uk>
;   Date: May 2001
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

;
; Tests specific to this class
;
(define prefix "The Class 'NSProxy' ")
(define theClass (gstep-lookup-class "NSProxy"))

(greg-testcase (string-append prefix "uses +class to return self") #t
(lambda ()
    (equal? theClass ([] "NSProxy" class))
  )
)

(greg-testcase (string-append prefix "implements -isProxy to return YES") #t
(lambda ()
    (gstep-bool ([] ([] "NSProxy" alloc) isProxy))
  )
)

(greg-testcase (string-append prefix "implements -description") #t
(lambda ()
    (not (equal? gstep-nil ([] ([] "NSProxy" alloc) description)))
  )
)

(greg-testcase "Can create an NSProxy subclass" #t
  (lambda ()
    (begin
      (gstep-new-class "MyProxy" "NSProxy"
	(list	; Instance variables
	  (cons "remote" "@")
	)
	(list	; Instance methods
	  ;
	  ; Designated initialiser ... sets the remote object.
	  ;
	  (list "initWithRemote:" "@@:@"
	    (lambda (receiver method args)
	      (begin
		([] receiver setRemote: (car args))
		receiver
	      )
	    )
	  )
	  ;
	  ; Default initialiser ... init sets a default string object.
	  ;
	  (list "init" "@@:"
	    (lambda (receiver method args)
	      (begin
		([] receiver initWithRemote: ($$ "default string value"))
		receiver
	      )
	    )
	  )
	  ;
	  ; Release remote object on deallocation
	  ;
	  (list "dealloc" "v@:"
	    (lambda (receiver method args)
	      ([] (gstep-get-ivar receiver "remote") release)
	      (gstep-sup-send receiver dealloc)
	    )
	  )
	  ;
	  ; description ... override the default implementation to return
	  ; the description of the remote object.
	  ;
	  (list "description" "@@:"
	    (lambda (receiver method args)
	      ([] (gstep-get-ivar receiver "remote") description)
	    )
	  )
	  ;
	  ; return the remote object.
	  ;
	  (list "remote" "@@:"
	    (lambda (receiver method args)
	      (gstep-get-ivar receiver "remote")
	    )
	  )
	  ;
	  ; Set the remote object, performing retain/release as necessary.
	  ;
	  (list "setRemote:" "v@:@"
	    (lambda (receiver method args)
	      ([] (car args) retain)
	      ([] (gstep-get-ivar receiver "remote") release)
	      (gstep-set-ivar receiver "remote" (car args))
	    )
	  )
	  ;
	  ; Core functionality - override methodSignatureForSelector: so we
	  ; can return the information for the remote object if available.
	  ;
	  (list "methodSignatureForSelector:" "@@::"
	    (lambda (receiver method args)
	      (let*
		(
		  (obj (gstep-get-ivar receiver "remote"))
		  (sig ([] obj methodSignatureForSelector: (car args)))
		)
		(begin
		  (if (equal? sig gstep-nil)
		    (set! sig (gstep-sup-send receiver method (car args)))
		  )
		  sig
		)
	      )
	    )
	  )
	  ;
	  ; Core functionality - handle forwarding of messages to the
	  ; remote object.
	  ;
	  (list "forwardInvocation:" "v@:@"
	    (lambda (receiver method args)
	      (let*
		(
		  (obj (gstep-get-ivar receiver "remote"))
		  (inv (car args))
		)
		(begin
		  ([] inv setTarget: obj)
		  ([] inv invoke)
		)
	      )
	    )
	  )
	)
	'()	; Class variables
      )
      #t
    )
  )
)

(define obj gstep-nil)
(define rem ($$ "Remote"))

(greg-testcase "Can create a MyProxy instance" #t
  (lambda ()
    (set! obj ([] ([] "MyProxy" alloc) init))
    (not (equal? obj gstep-nil))
  )
)

(greg-testcase "Can set the remote object for the proxy" #t
  (lambda ()
    ([] obj setRemote: rem)
    (equal? ([] obj remote) rem)
  )
)

(greg-testcase "Can get the length of the remote object" #t
  (lambda ()
    (equal? ([] obj length) ([] rem length))
  )
)


