;   performers.scm - Tests of -performSelector:target:argument:order:modes:
;   Copyright (C) 2000 Free Software Foundation, Inc.
;
;   Written by: Alexander Malmberg <alexander@malmberg.org>
;   Date: 2003-02-15
;
;   Base on code written by: Adam Fedor <fedor@gnu.org>
;   Date: Dec 2000
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

(begin
  (define run gstep-nil)
  (define date gstep-nil)
  (set! run ([] "NSRunLoop" currentRunLoop))

  (greg-testcase "-performSelector:target:argument:order:modes: works for one performer" #t
    (lambda ()
      (define str ([] ([] "NSMutableString" alloc) init))

      ([] run performSelector: "appendString:"
	target: str
	argument: ($$ "foo")
	order: 0
	modes: ([] "NSArray" arrayWithObject: NSDefaultRunLoopMode))

      (set! date ([] "NSDate" dateWithTimeIntervalSinceNow: 0.1))
      ([] run runUntilDate: date)

      (gstep-bool->bool ([] str isEqual: ($$ "foo")))
    )
  )

  (greg-testcase "-performSelector:target:argument:order:modes: only sends the message once" #t
    (lambda ()
      (define str ([] ([] "NSMutableString" alloc) init))

      ([] run performSelector: "appendString:"
	target: str
	argument: ($$ "foo")
	order: 0
	modes: ([] "NSArray" arrayWithObject: NSDefaultRunLoopMode))

      (set! date ([] "NSDate" dateWithTimeIntervalSinceNow: 0.1))
      ([] run runUntilDate: date)

      ; The performer should not fire now.
      ([] run runUntilDate: date)

      (gstep-bool->bool ([] str isEqual: ($$ "foo")))
    )
  )

  (greg-testcase "-performSelector:target:argument:order:modes: orders performers correctly" #t
    (lambda ()
      (define str ([] ([] "NSMutableString" alloc) init))

      ([] run performSelector: "appendString:"
	target: str
	argument: ($$ "foo")
	order: 10
	modes: ([] "NSArray" arrayWithObject: NSDefaultRunLoopMode))
      ([] run performSelector: "appendString:"
	target: str
	argument: ($$ "bar")
	order: 11
	modes: ([] "NSArray" arrayWithObject: NSDefaultRunLoopMode))

      (set! date ([] "NSDate" dateWithTimeIntervalSinceNow: 0.1))
      ([] run runUntilDate: date)

      (gstep-bool->bool ([] str isEqual: ($$ "foobar")))
    )
  )

  (greg-testcase "-cancelPerformSelector:target:argument: works" #t
    (lambda ()
      (define str ([] ([] "NSMutableString" alloc) init))

      ([] run performSelector: "appendString:"
	target: str
	argument: ($$ "foo")
	order: 10
	modes: ([] "NSArray" arrayWithObject: NSDefaultRunLoopMode))
      ([] run performSelector: "appendString:"
	target: str
	argument: ($$ "bar")
	order: 11
	modes: ([] "NSArray" arrayWithObject: NSDefaultRunLoopMode))
      ([] run performSelector: "appendString:"
	target: str
	argument: ($$ "zot")
	order: 11
	modes: ([] "NSArray" arrayWithObject: NSDefaultRunLoopMode))

      ([] run cancelPerformSelector: "appendString:"
	target: str
	argument: ($$ "bar"))

      (set! date ([] "NSDate" dateWithTimeIntervalSinceNow: 0.1))
      ([] run runUntilDate: date)

      (gstep-bool->bool ([] str isEqual: ($$ "foozot")))
    )
  )

  (greg-testcase "-cancelPerformSelectorsWithTarget: works" #t
    (lambda ()
      (define str ([] ([] "NSMutableString" alloc) init))

      ([] run performSelector: "appendString:"
	target: str
	argument: ($$ "foo")
	order: 10
	modes: ([] "NSArray" arrayWithObject: NSDefaultRunLoopMode))
      ([] run performSelector: "appendString:"
	target: str
	argument: ($$ "bar")
	order: 11
	modes: ([] "NSArray" arrayWithObject: NSDefaultRunLoopMode))

      ([] run cancelPerformSelectorsWithTarget: str)

      (set! date ([] "NSDate" dateWithTimeIntervalSinceNow: 0.1))
      ([] run runUntilDate: date)

      (gstep-bool->bool ([] str isEqual: ($$ "")))
    )
  )
)

