;   test00.scm - Tests for NSScanner
;   Copyright (C) 2001 Free SoftwareFoundation, Inc.
;
;   Written by:  Nicola Pero <nicola@brainstorm.co.uk>
;   Date: March 2001
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

(begin

  (define test-three-word-string 
    (lambda (testString separators)
      (let ((theScanner ([] "NSScanner" scannerWithString: testString)))
	(let ((scan-a-word 
	       (lambda ()
		 ([] theScanner 
		  scanUpToCharactersFromSet: separators  
		  intoString: (voidp-nil))
		 ([] theScanner 
		  scanCharactersFromSet: separators
		  intoString: (voidp-nil))
		 #t)))
	  (and 
	   (scan-a-word)
	   (not (gstep-bool ([] theScanner isAtEnd)))
	   (scan-a-word)
	   (not (gstep-bool ([] theScanner isAtEnd)))
	   (scan-a-word)
	   (gstep-bool ([] theScanner isAtEnd)))))))

  (greg-testcase 
   "scanning three words separated by newlines" #t
   (lambda ()
     (test-three-word-string 
      ($$ "one\ntwo\nthree\n")
      ([] "NSCharacterSet" whitespaceAndNewlineCharacterSet))))
  
  (greg-testcase 
   "scanning three words separated by spaces" #t
   (lambda ()
     (test-three-word-string
      ($$ "one two three ")
      ([] "NSCharacterSet" whitespaceCharacterSet))))

  (greg-testcase 
   "scanning three words separated by tabs" #t
   (lambda ()
     (test-three-word-string
      ($$ "one\ttwo\tthree\t")
      ([] "NSCharacterSet" whitespaceCharacterSet))))

  (greg-testcase 
   "scanning three words separated by newlines and spaces" #t
   (lambda ()
     (test-three-word-string 
      ($$ "one \ntwo\n three \n")
      ([] "NSCharacterSet" whitespaceAndNewlineCharacterSet))))

  (greg-testcase 
   "scanning three words separated by newlines and spaces and tabs" #t
   (lambda ()
     (test-three-word-string 
      ($$ "one \ntwo\n \tthree \t\n")
      ([] "NSCharacterSet" whitespaceAndNewlineCharacterSet))))

  (greg-testcase 
   "scanning three words separated by a whitespaces and ;" #t
   (lambda ()
     (let ((character-set 
	    ([] ([] "NSCharacterSet" whitespaceCharacterSet) mutableCopy)))
       ([] character-set addCharactersInString: ($$ ";"))
       (test-three-word-string 
	($$ "one ;two ; three ;")
	character-set))))
)





