;   test01.scm - Tests for NSScanner
;   Copyright (C) 2001 Free SoftwareFoundation, Inc.
;
;   Written by:  Nicola Pero <nicola@brainstorm.co.uk>
;   Date: March 2001
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

(begin

  (define str gstep-nil)
  (define scn gstep-nil)
  (define ret gstep-nil)
  (define retp voidp-nil)

  (define intmax 2147483647)
  (define intmin (- 0 (- intmax 1)))

  ;
  ; Integer tests
  ;
  (define test-scan-int-good
    (lambda (value)
        (set! retp (list->voidp (list 0) "i"))
        (set! str ([] ([] NSNumber numberWithInt: value) description))
        (set! scn ([] NSScanner scannerWithString: str))
	(and
	  (gstep-bool ([] scn scanInt: retp))
	  (equal? (list value) (voidp->list retp "i" 1))
        )
    )
  )

  (define test-scan-int-overflow
    (lambda (value)
        (set! retp (list->voidp (list 0) "i"))
        (set! str ([] ([] NSNumber numberWithFloat: value) description))
        (set! scn ([] NSScanner scannerWithString: str))
	(and
	  (gstep-bool ([] scn scanInt: retp))
	  (cond
	    ((< 0 value)
	      (equal? intmin (voidp->list retp "i" 1))
            )
            (else
	      (equal? intmax (voidp->list retp "i" 1))
            )
          )
        )
    )
  )

  (greg-testcase "NSScanner large ints" #t
    (lambda ()
      (test-scan-int-good (- intmax 20))
    )
  )

  (greg-testcase "NSScanner small ints" #t
    (lambda ()
      (test-scan-int-good (+ intmin 20))
    )
  )

  (greg-testcase "NSScanner non-digits terminate scan" #t
    (lambda ()
        (set! retp (list->voidp (list 0) "i"))
        (set! str ($$ "1234F00"))
        (set! scn ([] NSScanner scannerWithString: str))
	(and
	  (gstep-bool ([] scn scanInt: retp))
	  (equal? 4 ([] scn scanLocation))
        )
    )
  )

  (greg-testcase "NSScanner non-digits dont move scan location" #t
    (lambda ()
        (set! retp (list->voidp (list 0) "i"))
        (set! str ($$ "junk"))
        (set! scn ([] NSScanner scannerWithString: str))
	(and
	  (not (gstep-bool ([] scn scanInt: retp)))
	  (equal? 0 ([] scn scanLocation))
        )
    )
  )

  (greg-testcase "NSScanner non-digits dont consume characters to be skipped" #t
    (lambda ()
        (set! retp (list->voidp (list 0) "i"))
        (set! str ($$ "     junk"))
        (set! scn ([] NSScanner scannerWithString: str))
	(and
	  (not (gstep-bool ([] scn scanInt: retp)) )
	  (equal? 0 ([] scn scanLocation))
        )
    )
  )

  ;
  ; Radix Unsigned Integer tests
  ;
  (define test-scan-radix-unsigned
    (lambda (str expectValue expectedValue expectedScanLoc)
        (set! retp (list->voidp (list 0) "i"))
        (set! scn ([] NSScanner scannerWithString: ( $$ str )))
        (gstep-bool ([] scn scanRadixUnsignedInt: retp))
	(and
	  (cond
	    ((= 1 expectValue)
	      (equal? (list expectedValue) (voidp->list retp "i" 1))
            )
	    (else #t)
          )
	  (equal? expectedScanLoc ([] scn scanLocation))
        )
    )
  )

  (greg-testcase "NSScanner radiux unsigned (non-digits terminate scan)" #t
    (lambda ()
      (and
         (test-scan-radix-unsigned "1234FOO" 1 1234 4)
         (test-scan-radix-unsigned "01234FOO" 1 (string->number "#o1234") 5 )
         (test-scan-radix-unsigned "0x1234FOO" 1 (string->number "#x1234F") 7 )
         (test-scan-radix-unsigned "0X1234FOO" 1 (string->number "#x1234F") 7 )
         (test-scan-radix-unsigned "012348FOO" 1 (string->number "#o1234") 5 )
      )
    )
  )

  (greg-testcase "NSScanner radiux unsigned (non-digits dont move scan)" #t
    (lambda ()
      (and
         (test-scan-radix-unsigned "FOO" 0 0 0)
         (test-scan-radix-unsigned "  FOO" 0 0 0)
         (test-scan-radix-unsigned "  0x " 0 0 0)
      )
    )
  )

  ;
  ; Hex int tests
  ;
  (define test-scan-hex
    (lambda (str expectValue expectedValue expectedScanLoc)
        (set! retp (list->voidp (list 0) "i"))
        (set! scn ([] NSScanner scannerWithString: ( $$ str )))
        (gstep-bool ([] scn scanHexInt: retp))
	(and
	  (cond
	    ((= 1 expectValue)
	      (equal? (list expectedValue) (voidp->list retp "i" 1))
            )
	    (else #t)
          )
	  (equal? expectedScanLoc ([] scn scanLocation))
        )
    )
  )

  (greg-testcase "NSScanner hex (non-digits terminate scan)" #t
    (lambda ()
      (and
         (test-scan-hex "1234FOO" 1 (string->number "#x1234F") 5)
         (test-scan-hex "01234FOO" 1 (string->number "#x1234F") 6)
      )
    )
  )

  ;
  ; Double tests
  ;
  (define test-scan-double
    (lambda (str expectedValue)
        (set! retp (list->voidp (list 0) "d"))
        (set! scn ([] NSScanner scannerWithString: ( $$ str )))
        (gstep-bool ([] scn scanDouble: retp))
        (set! ret (list-ref (voidp->list retp "d" 1) 0))
	(and
          (>= .00000000001 (abs (- expectedValue ret)))
	  (equal?  (string-length str) ([] scn scanLocation))
        )
    )
  )

  (greg-testcase "NSScanner scans doubles" #t
    (lambda ()
      (and
         (test-scan-double "123.456" 123.456)
         (test-scan-double "123.45678901234567890123456789012345678901234567"
                          123.45678901234567890123456789012345678901234567)
         (test-scan-double "0.000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000123456789e+100"
                                               1.23456789)
      )
    )
  )

  (greg-testcase "NSScanner scans double with exponents" #t
    (lambda ()
      (and
         (test-scan-double "1e0" 1)
         (test-scan-double "1e1" 10)
         (test-scan-double "1e+1" 10 )
         (test-scan-double "1e+10" 1e10 )
         (test-scan-double  "1e-1" 1e-1)
         (test-scan-double "1e-10" 1e-10 )
      )
    )
  )

)





