
;   test00.scm - Tests of string comparison and searches
;   Copyright (C) 1998-1999 Free SoftwareFoundation, Inc.
;
;   Written by:  Richard Frith-Macdonald <richard@brainstorm.co.uk>
;   Date: July 1998
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

(test-exception
  "NSString extracting substring with range beyond end of string"
  NSRangeException
  (lambda ()
    (define r (NSMakeRange 6 4))
    (define s ([] "NSString" stringWithCString: "Hello"))
    ([] s substringWithRange: r)
    #f
  )
)

;
; Check ranges for equality - all zero-length ranges are considered equal.
;
(define (chk-range r0 r1)
  (if (equal? r0 r1)
    #t
    (if (and (<= (NSRange-length r0) 0) (<= (NSRange-length r1) 0))
      #t
      #f
    )
  )
)

; s0	The receiver for the comparison
; s1	The other string to compare against
; ci	Expected result for case-insensitive comparison
; cs	Expected result for case-sensitive comparison
; lci	Expected result for case-insensitive literal comparison
; lcs	Expected result for case-sensitive literal comparison
; ra	Range of receiver to be compared with other
(define (test-strcompare s0 s1 ci cs lci lcs ra)
  (begin
    (define con "\" and \"")
    (define want #t)
    (define txt #t)
    (define res #t)
    (define cs0 #t)
    (define cs1 #t)
    (define us0 #t)
    (define us1 #t)
    (define l gstep-nil)
    (define d gstep-nil)
    (define b gstep-nil)
    (define typ "")
    (define opts 0)

    ;
    ; We want to check that compares work for
    ;  cstring - cstring
    ;  unicode - unicode
    ;  cstring - unicode
    ;  unicode - cstring
    ; so we set up a cstring and a unicode version of each string
    ;
    (greg-testcase
      (string-append "create strings for compare of \"" s0 con s1 "\" is ok") #t
      (lambda ()
	(set! cs0 ([] "NSString" stringWithCString: s0))
	(set! l ([] cs0 length))
	(set! d ([] "NSMutableData" dataWithLength: (* 2 l)))
	(set! b ([] d mutableBytes))
	([] cs0 getCharacters: b)
	(set! us0 ([] "NSString" stringWithCharacters: b length: l))
	(set! cs1 ([] "NSString" stringWithCString: s1))
	(set! l ([] cs1 length))
	(set! d ([] "NSMutableData" dataWithLength: (* 2 l)))
	(set! b ([] d mutableBytes))
	([] cs1 getCharacters: b)
	(set! us1 ([] "NSString" stringWithCharacters: b length: l))
	#t
      )
    )

    (set! typ " case insensitive comparison for \"")
    (set! opts NSCaseInsensitiveSearch)
    (if (equal? ci NSOrderedAscending)
      (set! txt "\" is NSOrderedAscending")
      (if (equal? ci NSOrderedDescending)
	(set! txt "\" is NSOrderedDescending")
	(begin
	  (set! ci NSOrderedSame)
	  (set! txt "\" is NSOrderedSame")
	)
      )
    )
    (set! want ci)
    (greg-testcase
      (string-append "CCString" typ s0 con s1 txt) #t
      (lambda ()
	(set! res ([] cs0 compare: cs1 options: opts range: ra))
	(equal? res want)
      )
    )
    (greg-testcase
      (string-append "UUString" typ s0 con s1 txt) #t
      (lambda ()
	(set! res ([] us0 compare: us1 options: opts range: ra))
	(equal? res want)
      )
    )
    (greg-testcase
      (string-append "UCString" typ s0 con s1 txt) #t
      (lambda ()
	(set! res ([] us0 compare: cs1 options: opts range: ra))
	(equal? res want)
      )
    )
    (greg-testcase
      (string-append "CUString" typ s0 con s1 txt) #t
      (lambda ()
	(set! res ([] cs0 compare: us1 options: opts range: ra))
	(equal? res want)
      )
    )

    (set! typ " case sensitive comparison for \"")
    (set! opts 0)
    (if (equal? cs NSOrderedAscending)
      (set! txt "\" is NSOrderedAscending")
      (if (equal? cs NSOrderedDescending)
	(set! txt "\" is NSOrderedDescending")
	(begin
	  (set! cs NSOrderedSame)
	  (set! txt "\" is NSOrderedSame")
	)
      )
    )
    (set! want cs)
    (greg-testcase
      (string-append "CCString" typ s0 con s1 txt) #t
      (lambda ()
	(set! res ([] cs0 compare: cs1 options: opts range: ra))
	(equal? res want)
      )
    )
    (greg-testcase
      (string-append "UUString" typ s0 con s1 txt) #t
      (lambda ()
	(set! res ([] us0 compare: us1 options: opts range: ra))
	(equal? res want)
      )
    )
    (greg-testcase
      (string-append "UCString" typ s0 con s1 txt) #t
      (lambda ()
	(set! res ([] us0 compare: cs1 options: opts range: ra))
	(equal? res want)
      )
    )
    (greg-testcase
      (string-append "CUString" typ s0 con s1 txt) #t
      (lambda ()
	(set! res ([] cs0 compare: us1 options: opts range: ra))
	(equal? res want)
      )
    )

    (set! typ " case insensitive literal comparison for \"")
    (set! opts (logior NSCaseInsensitiveSearch NSLiteralSearch))
    (if (equal? lci NSOrderedAscending)
      (set! txt "\" is NSOrderedAscending")
      (if (equal? lci NSOrderedDescending)
	(set! txt "\" is NSOrderedDescending")
	(begin
	  (set! lci NSOrderedSame)
	  (set! txt "\" is NSOrderedSame")
	)
      )
    )
    (set! want lci)
    (greg-testcase
      (string-append "CCString" typ s0 con s1 txt) #t
      (lambda ()
	(set! res ([] cs0 compare: cs1 options: opts range: ra))
	(equal? res want)
      )
    )
    (greg-testcase
      (string-append "UUString" typ s0 con s1 txt) #t
      (lambda ()
	(set! res ([] us0 compare: us1 options: opts range: ra))
	(equal? res want)
      )
    )
    (greg-testcase
      (string-append "UCString" typ s0 con s1 txt) #t
      (lambda ()
	(set! res ([] us0 compare: cs1 options: opts range: ra))
	(equal? res want)
      )
    )
    (greg-testcase
      (string-append "CUString" typ s0 con s1 txt) #t
      (lambda ()
	(set! res ([] cs0 compare: us1 options: opts range: ra))
	(equal? res want)
      )
    )

    (set! typ " case sensitive literal comparison for \"")
    (set! opts NSLiteralSearch)
    (if (equal? lcs NSOrderedAscending)
      (set! txt "\" is NSOrderedAscending")
      (if (equal? lcs NSOrderedDescending)
	(set! txt "\" is NSOrderedDescending")
	(begin
	  (set! lcs NSOrderedSame)
	  (set! txt "\" is NSOrderedSame")
	)
      )
    )

    (set! want lcs)
    (greg-testcase
      (string-append "CCString" typ s0 con s1 txt) #t
      (lambda ()
	(set! res ([] cs0 compare: cs1 options: opts range: ra))
	(equal? res want)
      )
    )
    (greg-testcase
      (string-append "UUString" typ s0 con s1 txt) #t
      (lambda ()
	(set! res ([] us0 compare: us1 options: opts range: ra))
	(equal? res want)
      )
    )
    (greg-testcase
      (string-append "UCString" typ s0 con s1 txt) #t
      (lambda ()
	(set! res ([] us0 compare: cs1 options: opts range: ra))
	(equal? res want)
      )
    )
    (greg-testcase
      (string-append "CUString" typ s0 con s1 txt) #t
      (lambda ()
	(set! res ([] cs0 compare: us1 options: opts range: ra))
	(equal? res want)
      )
    )

  )
)

(define (test-strrange s0 s1 opts range want)
  (begin
    (define con "\" and \"")
    (define res #t)
    (define cs0 #t)
    (define cs1 #t)
    (define us0 #t)
    (define us1 #t)
    (define l gstep-nil)
    (define d gstep-nil)
    (define b gstep-nil)
    (define typ "")

    ;
    ; We want to check that finding range of string works for
    ;  cstring - cstring
    ;  unicode - unicode
    ;  cstring - unicode
    ;  unicode - cstring
    ; so we set up a cstring and a unicode version of each string
    ;
    (greg-testcase
      (string-append "create strings for range with \"" s0 con s1 "\" is ok") #t
      (lambda ()
	(set! cs0 ([] "NSString" stringWithCString: s0))
	(set! l ([] cs0 length))
	(set! d ([] "NSMutableData" dataWithLength: (* 2 l)))
	(set! b ([] d mutableBytes))
	([] cs0 getCharacters: b)
	(set! us0 ([] "NSString" stringWithCharacters: b length: l))
	(set! cs1 ([] "NSString" stringWithCString: s1))
	(set! l ([] cs1 length))
	(set! d ([] "NSMutableData" dataWithLength: (* 2 l)))
	(set! b ([] d mutableBytes))
	([] cs1 getCharacters: b)
	(set! us1 ([] "NSString" stringWithCharacters: b length: l))
	#t
      )
    )

    (greg-testcase
      (string-append "CCString range for " s0 con s1 " is ok") #t
      (lambda ()
	(set! res ([] cs0 rangeOfString: cs1 options: opts range: range))
	(chk-range res want)
      )
    )

    (greg-testcase
      (string-append "UUString range for " s0 con s1 " is ok") #t
      (lambda ()
	(set! res ([] us0 rangeOfString: us1 options: opts range: range))
	(chk-range res want)
      )
    )

    (greg-testcase
      (string-append "UCString range for " s0 con s1 " is ok") #t
      (lambda ()
	(set! res ([] us0 rangeOfString: cs1 options: opts range: range))
	(chk-range res want)
      )
    )

    (greg-testcase
      (string-append "CUString range for " s0 con s1 " is ok") #t
      (lambda ()
	(set! res ([] cs0 rangeOfString: us1 options: opts range: range))
	(chk-range res want)
      )
    )
  )
)

(define (test-setrange s c o range want)
  (begin
    (define con "\" and \"")
    (define res #t)
    (define cs0 #t)
    (define cs1 #t)
    (define us0 #t)
    (define us1 #t)
    (define l gstep-nil)
    (define d gstep-nil)
    (define b gstep-nil)
    (define typ "")

    ;
    ; We want to check that finding range of string works for
    ;  cstring
    ;  unicode
    ;  mutable cstring
    ;  mutable unicode
    ; so we set up a versions of each string
    ;
    (greg-testcase
      (string-append "create strings for range with \"" s "\" is ok") #t
      (lambda ()
	(set! cs0 ([] "NSString" stringWithCString: s))
	(set! l ([] cs0 length))
	(set! d ([] "NSMutableData" dataWithLength: (* 2 l)))
	(set! b ([] d mutableBytes))
	([] cs0 getCharacters: b)
	(set! us0 ([] "NSString" stringWithCharacters: b length: l))
	(set! cs1 ([] "NSMutableString" stringWithCString: s))
	(set! l ([] cs1 length))
	(set! d ([] "NSMutableData" dataWithLength: (* 2 l)))
	(set! b ([] d mutableBytes))
	([] cs1 getCharacters: b)
	(set! us1 ([] "NSMutableString" stringWithCharacters: b length: l))
	#t
      )
    )

    (greg-testcase
      (string-append "CString range for " s " is ok") #t
      (lambda ()
	(set! res ([] cs0 rangeOfCharacterFromSet: c options: o range: range))
	(chk-range res want)
      )
    )

    (greg-testcase
      (string-append "UString range for " s " is ok") #t
      (lambda ()
	(set! res ([] us0 rangeOfCharacterFromSet: c options: o range: range))
	(chk-range res want)
      )
    )

    (greg-testcase
      (string-append "MCString range for " s " is ok") #t
      (lambda ()
	(set! res ([] cs1 rangeOfCharacterFromSet: c options: o range: range))
	(chk-range res want)
      )
    )

    (greg-testcase
      (string-append "MUString range for " s " is ok") #t
      (lambda ()
	(set! res ([] us1 rangeOfCharacterFromSet: c options: o range: range))
	(chk-range res want)
      )
    )
  )
)

;
; Now do some string comparison tests.
;
(test-exception
  "NSString comparison with range beyond end of string"
  NSRangeException
  (lambda ()
    (define r (NSMakeRange 4 3))
    (define s ([] "NSString" stringWithCString: "Hello"))
    (define o ([] "NSString" stringWithCString: "Hello"))
    ([] s compare: o options: NSLiteralSearch range: r)
    #f
  )
)

(test-strcompare
  "hello" "hello"
  NSOrderedSame NSOrderedSame NSOrderedSame NSOrderedSame
  (NSMakeRange 0 5)
)

(test-strcompare
  "" ""
  NSOrderedSame NSOrderedSame NSOrderedSame NSOrderedSame
  (NSMakeRange 0 0)
)

(test-strcompare
  "hello" "Hello"
  NSOrderedSame NSOrderedDescending NSOrderedSame NSOrderedDescending
  (NSMakeRange 0 5)
)

(test-strcompare
  "Hello" "hello"
  NSOrderedSame NSOrderedAscending NSOrderedSame NSOrderedAscending
  (NSMakeRange 0 5)
)

(test-strcompare
  "abc" "ab"
  NSOrderedDescending NSOrderedDescending
  NSOrderedDescending NSOrderedDescending
  (NSMakeRange 0 3)
)

(test-strcompare
  "ab" "abc"
  NSOrderedAscending NSOrderedAscending
  NSOrderedAscending NSOrderedAscending
  (NSMakeRange 0 2)
)

(test-strcompare
  "" "a"
  NSOrderedAscending NSOrderedAscending
  NSOrderedAscending NSOrderedAscending
  (NSMakeRange 0 0)
)

(test-strcompare
  "a" ""
  NSOrderedDescending NSOrderedDescending
  NSOrderedDescending NSOrderedDescending
  (NSMakeRange 0 1)
)

(test-strcompare
  "a" ""
  NSOrderedSame NSOrderedSame
  NSOrderedSame NSOrderedSame
  (NSMakeRange 0 0)
)

(test-strcompare
  "Location" "LoCaTiOn"
  NSOrderedSame NSOrderedDescending
  NSOrderedSame NSOrderedDescending
  (NSMakeRange 0 8)
)

(test-strcompare
  "1234567890_!@$%^&*()QWERTYUIOP{}ASDFGHJKL:;'ZXCVBNM,./<>?"
  "1234567890_!@$%^&*()qwertyuiop{}asdfghjkl:;'zxcvbnm,./<>?"
  NSOrderedSame NSOrderedAscending
  NSOrderedSame NSOrderedAscending
  (NSMakeRange 0 57)
)

(test-strcompare
  "1234567890_!@$%^&*()QWERTYUIOP{}ASDFGHJKL:;'ZXCVBNM,./<>?"
  "1234567890_!@$%^&*()qwertyuiop{}asdfghjkl:;'zxcvbnm,./<>"
  NSOrderedSame NSOrderedAscending
  NSOrderedSame NSOrderedAscending
  (NSMakeRange 0 56)
)

; A test where return should be ascending order 'cos other string is longer
; than the range of the receiver tested.
(test-strcompare
  "1234567890_!@$%^&*()QWERTYUIOP{}ASDFGHJKL:;'ZXCVBNM,./<>?"
  "1234567890_!@$%^&*()qwertyuiop{}asdfghjkl:;'zxcvbnm,./<>?"
  NSOrderedAscending NSOrderedAscending
  NSOrderedAscending NSOrderedAscending
  (NSMakeRange 0 56)
)

(test-strcompare
  "abcdefg" "ABCDEFG"
  NSOrderedSame NSOrderedDescending
  NSOrderedSame NSOrderedDescending
  (NSMakeRange 0 7)
)

(test-strcompare
  "abcdefg" "CDE"
  NSOrderedSame NSOrderedDescending
  NSOrderedSame NSOrderedDescending
  (NSMakeRange 2 3)
)

(test-strcompare
  "abcdefg" "CDEF"
  NSOrderedAscending NSOrderedDescending
  NSOrderedAscending NSOrderedDescending
  (NSMakeRange 2 3)
)

;
; And some string searches
;
;(test-exception
;  "NSString search with range beyond end of string"
;  NSRangeException
;  (lambda ()
;    (define r (NSMakeRange 4 2))
;    (define s ([] "NSString" stringWithCString: "Hello"))
;    (define o ([] "NSString" stringWithCString: "Hello"))
;    ([] s rangeOfString: o options: NSLiteralSearch range: r)
;    #f
;  )
;)

(test-strrange "hello" "hello"
  NSAnchoredSearch
  (NSMakeRange 0 5) (NSMakeRange 0 5)
)

(test-strrange "hello" "hello"
  (logior NSAnchoredSearch NSBackwardsSearch)
  (NSMakeRange 0 5) (NSMakeRange 0 5)
)

(test-strrange "hello" "hElLo"
  NSLiteralSearch
  (NSMakeRange 0 5) (NSMakeRange 0 0)
)

(test-strrange "hello" "hElLo"
  NSCaseInsensitiveSearch
  (NSMakeRange 0 5) (NSMakeRange 0 5)
)

(test-strrange "hello" "hell"
  NSAnchoredSearch
  (NSMakeRange 0 5) (NSMakeRange 0 4)
)

(test-strrange "hello" "hel"
  NSBackwardsSearch
  (NSMakeRange 0 5) (NSMakeRange 0 3)
)

(test-strrange "hello" "he"
  NSLiteralSearch
  (NSMakeRange 0 5) (NSMakeRange 0 2)
)

(test-strrange "hello" "h"
  NSLiteralSearch
  (NSMakeRange 0 5) (NSMakeRange 0 1)
)

(test-strrange "hello" ""
  NSLiteralSearch
  (NSMakeRange 0 5) (NSMakeRange 0 0)
)

(test-strrange "hello" "el"
  NSLiteralSearch
  (NSMakeRange 0 5) (NSMakeRange 1 2)
)

(test-strrange "hello" "el"
  NSLiteralSearch
  (NSMakeRange 0 2) (NSMakeRange 0 0)
)

(test-strrange "hello" "el"
  NSLiteralSearch
  (NSMakeRange 2 3) (NSMakeRange 0 0)
)

(test-strrange "hello" "ell"
  NSLiteralSearch
  (NSMakeRange 0 5) (NSMakeRange 1 3)
)

(test-strrange "hello" "lo"
  NSLiteralSearch
  (NSMakeRange 2 3) (NSMakeRange 3 2)
)

(test-strrange "boaboaboa" "abo"
  NSLiteralSearch
  (NSMakeRange 0 9) (NSMakeRange 2 3)
)

(test-strrange "boaboaboa" "abo"
  NSBackwardsSearch
  (NSMakeRange 0 9) (NSMakeRange 5 3)
)

(test-strrange "boaboaboa" "ABO"
  NSCaseInsensitiveSearch
  (NSMakeRange 0 9) (NSMakeRange 2 3)
)

(test-strrange "boaboaboa" "abo"
  (logior NSCaseInsensitiveSearch NSBackwardsSearch)
  (NSMakeRange 0 9) (NSMakeRange 5 3)
)

(test-setrange "boaboaboa"
  ([] NSCharacterSet alphanumericCharacterSet)
  (logior NSCaseInsensitiveSearch NSBackwardsSearch)
  (NSMakeRange 0 9) (NSMakeRange 8 1)
)

(test-setrange "boaboaboa"
  ([] NSCharacterSet whitespaceCharacterSet)
  (logior NSCaseInsensitiveSearch NSBackwardsSearch)
  (NSMakeRange 0 9) (NSMakeRange NSNotFound 0)
)

(test-setrange "bo boaboa"
  ([] NSCharacterSet whitespaceCharacterSet)
  (logior NSCaseInsensitiveSearch NSBackwardsSearch)
  (NSMakeRange 0 9) (NSMakeRange 2 1)
)


;
; Line Range Searches
;
(define (test-linerange s range want)
  (begin
    (define res #t)
    (define cs0 #t)
    (define l gstep-nil)

    ;
    ; We want to check that finding range of string works for
    ;  cstring
    ;  unicode
    ;  mutable cstring
    ;  mutable unicode
    ; so we set up a versions of each string
    ;
    (greg-testcase
      (string-append "lineRangeForRange: with \"" s "\" is ok") #t
      (lambda ()
	(set! cs0 ([] "NSString" stringWithCString: s))
	(set! l ([] cs0 length))
	(set! res ([] cs0 lineRangeForRange: range))
	(chk-range res want)
      )
    )

  )
)

(test-linerange "This is a line of text\n" (NSMakeRange 10 10)
		(NSMakeRange 0 23))

(test-linerange "This is a line of text\r\n" (NSMakeRange 10 10)
		(NSMakeRange 0 24))

(test-linerange "This is a line of text\r\r" (NSMakeRange 10 10)
		(NSMakeRange 0 23))
