;   test02.scm - More Tests of string path operations
;   Copyright (C) 2001 Free SoftwareFoundation, Inc.
;
;   Written by:  Nicola Pero <nicola@brainstorm.co.uk>
;   Date: May 2001
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

;; NSString -pathComponents

(greg-testcase 
 "[@\"home\" pathComponents] == (@\"home\")" #t 
 (lambda () 
   (gstep-bool 
    ([] ([] ([] ($$ "home") pathComponents) objectAtIndex: 0) isEqual:
     ($$ "home")))))

(greg-testcase 
 "[@\"/home\" pathComponents] == (@\"/\", @\"home\")" #t 
 (lambda () 
   (let ((result ([] ($$ "/home") pathComponents)))
     (and 
      (gstep-bool ([] ([] result objectAtIndex: 0) isEqual: ($$ "/")))
      (gstep-bool ([] ([] result objectAtIndex: 1) isEqual: ($$ "home")))))))

(greg-testcase 
 "[@\"/home/\" pathComponents] == (@\"/\", @\"home\", @\"/\")" #t 
 (lambda () 
   (let ((result ([] ($$ "/home/") pathComponents)))
     (and 
      (gstep-bool ([] ([] result objectAtIndex: 0) isEqual: ($$ "/")))
      (gstep-bool ([] ([] result objectAtIndex: 1) isEqual: ($$ "home")))
      (gstep-bool ([] ([] result objectAtIndex: 2) isEqual: ($$ "/")))))))

(greg-testcase 
 "[@\"/home/nicola\" pathComponents] == (@\"/\", @\"home\", @\"nicola\")" #t 
 (lambda () 
   (let ((result ([] ($$ "/home/nicola") pathComponents)))
     (and 
      (gstep-bool ([] ([] result objectAtIndex: 0) isEqual: ($$ "/")))
      (gstep-bool ([] ([] result objectAtIndex: 1) isEqual: ($$ "home")))
      (gstep-bool ([] ([] result objectAtIndex: 2) isEqual: ($$ "nicola")))))))

(greg-testcase 
 "[@\"/home//nicola\" pathComponents] == (@\"/\", @\"home\", @\"nicola\")" #t 
 (lambda () 
   (let ((result ([] ($$ "/home//nicola") pathComponents)))
     (and 
      (gstep-bool ([] ([] result objectAtIndex: 0) isEqual: ($$ "/")))
      (gstep-bool ([] ([] result objectAtIndex: 1) isEqual: ($$ "home")))
      (gstep-bool ([] ([] result objectAtIndex: 2) isEqual: ($$ "nicola")))))))

(greg-testcase 
 "[@\"//home//nicola\" pathComponents] == (@\"/\", @\"home\", @\"nicola\")" #t 
 (lambda () 
   (let ((result ([] ($$ "//home//nicola") pathComponents)))
     (and 
      (gstep-bool ([] ([] result objectAtIndex: 0) isEqual: ($$ "/")))
      (gstep-bool ([] ([] result objectAtIndex: 1) isEqual: ($$ "home")))
      (gstep-bool ([] ([] result objectAtIndex: 2) isEqual: ($$ "nicola")))))))

;; NSString -pathExtension

(greg-testcase 
 "[@\"home/nicola.jpg\" pathExtension] == @\"jpg\"" #t
 (lambda () 
   (gstep-bool ([] ([] ($$ "home/nicola.jpg") pathExtension) isEqual: 
		($$ "jpg")))))

(greg-testcase 
 "[@\"/home/nicola.jpg\" pathExtension] == @\"jpg\"" #t
 (lambda () 
   (gstep-bool ([] ([] ($$ "/home/nicola.jpg") pathExtension) isEqual: 
		($$ "jpg")))))

(greg-testcase 
 "[@\"/home/nicola\" pathExtension] == @\"\"" #t
 (lambda () 
   (gstep-bool ([] ([] ($$ "home/nicola") pathExtension) isEqual: ($$ "")))))

(greg-testcase 
 "[@\"/home/nicola/\" pathExtension] == @\"\"" #t
 (lambda () 
   (gstep-bool ([] ([] ($$ "home/nicola/") pathExtension) isEqual: ($$ "")))))

(greg-testcase 
 "[@\"/home/nicola..jpg\" pathExtension] == @\"jpg\"" #t
 (lambda () 
   (gstep-bool ([] ([] ($$ "home/nicola..jpg") pathExtension) isEqual: 
		($$ "jpg")))))

;; NSString -stringByAppendingPathComponent:

(greg-testcase
 "[@\"/home\" stringByAppendingPathComponent: @\"nicola.jpg\"] == @\"/home/nicola.jpg\"" #t
 (lambda ()
   (gstep-bool ([] ([] ($$ "/home") stringByAppendingPathComponent: 
		    ($$ "nicola.jpg")) isEqual:
		    ($$ "/home/nicola.jpg")))))

(greg-testcase
 "[@\"/home/\" stringByAppendingPathComponent: @\"nicola.jpg\"] == @\"/home/nicola.jpg\"" #t
 (lambda ()
   (gstep-bool ([] ([] ($$ "/home/") stringByAppendingPathComponent: 
		    ($$ "nicola.jpg")) isEqual:
		    ($$ "/home/nicola.jpg")))))

(greg-testcase
 "[@\"/\" stringByAppendingPathComponent: @\"nicola.jpg\"] == @\"/nicola.jpg\"" #t
 (lambda ()
   (gstep-bool ([] ([] ($$ "/") stringByAppendingPathComponent: 
		    ($$ "nicola.jpg")) isEqual:
		    ($$ "/nicola.jpg")))))

(greg-testcase
 "[@\"\" stringByAppendingPathComponent: @\"nicola.jpg\"] == @\"nicola.jpg\"" #t
 (lambda ()
   (gstep-bool ([] ([] ($$ "") stringByAppendingPathComponent: 
		    ($$ "nicola.jpg")) isEqual:
		    ($$ "nicola.jpg")))))

;; NSString -stringByAppendingPathExtension:

(greg-testcase
 "[@\"/home/nicola.jpg\" stringByAppendingPathExtension: @\"jpg\"] == @\"/home/nicola.jpg.jpg\"" #t
 (lambda ()
   (gstep-bool ([] ([] ($$ "/home/nicola.jpg") stringByAppendingPathExtension: 
		    ($$ "jpg")) isEqual:
		    ($$ "/home/nicola.jpg.jpg")))))

(greg-testcase
 "[@\"/home/nicola.\" stringByAppendingPathExtension: @\"jpg\"] == @\"/home/nicola..jpg\"" #t
 (lambda ()
   (gstep-bool ([] ([] ($$ "/home/nicola.") stringByAppendingPathExtension: 
		    ($$ "jpg")) isEqual:
		    ($$ "/home/nicola..jpg")))))

(greg-testcase
 "[@\"/home/nicola/\" stringByAppendingPathExtension: @\"jpg\"] == @\"/home/nicola/.jpg\"" #t
 (lambda ()
   (gstep-bool ([] ([] ($$ "/home/nicola/") stringByAppendingPathExtension: 
		    ($$ "jpg")) isEqual:
		    ($$ "/home/nicola.jpg")))))

(greg-testcase
 "[@\"nicola\" stringByAppendingPathExtension: @\"jpg\"] == @\"nicola.jpg\"" #t
 (lambda ()
   (gstep-bool ([] ([] ($$ "nicola") stringByAppendingPathExtension: 
		    ($$ "jpg")) isEqual:
		    ($$ "nicola.jpg")))))

;; NSString -stringByDeletingLastPathComponent

(greg-testcase
 "[@\"/home/nicola.jpg\" stringByDeletingLastPathComponent] == @\"/home\"" #t
 (lambda ()
   (gstep-bool 
    ([] ([] ($$ "/home/nicola.jpg") stringByDeletingLastPathComponent) 
     isEqual: ($$ "/home")))))

(greg-testcase
 "[@\"/home/nicola/\" stringByDeletingLastPathComponent] == @\"/home\"" #t
 (lambda ()
   (gstep-bool 
    ([] ([] ($$ "/home/nicola/") stringByDeletingLastPathComponent) 
     isEqual: ($$ "/home")))))

(greg-testcase
 "[@\"/home/\" stringByDeletingLastPathComponent] == @\"/\"" #t
 (lambda ()
   (gstep-bool 
    ([] ([] ($$ "/home/") stringByDeletingLastPathComponent) 
     isEqual: ($$ "/")))))

(greg-testcase
 "[@\"/home\" stringByDeletingLastPathComponent] == @\"/\"" #t
 (lambda ()
   (gstep-bool 
    ([] ([] ($$ "/home") stringByDeletingLastPathComponent) 
     isEqual: ($$ "/")))))

(greg-testcase
 "[@\"/\" stringByDeletingLastPathComponent] == @\"/\"" #t
 (lambda ()
   (gstep-bool 
    ([] ([] ($$ "/") stringByDeletingLastPathComponent) 
     isEqual: ($$ "/")))))

(greg-testcase
 "[@\"hello\" stringByDeletingLastPathComponent] == @\"\"" #t
 (lambda ()
   (gstep-bool 
    ([] ([] ($$ "hello") stringByDeletingLastPathComponent) 
     isEqual: ($$ "")))))

;; NSString -stringByDeletingPathExtension

(greg-testcase
 "[@\"/home/nicola.jpg\" stringByDeletingPathExtension] == @\"/home/nicola\"" #t
 (lambda ()
   (gstep-bool 
    ([] ([] ($$ "/home/nicola.jpg") stringByDeletingPathExtension) 
     isEqual: ($$ "/home/nicola")))))

(greg-testcase
 "[@\"/home/\" stringByDeletingPathExtension] == @\"/home\"" #t
 (lambda ()
   (gstep-bool 
    ([] ([] ($$ "/home/") stringByDeletingPathExtension) 
     isEqual: ($$ "/home")))))

(greg-testcase
 "[@\"nicola.jpg/\" stringByDeletingPathExtension] == @\"nicola\"" #t
 (lambda ()
   (gstep-bool 
   ([] ([] ($$ "nicola.jpg/") stringByDeletingPathExtension) 
     isEqual: ($$ "nicola")))))

(greg-testcase
 "[@\"nicola..jpg\" stringByDeletingPathExtension] == @\"nicola.\"" #t
 (lambda ()
   (gstep-bool 
    ([] ([] ($$ "nicola..jpg") stringByDeletingPathExtension) 
     isEqual: ($$ "nicola.")))))

(greg-testcase
 "[@\".jpg\" stringByDeletingPathExtension] == @\"\"" #t
 (lambda ()
   (gstep-bool 
    ([] ([] ($$ ".jpg") stringByDeletingPathExtension) 
     isEqual: ($$ "")))))

(greg-testcase
 "[@\"/\" stringByDeletingPathExtension] == @\"/\"" #t
 (lambda ()
   (gstep-bool 
    ([] ([] ($$ "/") stringByDeletingPathExtension) 
     isEqual: ($$ "/")))))

;; NSString -stringByExpandingTildeInPath

(greg-testcase
 "[@\"/home/nicola/nil\" stringByExpandingTildeInPath] == @\"/home/nicola/nil\"" #t
 (lambda ()
   (gstep-bool 
    ([] ([] ($$ "/home/nicola/nil") stringByExpandingTildeInPath)
     isEqual: ($$ "/home/nicola/nil")))))

(greg-testcase
 "[@\"~/nil\" stringByExpandingTildeInPath] != @\"~/nil\"" #t
 (lambda ()
   (not (gstep-bool 
	 ([] ([] ($$ "~/nil") stringByExpandingTildeInPath)
	  isEqual: ($$ "~/nil"))))))

(greg-testcase
 "[@\"~root\" stringByExpandingTildeInPath] != @\"~root\"" #t
 (lambda ()
   (not (gstep-bool 
	 ([] ([] ($$ "~root") stringByExpandingTildeInPath)
	  isEqual: ($$ "~root"))))))

;; NSString -stringByStandardizingPath

(define user (passwd:name (getpw)))

(greg-testcase
 "[@\"/home//user\" stringByStandardizingPath] == @\"/home/user\"" #t
 (lambda ()
   (gstep-bool 
    ([] ([] ($$ (string-append "/home//" user)) stringByStandardizingPath)
     isEqual: ($$ (string-append "/home/" user))))))

(greg-testcase
 "[@\"/home/./user\" stringByStandardizingPath] == @\"/home/user\"" #t
 (lambda ()
   (gstep-bool 
    ([] ([] ($$ (string-append "/home/./" user)) stringByStandardizingPath)
     isEqual: ($$ (string-append "/home/" user))))))

(greg-testcase
 "[@\"/home/.//././user\" stringByStandardizingPath] == @\"/home/user\"" #t
 (lambda ()
   (gstep-bool 
    ([] ([] ($$ (string-append "/home/.//././" user)) stringByStandardizingPath)
     isEqual: ($$ (string-append "/home/" user))))))

(greg-testcase
 "[@\"home/../nicola\" stringByStandardizingPath] == @\"home/../nicola\"" #t
 (lambda ()
   (gstep-bool 
    ([] ([] ($$ "home/../nicola") stringByStandardizingPath)
     isEqual: ($$ "home/../nicola")))))

;; NSString -stringsByAppendingPaths

(greg-testcase
 "[@\"home\" stringByAppendingPaths: (@\"nicola\", @\"core\")] == @\"home/nicola/core\"" #t
 (lambda ()
   (let ((array ([] "NSMutableArray" new)))
     (begin 
       ([] array addObject: ($$ "nicola"))
       ([] array addObject: ($$ "core"))
       (eq?
	([] ([] ($$ "home") stringsByAppendingPaths: array) count) 2)))))

;; NSString -isAbsolutePath

(greg-testcase
 "[@\"home\" isAbsolutePath] == NO" #t
 (lambda ()
   (not (gstep-bool ([] ($$ "home") isAbsolutePath)))))

(greg-testcase
 "[@\"/home\" isAbsolutePath] == YES" #t
 (lambda ()
   (gstep-bool ([] ($$ "/home") isAbsolutePath))))

;; NSString +pathWithComponents:

(greg-testcase
 "[NSString pathWithComponents: (@\"home\", @\"nicola\")] == @\"home/nicola\"" #t
 (lambda ()
   (let ((array ([] "NSMutableArray" new)))
     (begin 
       ([] array addObject: ($$ "home"))
       ([] array addObject: ($$ "nicola"))   
       (gstep-bool ([] ([] "NSString" pathWithComponents: array)
		    isEqual: ($$ "home/nicola")))))))

(greg-testcase
 "[NSString pathWithComponents: (@\"/\", @\"home\", @\"nicola\")] == @\"/home/nicola\"" #t
 (lambda ()
   (let ((array ([] "NSMutableArray" new)))
     (begin 
       ([] array addObject: ($$ "/"))
       ([] array addObject: ($$ "home"))
       ([] array addObject: ($$ "nicola"))   
       (gstep-bool ([] ([] "NSString" pathWithComponents: array)
		    isEqual: ($$ "/home/nicola")))))))
		    
