;   test03.scm - Tests of string trimming
;   Copyright (C) 2001 Free SoftwareFoundation, Inc.
;
;   Written by:  Nicola Pero <nicola@brainstorm.co.uk>
;   More tests by:  Richard Frith-Macdonald <rfm@gnu.org>
;   Date: May 2001
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


(define ws ([] "NSCharacterSet" whitespaceAndNewlineCharacterSet))

(greg-testcase 
 "[@\"\" stringByTrimmingLeadSpaces] == (@\"\")" #t 
 (lambda () 
   (gstep-bool 
    ([] ([] ($$ "") stringByTrimmingLeadSpaces) isEqual: ($$ "")))))

(greg-testcase 
 "[@\"home\" stringByTrimmingLeadSpaces] == (@\"home\")" #t 
 (lambda () 
   (gstep-bool 
    ([] ([] ($$ "home") stringByTrimmingLeadSpaces) isEqual: ($$ "home")))))

(greg-testcase 
 "[@\" hello\" stringByTrimmingLeadSpaces] == (@\"hello\")" #t 
 (lambda () 
   (gstep-bool 
    ([] ([] ($$ " hello") stringByTrimmingLeadSpaces) isEqual: ($$ "hello")))))

(greg-testcase 
 "[@\"\thello\" stringByTrimmingLeadSpaces] == (@\"hello\")" #t 
 (lambda () 
   (gstep-bool 
    ([] ([] ($$ "\thello") stringByTrimmingLeadSpaces) isEqual: ($$ "hello")))))

(greg-testcase 
 "[@\"\nhello\" stringByTrimmingLeadSpaces] == (@\"hello\")" #t 
 (lambda () 
   (gstep-bool 
    ([] ([] ($$ "\nhello") stringByTrimmingLeadSpaces) isEqual: ($$ "hello")))))

(greg-testcase 
 "[@\"\" stringByTrimmingTailSpaces] == (@\"\")" #t 
 (lambda () 
   (gstep-bool 
    ([] ([] ($$ "") stringByTrimmingTailSpaces) isEqual: ($$ "")))))

(greg-testcase 
 "[@\"home\" stringByTrimmingTailSpaces] == (@\"home\")" #t 
 (lambda () 
   (gstep-bool 
    ([] ([] ($$ "home") stringByTrimmingTailSpaces) isEqual: ($$ "home")))))

(greg-testcase 
 "[@\"hello \" stringByTrimmingTailSpaces] == (@\"hello\")" #t 
 (lambda () 
   (gstep-bool 
    ([] ([] ($$ "hello ") stringByTrimmingTailSpaces) isEqual: ($$ "hello")))))

(greg-testcase 
 "[@\"hello\t\" stringByTrimmingTailSpaces] == (@\"hello\")" #t 
 (lambda () 
   (gstep-bool 
    ([] ([] ($$ "hello\t") stringByTrimmingTailSpaces) isEqual: ($$ "hello")))))

(greg-testcase 
 "[@\"hello\n\" stringByTrimmingTailSpaces] == (@\"hello\")" #t 
 (lambda () 
   (gstep-bool 
    ([] ([] ($$ "hello\n") stringByTrimmingTailSpaces) isEqual: ($$ "hello")))))


(greg-testcase 
 "[@\"\" stringByTrimmingSpaces] == (@\"\")" #t 
 (lambda () 
   (gstep-bool 
    ([] ([] ($$ "") stringByTrimmingSpaces) isEqual: ($$ "")))))

(greg-testcase 
 "[@\"home\" stringByTrimmingSpaces] == (@\"home\")" #t 
 (lambda () 
   (gstep-bool 
    ([] ([] ($$ "home") stringByTrimmingSpaces) isEqual: ($$ "home")))))

(greg-testcase 
 "[@\" hello\" stringByTrimmingSpaces] == (@\"hello\")" #t 
 (lambda () 
   (gstep-bool 
    ([] ([] ($$ " hello") stringByTrimmingSpaces) isEqual: ($$ "hello")))))

(greg-testcase 
 "[@\"\thello\" stringByTrimmingSpaces] == (@\"hello\")" #t 
 (lambda () 
   (gstep-bool 
    ([] ([] ($$ "\thello") stringByTrimmingSpaces) isEqual: ($$ "hello")))))

(greg-testcase 
 "[@\"\nhello\" stringByTrimmingSpaces] == (@\"hello\")" #t 
 (lambda () 
   (gstep-bool 
    ([] ([] ($$ "\nhello") stringByTrimmingSpaces) isEqual: ($$ "hello")))))

(greg-testcase 
 "[@\"\" stringByTrimmingSpaces] == (@\"\")" #t 
 (lambda () 
   (gstep-bool 
    ([] ([] ($$ "") stringByTrimmingSpaces) isEqual: ($$ "")))))

(greg-testcase 
 "[@\"home\" stringByTrimmingSpaces] == (@\"home\")" #t 
 (lambda () 
   (gstep-bool 
    ([] ([] ($$ "home") stringByTrimmingSpaces) isEqual: ($$ "home")))))

(greg-testcase 
 "[@\"hello \" stringByTrimmingSpaces] == (@\"hello\")" #t 
 (lambda () 
   (gstep-bool 
    ([] ([] ($$ "hello ") stringByTrimmingSpaces) isEqual: ($$ "hello")))))

(greg-testcase 
 "[@\"hello\t\" stringByTrimmingSpaces] == (@\"hello\")" #t 
 (lambda () 
   (gstep-bool 
    ([] ([] ($$ "hello\t") stringByTrimmingSpaces) isEqual: ($$ "hello")))))

(greg-testcase 
 "[@\"hello\n\" stringByTrimmingSpaces] == (@\"hello\")" #t 
 (lambda () 
   (gstep-bool 
    ([] ([] ($$ "hello\n") stringByTrimmingSpaces) isEqual: ($$ "hello")))))

(greg-testcase 
 "[@\"hello\" stringByTrimmingCharactersInSet: ws] == (@\"hello\")" #t 
 (lambda () 
   (gstep-bool 
    ([] ([] ($$ "hello") stringByTrimmingCharactersInSet: ws) isEqual: ($$ "hello")))))

(greg-testcase 
 "[@\"hello\n\" stringByTrimmingCharactersInSet: ws] == (@\"hello\")" #t 
 (lambda () 
   (gstep-bool 
    ([] ([] ($$ "hello\n") stringByTrimmingCharactersInSet: ws) isEqual: ($$ "hello")))))

(greg-testcase 
 "[@\"\nhello\" stringByTrimmingCharactersInSet: ws] == (@\"hello\")" #t 
 (lambda () 
   (gstep-bool 
    ([] ([] ($$ "\nhello") stringByTrimmingCharactersInSet: ws) isEqual: ($$ "hello")))))

(greg-testcase 
 "[@\"\nhello\n\" stringByTrimmingCharactersInSet: ws] == (@\"hello\")" #t 
 (lambda () 
   (gstep-bool 
    ([] ([] ($$ "\nhello\n") stringByTrimmingCharactersInSet: ws) isEqual: ($$ "hello")))))

(greg-testcase 
 "[@\"\n   \n\" stringByTrimmingCharactersInSet: ws] == (@\"\")" #t 
 (lambda () 
   (gstep-bool 
    ([] ([] ($$ "\n   \n") stringByTrimmingCharactersInSet: ws) isEqual: ($$ "")))))

(greg-testcase 
 "[@\"hello\" stringByPaddingToLength: 3 withString: @\".\" startingAtIndex: 0] ==(@\"hel\")" #t 
 (lambda () 
   (gstep-bool 
    ([] ([] ($$ "hello") stringByPaddingToLength: 3 withString: ($$ ".") startingAtIndex: 0) isEqual: ($$ "hel")))))

(greg-testcase 
 "[@\"hello\" stringByPaddingToLength: 8 withString: @\".\" startingAtIndex: 0] ==(@\"hello...\")" #t 
 (lambda () 
   (gstep-bool 
    ([] ([] ($$ "hello") stringByPaddingToLength: 8 withString: ($$ ".") startingAtIndex: 0) isEqual: ($$ "hello...")))))

(greg-testcase 
 "[@\"hello\" stringByPaddingToLength: 8 withString: @\"xy\" startingAtIndex: 1] ==(@\"helloyxy\")" #t 
 (lambda () 
   (gstep-bool 
    ([] ([] ($$ "hello") stringByPaddingToLength: 8 withString: ($$ "xy") startingAtIndex: 1) isEqual: ($$ "helloyxy")))))

