; Alexander Malmberg <alexander@malmberg.org>
;
; Test string encoding and decoding
; TODO: hard-coding the string encoding numbers isn't nice

(begin

  ;; For format
  (use-modules (ice-9 format))
  
  (define test-data-string
    (lambda (data encoding string test-decode test-encode)

  (let ((the-data ([] "NSData" dataWithBytes: (list->voidp data "C")
				      length: (length data)))
	(the-string ([] "NSString" stringWithCharacters: (list->voidp string "S")
						 length: (length string))))

    (if test-decode
      (greg-testcase (format #f "decode ~S ~S to ~S"
      				data
				([] "NSString" localizedNameOfStringEncoding: encoding)
				string)
		     #t
        (lambda ()
	  (let ((s ([] ([] "NSString" alloc)
	  	initWithData: the-data
	            encoding: encoding)))
	    (and
	      (not (eq? s gstep-nil))
	      (gstep-bool ([] s isEqual: the-string))
	      )
	    )
        )))

    (if test-encode
      (greg-testcase (format #f "encode ~S ~S to ~S"
      				the-string
				([] "NSString" localizedNameOfStringEncoding: encoding)
				the-data)
		     #t
        (lambda ()
	  (let ((d ([] the-string dataUsingEncoding: encoding)))
	    (and
	      (not (eq? d gstep-nil))
	      (gstep-bool ([] d isEqual: the-data))
	      )
	    )
	 )))
      #t
    )))


  (greg-testcase "[NSString -initWithData: encoding:] returns nil for unknown encodings" #t
    (lambda ()
      (let ((d ([] "NSData" dataWithBytes: (string->voidp "foo") length: 3)))
        (eq? gstep-nil ([] ([] "NSString" alloc) initWithData: d encoding: 0))
      )))

  (test-data-string
    '(65 66 67) 1 ; ascii
    '(65 66 67) #t #t)

  (test-data-string
    '(65 66 67) 4 ; utf-8
    '(65 66 67) #t #t)

  (test-data-string
    '(#xc3 #xa5 #xc3 #xa4 #xc3 #xb6
      #xd7 #xa9 #xd7 #x9c #xd7 #x95 #xd7 #x9d) 4 ; utf-8
    '(#xe5 #xe4 #xf6
      #x5e9 #x5dc #x5d5 #x5dd) #t #t)

  (test-data-string
    '(#xf9 #xec #xe5 #xed) 55 ; iso-8859-8
    '(#x5e9 #x5dc #x5d5 #x5dd) #t #t)

  (test-data-string
    '(116 101 115 116 45 229 228 246) 5 ; iso-8859-1
    '(116 101 115 116 45 229 228 246) #t #t)

  (test-data-string
    '(#xe0 #xb8 #xa0  #xe0 #xb8 #xb2  #xe0 #xb8 #xa9
      #xe0 #xb8 #xb2  #xe0 #xb9 #x84  #xe0 #xb8 #x97
      #xe0 #xb8 #xa2) 4 ; utf-8
    '(#xe20 #xe32 #xe29
      #xe32 #xe44 #xe17
      #xe22) #t #t)

;  (test-data-string
;    '(#xc0 #xd2 #xc9 #xd2 #xe4 #xb7 #xc2) 59 ; iso-8859-11, not yet implemented
;    '(#xe20 #xe32 #xe29
;      #xe32 #xe44 #xe17
;      #xe22) #t #t)

  (test-data-string
    '(#x41 #x42 #x43 #x20 #xa7 #x41 #xa6 #x6e #x21) 66 ; big5
    '(#x41 #x42 #x43 #x20 #x4f60 #x597d #x21) #t #t)

  )

