
;   use.scm - Tests of calling Objective-C methods from Guile
;   Copyright (C) 2002 Free SoftwareFoundation, Inc.
;
;   Written by: Richard Frith-Macdonald <rfm@gnu.org>
;   Date: Oct 2002
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

(begin

  (define current gstep-nil)
  (define localh ([] "NSTimeZone" defaultTimeZone))
  (define offset ([] localh secondsFromGMT))

  (greg-testcase "+timeZoneForSecondsFromGMT works" #t
    (lambda ()
      (set! current ([] "NSTimeZone" timeZoneForSecondsFromGMT: 900))
      (and
	(not (equal? current gstep-nil))
	(eq? ([] current secondsFromGMT) 900)
      )
    )
  )

  (greg-testcase "+timeZoneForSecondsFromGMT rounds to minute" #t
    (lambda ()
      (set! current ([] "NSTimeZone" timeZoneForSecondsFromGMT: -45))
      (and
	(not (equal? current gstep-nil))
	(eq? ([] current secondsFromGMT) -60)
      )
    )
  )

  (greg-testcase "+timeZoneForSecondsFromGMT has correct name" #t
    (lambda ()
      (set! current ([] "NSTimeZone" timeZoneForSecondsFromGMT: 7260))
      (and
	(not (equal? current gstep-nil))
	(gstep-bool ([] ([] current name) isEqual: ($$ "GMT+0201")))
      )
    )
  )

  (greg-testcase "+timeZoneForSecondsFromGMT has correct abbreviation" #t
    (lambda ()
      (set! current ([] "NSTimeZone" timeZoneForSecondsFromGMT: -3600))
      (and
	(not (equal? current gstep-nil))
	(gstep-bool ([] ([] current abbreviation) isEqual: ($$ "GMT-0100")))
      )
    )
  )

  (greg-testcase "+timeZoneForSecondsFromGMT has DST NO" #t
    (lambda ()
      (set! current ([] "NSTimeZone" timeZoneForSecondsFromGMT: -3600))
      (and
	(not (equal? current gstep-nil))
	(not (gstep-bool ([] current isDaylightSavingTime)))
      )
    )
  )

  (greg-testcase "+timeZoneForSecondsFromGMT has DST NO" #t
    (lambda ()
      (set! current ([] "NSTimeZone" timeZoneForSecondsFromGMT: -3600))
      (and
	(not (equal? current gstep-nil))
	(not (gstep-bool ([] current isDaylightSavingTime)))
      )
    )
  )

  (if (eq? offset 0)
    (set! offset 3600)
    (set! offset 0)
  )

  (greg-testcase "can set default time zone" #t
    (lambda ()
      (set! current ([] "NSTimeZone" timeZoneForSecondsFromGMT: offset))
      ([] "NSTimeZone" setDefaultTimeZone: current)
      (set! current ([] "NSTimeZone" localTimeZone))
      (and
	(not (equal? current gstep-nil))
	(eq? ([] current secondsFromGMT) offset)
	(not (gstep-bool ([] current isDaylightSavingTime)))
      )
    )
  )


)
