;   test01.scm - Tests for generating property lists
;   Copyright (C) 2001 Free SoftwareFoundation, Inc.
;
;   Written by:  Nicola Pero <nicola@brainstorm.co.uk>
;   Date: May 2001
;
;   This file is part of the GNUstep project.
;
;   This library is free software; you can redistribute it and/or
;   modify it under the terms of the GNU Library General Public
;   License as published by the Free Software Foundation; either
;   version 2 of the License, or (at your option) any later version.
;   
;   This library is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;   Library General Public License for more details.
;
;   You should have received a copy of the GNU Library General Public
;   License along with this library; if not, write to the Free
;   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

(begin

  (define test-parse-unparse 
    (lambda (object)
      (gstep-bool 
       ([] ([] ([] object description) propertyList) isEqual: object))))
  
  (greg-testcase 
   "We can generate a property list from a string" #t
   (lambda ()
     (test-parse-unparse ($$ "ariosto"))))

  (greg-testcase 
   "We can generate a property list from an empty array" #t
   (lambda ()
     (test-parse-unparse ([] "NSArray" array))))

  (greg-testcase 
   "We can generate a property list from an array with a single object" #t
   (lambda ()
     (test-parse-unparse ([] "NSArray" arrayWithObject: ($$ "Palinuro")))))

  (greg-testcase 
   "We can generate a property list from an array with two objects" #t
   (lambda ()
     (let ((object gstep-nil))
       (set! object ([] "NSArray" arrayWithObject: ($$ "Palinuro")))
       (set! object ([] object arrayByAddingObject: ($$ "Enea")))
       (test-parse-unparse object))))

  (greg-testcase 
   "We can generate a property list from an array with three objects and \"s" 
   #t
   (lambda ()
     (let ((object gstep-nil))
       (set! object ([] "NSArray" arrayWithObject: ($$ "Palinuro")))
       (set! object ([] object arrayByAddingObject: ($$ "Enea")))
       (set! object ([] object arrayByAddingObject: ($$ "Eurialo e Niso")))
       (test-parse-unparse object))))

  (greg-testcase 
   "We can generate a property list from an empty dictionary" #t
   (lambda ()
     (let ((object gstep-nil))
       (set! object ([] "NSDictionary" dictionary))
       (test-parse-unparse object))))

  (greg-testcase 
   "We can generate a property list from a dictionary with a single key/value pair" #t
   (lambda ()
     (let ((object gstep-nil))
       (set! object ([] "NSDictionary" dictionaryWithObject: ($$ "Virgilio")
		     forKey: ($$ "Autore")))
       (test-parse-unparse object))))

  (greg-testcase 
   "We can generate a property list from a dictionary with two key/value pairs" #t
   (lambda ()
     (let ((object gstep-nil))
       (set! object ([] "NSMutableDictionary" dictionary))
       ([] object setObject: ($$ "Virgilio") forKey: ($$ "Autore"))
       ([] object setObject: ($$ "Eneide") forKey: ($$ "Titolo"))
       (test-parse-unparse object))))

  (greg-testcase 
   "We can generate a property list from a dictionary with an array inside" #t
   (lambda ()
     (let ((object gstep-nil))
       (set! object ([] "NSMutableDictionary" dictionary))
       ([] object setObject: ($$ "Virgilio") forKey: ($$ "Autore"))
       ([] object setObject: ([] NSArray arrayWithObject: ($$ "Molte"))
	forKey: ($$ "Opere"))
       (test-parse-unparse object))))

  (greg-testcase 
   "We can generate a medium-size property list (1)" #t
   (lambda ()
     (let ((object gstep-nil) 
	   (objectA gstep-nil) 
	   (objectB gstep-nil) 
	   (objectC gstep-nil) 
	   (objectD gstep-nil))
       (set! object ([] "NSMutableDictionary" dictionary))
       (set! objectA ([] NSArray arrayWithObject: ($$ "Ciao,")))
       (set! objectB ([] NSArray arrayWithObject: objectA))
       (set! objectC ([] NSDictionary dictionary))
       (set! objectD ([] NSArray arrayWithObject: 
		      ([] NSArray arrayWithObject:
		       ([] NSDictionary dictionaryWithObject:
			([] NSArray arrayWithObject: ($$ "Ciao,"))
			forKey: ([] NSArray arrayWithObject: ($$ ", Ciao"))))))
       ([] object setObject: ($$ "Utinam") forKey: objectA)
       ([] object setObject: objectC forKey: objectB)
       ([] object setObject: ($$ "Auri;") forKey: objectD)
       (test-parse-unparse object))))

  (greg-testcase 
   "We can generate a medium-size property list (2)" #t
   (lambda ()
     (let ((object gstep-nil) 
	   (objectA gstep-nil) 
	   (objectA-A gstep-nil) 
	   (objectA-B gstep-nil) 
	   (objectB gstep-nil) 
	   (objectB-A gstep-nil) 
	   (objectB-A-A gstep-nil) 
	   (objectB-A-B gstep-nil) 
	   (objectB-B gstep-nil))

       ;; objectA
       (set! objectA-A ([] "NSMutableDictionary" dictionary))
       ([] objectA-A setObject: ($$ "1 2 3 4 5 6 7 8 9 0") 
	forKey: ($$ "numeri"))
       ([] objectA-A setObject: ($$ "A B C D E F G H I J")
	forKey: ($$ "lettere"))
       
       (set! objectA-B ([] "NSMutableDictionary" dictionary))
       ([] objectA-B setObject: ($$ "3.1415296") forKey: ($$ "PI greco"))
       ([] objectA-B setObject: ($$ "0") forKey: ($$ "zero"))
       ([] objectA-B setObject: ($$ "1") forKey: ($$ "uno"))
       
       (set! objectA ([] "NSMutableDictionary" dictionary))
       ([] objectA setObject: objectA-A  forKey: ($$ "Informazioni Utili"))
       ([] objectA setObject: objectA-B  forKey: ($$ "Costanti Numeriche"))

       ;; objectB
       (set! objectB-A ([] "NSMutableDictionary" dictionary))

       (set! objectB-A-A ([] "NSMutableArray" array))
       ([] objectB-A-A addObject: ($$ "1"))
       ([] objectB-A-A addObject: ($$ "2"))
       ([] objectB-A-A addObject: ($$ "3"))
       ([] objectB-A-A addObject: ($$ "4"))
       ([] objectB-A-A addObject: ($$ "5"))
       ([] objectB-A-A addObject: ($$ "6"))
       ([] objectB-A-A addObject: ($$ "7"))
       ([] objectB-A-A addObject: ($$ "8"))
       ([] objectB-A-A addObject: ($$ "9"))
       ([] objectB-A-A addObject: ($$ "0"))
       ([] objectB-A setObject: objectB-A-A forKey: ($$ "numeri"))
       
       (set! objectB-A-B ([] "NSMutableArray" array))
       ([] objectB-A-B addObject: ($$ "A"))
       ([] objectB-A-B addObject: ($$ "B"))
       ([] objectB-A-B addObject: ($$ "C"))
       ([] objectB-A-B addObject: ($$ "D"))
       ([] objectB-A-B addObject: ($$ "E"))
       ([] objectB-A-B addObject: ($$ "F"))
       ([] objectB-A-B addObject: ($$ "G"))
       ([] objectB-A-B addObject: ($$ "H"))
       ([] objectB-A-B addObject: ($$ "I"))
       ([] objectB-A-B addObject: ($$ "J"))
       ([] objectB-A setObject: objectB-A-B forKey: ($$ "letterine"))
       
       (set! objectB-B ([] "NSMutableDictionary" dictionary))
       ([] objectB-B setObject: ($$ "3.1415296") forKey: ($$ "PI greca"))
       ([] objectB-B setObject: ($$ "0") forKey: ($$ "el zero"))
       ([] objectB-B setObject: ($$ "1") forKey: ($$ "el uno"))
       
       (set! objectB ([] "NSMutableDictionary" dictionary))
       ([] objectB setObject: objectB-A  forKey: ($$ "Informazioni Utili"))
       ([] objectB setObject: objectB-B  forKey: ($$ "Costanti Numeriche"))

       ;; object
       (set! object ([] "NSMutableDictionary" dictionary))
       ([] object setObject: objectA forKey: ($$ "Un dizionario"))
       ([] object setObject: objectB forKey: ($$ "Un altro dizionario"))
       
       (test-parse-unparse object))))

  (greg-testcase 
   "We can generate a property list from an empty data" #t
   (lambda ()
     (let ((object gstep-nil))
       (set! object ([] "NSData" data))
       (test-parse-unparse object))))

  (greg-testcase 
   "We can generate a property list from very simple data" #t
   (lambda ()
     (let ((object gstep-nil))
       (set! object ([] ($$ "Questo e` un test") dataUsingEncoding: 1))
       (test-parse-unparse object))))

  (greg-testcase 
   "We can generate a property list from very simple data (2)" #t
   (lambda ()
     (let ((object gstep-nil))
       (set! object ([] ([] ([] "NSProcessInfo" processInfo) 
			 globallyUniqueString) dataUsingEncoding: 7))
       (test-parse-unparse object))))
  
  (greg-testcase 
   "We can generate a property list from an array containing very simple data" 
   #t
   (lambda ()
     (let ((object gstep-nil))
       (set! object ([] "NSMutableArray" array))
       ([] object addObject: ([] ($$ "*()3\"#@Q``''") dataUsingEncoding: 1))
       (test-parse-unparse object))))

  (greg-testcase 
   "We can generate a property list from an array containing various things" 
   #t
   (lambda ()
     (let ((object gstep-nil))
       (set! object ([] "NSMutableArray" array))
       ([] object addObject: ([] ($$ "*()3\"#@Q``''") dataUsingEncoding: 1))
       ([] object addObject: ($$ "nicola \" , ; <"))
       ([] object addObject: ($$ "<nicola"))
       ([] object addObject: ($$ "nicola;"))
       ([] object addObject: ($$ "nicola,"))
       ([] object addObject: ($$ "nicola>"))
       ([] object addObject: ($$ "nicola@"))
       ([] object addObject: ($$ "nicola "))
       ([] object addObject: ($$ "nicola="))
       ([] object addObject: ([] "NSArray" arrayWithObject: ($$ "=")))
       ([] object addObject: ([] "NSDictionary" dictionary))
       (test-parse-unparse object))))


   )





