/*
   IvarProxy test library - for usage from guile
 
   Copyright (C) 2002 Free Software Foundation, Inc.
 
   Written by: Nicola Pero <nicola@brainstorm.co.uk>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
 
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
   Library General Public License for more details.
 
   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02111, USA. 
*/
 
/* 

   We need to test that certain ivars are released (or not) when an
   object of a certain class is deallocated.  This is difficult to do
   from guile, because of the retain count problems.  It is easy to do
   it in Objective-C.  So this class provides an Objective-C
   implementation of the stuff difficult to do from guile.

   We need to make sure we never expose the object we are testing to
   guile, so that it's not affected by guile garbage collecting.

*/

#include <Foundation/Foundation.h>

@interface IvarProxy : NSObject
{
  id theObject;
}

/* Creates an internal object of class className (hidden to guile)  */
- (id) initWithClass: (NSString *)className;

/* Deallocates (immediately) the internal object.  */
- (void) deallocTheObject;

/* Set an ivar of the internal object.  */
- (void) setObjectIvar: (id)ivar  usingSelector: (SEL)selector;

@end

@implementation IvarProxy

- (id) initWithClass: (NSString *)className
{
  theObject = [[NSClassFromString (className) alloc] init];
  return [super init];
}

- (void) dealloc
{
  TEST_RELEASE (theObject);
  [super dealloc];
}

- (void) deallocTheObject
{
  RELEASE (theObject);
  theObject = nil;
}

- (void) setObjectIvar: (id)ivar
	 usingSelector: (SEL)selector
{
  [theObject performSelector: selector  withObject: ivar];
}
@end
