; Test that the item list is maintained properly
;
; Author: Alexander Malmberg <alexander@malmberg.org>
; Date: 2003-04-12

(begin
  (define check-number
    (lambda (cell num)
      (and
	(= ([] cell numberOfItems) num)
	(= ([] ([] cell itemArray) count) num)
	(= ([] ([] cell itemTitles) count) num)
	)
      )
    )

  (define check-items
    (lambda (cell list)
      (define check-r
	(lambda (cell idx list)
	  (if (eq? list '())
	    (= idx ([] cell numberOfItems))
	    (and
	      (< idx ([] cell numberOfItems))
	      (gstep-bool ([] ([] cell itemTitleAtIndex: idx) isEqual: ($$ (car list))))
	      (check-r cell (+ idx 1) (cdr list))
	      )
	    )
	  )
	)
      (and
	(check-r cell 0 list)
	(check-number cell (length list))
	)
      )
    )

  (define cell ([] "NSPopUpButtonCell" new))

  (greg-testcase "Starts out empty" #t
    (lambda ()
      (check-number cell 0)
      )
    )

  (greg-testcase "Items can be added" #t
    (lambda ()
      ([] cell addItemWithTitle: ($$ "foo"))
      ([] cell addItemWithTitle: ($$ "zot"))
      (check-items cell `("foo" "zot"))
      )
    )

  (greg-testcase "Items can be inserted" #t
    (lambda ()
      ([] cell insertItemWithTitle: ($$ "bar") atIndex: 1)
      (check-items cell `("foo" "bar" "zot"))
      )
    )

  (greg-testcase "Items can be removed by title" #t
    (lambda ()
      ([] cell removeItemWithTitle: ($$ "foo"))
      (check-items cell `("bar" "zot"))
      )
    )

  (greg-testcase "Items can be removed by index" #t
    (lambda ()
      ([] cell removeItemAtIndex: 0)
      ([] cell removeItemAtIndex: 0)
      (check-number cell 0)
      )
    )

  (greg-testcase "Items can be added again" #t
    (lambda ()
      ([] cell addItemWithTitle: ($$ "foo"))
      ([] cell addItemWithTitle: ($$ "zot"))
      (check-items cell `("foo" "zot"))
      )
    )

  (greg-testcase "removeAllItems removes all items" #t
    (lambda ()
      ([] cell removeAllItems)
      (check-number cell 0)
      )
    )

  (greg-testcase "Cell is really empty" #t
    (lambda ()
      (and
	(= ([] cell indexOfItemWithTitle: ($$ "")) -1)
	(= ([] cell indexOfItemWithTag: 0) -1)
	(= ([] cell indexOfItemWithRepresentedObject: gstep-nil) -1)
	)
      )
    )

  )

