; Basic tests of encoding and decoding of rtf
;
; Works by creating a bunch of attributed strings and making sure that
; encoding and decoding them result in the same strings.
;
; Author: Alexander Malmberg <alexander@malmberg.org>
; Date: February 2003

(begin

  (define test-attributed-string
    (lambda (string)
      (let*
	((encoded ([] string RTFFromRange: (NSMakeRange 0 ([] ([] string string) length))
	    documentAttributes: gstep-nil))
	 (decoded ([] ([] NSAttributedString alloc) initWithRTF: encoded
	    documentAttributes: gstep-nil))

;	 (is-ps ([] string attribute: ($$ "NSParagraphStyleAttributeName")
;	    atIndex: 0
;	    effectiveRange: gstep-nil))
;	 (got-ps ([] decoded attribute: ($$ "NSParagraphStyleAttributeName")
;	    atIndex: 0
;	    effectiveRange: gstep-nil))
	)

;	(format #t "\nis: |~S|\ngot |~S|\n" string decoded)
;	(format #t "~S\n" ([] ([] NSString alloc) initWithData: encoded encoding: 1))
        (gstep-bool ([] string isEqual: decoded))
	)
      )
    )

  (define attr-basic ([] ([] "NSMutableDictionary" alloc) init))
  ([] attr-basic setObject: ([] NSFont userFontOfSize: 0.0)
    forKey: ($$ "NSFontAttributeName"))
  ([] attr-basic setObject: ([] NSParagraphStyle defaultParagraphStyle)
    forKey: ($$ "NSParagraphStyleAttributeName"))


  (define test-simple-string
    (lambda (string)
      (greg-testcase (format #f "Simple string: ~S" string) #t
	(lambda ()
	  (let ((as ([] ([] "NSMutableAttributedString" alloc) initWithString: string
		attributes: attr-basic)))

	    (test-attributed-string as)
            )
	  )
	)
      )
    )

  ; Sanity.
  (test-simple-string ($$ "foo"))

  ; Some random test strings to make sure whitespace is preserved.
  (test-simple-string ($$ " foo"))
  (test-simple-string ($$ "foo "))
  (test-simple-string ($$ "  foo  "))
  (test-simple-string ($$ "foo\nbar"))
  (test-simple-string ($$ "\nfoo \n bar\n"))
  (test-simple-string ($$ "\n\n"))
  (test-simple-string ($$ "  "))

  ; Some non-ascii characters.
  (test-simple-string
    ([] "NSString" stringWithCharacters:
        (list->voidp '(#xe5 #xe4 #xf6 #x5e9 #x5dc #x5d5 #x5dd) "S")
      length: 7))

  ; Basic color tests.
  (let ((as ([] ([] "NSMutableAttributedString" alloc) initWithString: ($$ "red blue green")
        attributes: attr-basic)))

      ; Add the colors one by one and test after each change.
      ([] as addAttribute: ($$ "NSForegroundColorAttributeName")
        value: ([] NSColor redColor)
	range: (NSMakeRange 0 3))
      (greg-testcase (format #f "Foreground colors 1" string) #t
	(lambda () (test-attributed-string as)))

      ([] as addAttribute: ($$ "NSForegroundColorAttributeName")
	value: ([] NSColor blueColor)
	range: (NSMakeRange 4 4))
      (greg-testcase (format #f "Foreground colors 2" string) #t
	(lambda () (test-attributed-string as)))

      ([] as addAttribute: ($$ "NSForegroundColorAttributeName")
	value: ([] NSColor greenColor)
	range: (NSMakeRange 9 5))
      (greg-testcase (format #f "Foreground colors 3" string) #t
	(lambda () (test-attributed-string as)))


      ([] as addAttribute: ($$ "NSBackgroundColorAttributeName")
        value: ([] NSColor greenColor)
	range: (NSMakeRange 0 8))
      (greg-testcase (format #f "Background colors 1" string) #t
	(lambda () (test-attributed-string as)))

      ([] as addAttribute: ($$ "NSBackgroundColorAttributeName")
        value: ([] NSColor redColor)
	range: (NSMakeRange 8 5))
      (greg-testcase (format #f "Background colors 2" string) #t
	(lambda () (test-attributed-string as)))
    )

  ; Basic tests of paragraph styles.
  (define test-paragraph-style
    (lambda (paragraph-style description)
      (greg-testcase (format #f "Paragraph style: ~S" description) #t
	(lambda ()
	  (let ((as ([] ([] "NSMutableAttributedString" alloc)
		initWithString: ($$ "para1\npara2\npara3\n")
		attributes: attr-basic)))

	    ([] as addAttribute: ($$ "NSParagraphStyleAttributeName")
	      value: paragraph-style
	      range: (NSMakeRange 6 6))
	    (test-attributed-string as)
            )
	  )
	)
      )
    )
  (let ((ps ([] NSParagraphStyle defaultParagraphStyle)))
      (test-paragraph-style ps "default")

      (let ((mps ([] ps mutableCopy)))
	([] mps setLineSpacing: 5.0)
	  (test-paragraph-style mps "line spacing"))

      (let ((mps ([] ps mutableCopy)))
	([] mps setParagraphSpacing: 5.0)
	  (test-paragraph-style mps "paragraph spacing"))

      (let ((mps ([] ps mutableCopy)))
	([] mps setHeadIndent: 5.0)
	  (test-paragraph-style mps "head indent"))

      (let ((mps ([] ps mutableCopy)))
	([] mps setTailIndent: -5.0)
	  (test-paragraph-style mps "tail indent <0"))

      (let ((mps ([] ps mutableCopy)))
	([] mps setTailIndent: 50.0)
	  (test-paragraph-style mps "tail indent >0"))

      (let ((mps ([] ps mutableCopy)))
	([] mps setFirstLineHeadIndent: 5.0)
	  (test-paragraph-style mps "first line head indent"))

      (let ((mps ([] ps mutableCopy)))
	([] mps setMinimumLineHeight: 20.0)
	  (test-paragraph-style mps "minimum line height"))

      (let ((mps ([] ps mutableCopy)))
	([] mps setMaximumLineHeight: 20.0)
	  (test-paragraph-style mps "maximum line height"))

      (let ((mps ([] ps mutableCopy)))
	([] mps setAlignment: 2) ; center alignment
	  (test-paragraph-style mps "alignment"))

      (let ((mps ([] ps mutableCopy)))
	([] mps setLineBreakMode: 1) ; charecter wrapping
	  (test-paragraph-style mps "line break mode"))
    )
  )

